////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omConnector_
#define _omConnector_

#include <openmaia/commons.h>
#include <wx/wx.h>
#include <wx/socket.h>
//#include <wx/config.h>

#include <stdio.h>

#include <openmaia/omObject.h>
#include <openmaia/omTCPMessage.h>
#include <openmaia/omTCPParser.h>
#include <openmaia/omFunctions.h>

/** @ingroup omgen
    TCP/IP Communication object.

    This object is used to connect the OpenMAIA Application with the rest of
    the OpenMAIA system.<br>
    Every Application (agent) uses an instance of omConnector object to receive
    and send message to the other agents of the system.<br>
    This object provides basic interfaces for receiving and sending messages.<br>
    Messages are sent using SendMessage method of omConnector class.<br>
    Messages are received via NotifyMessage event generated from omConnector to
    an omObject or derived class (see image) <br><br>
    <img src="images/omConnector.png" usemap="#omConnector_map">
*/

class omConnector: public omObject, public wxEvtHandler
{
 private:
  wxSocketClient *sock ;         ///< Socket information.
  omObject *toNotify;          ///< Object to notify.
  wxString hostname;             ///< Hostname to connect.
  unsigned short service;        ///< Port to connect.
  bool connected;                ///< True if connection established.
  wxString currentIP;            ///< Current IP, setted after connection
  wxString currentHost;          ///< Store current host name after connection
  int currentPort;		 ///< Store current port number after connection 
  bool notifyMessages;           ///< flag
  
  omTCPParser tcpParser;

  //  bool endOfLine;
  wxString incomingMessage;   ///< Partial message.

  

  DECLARE_EVENT_TABLE()

 public:
  omConnector();
  omConnector(wxString, unsigned short) ;
  omConnector(omObject *);
  omConnector(omObject *, wxString, unsigned short);

  bool Connect() ;             // make a connection with openmaia server
  void SendMessage(omTCPMessage ) ;   // send a message to the server
  void SendClick();          // send the click event to the server
  
  void OnSocketEvent(wxSocketEvent &) ;   
  void OnSocketConnection(wxSocketEvent &);
  
};

#endif  // _omConnector_
