////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omDialog_
#define _omDialog_

#include <openmaia/commons.h>
#include <wx/wx.h>
#include <wx/xrc/xmlres.h>

#include <openmaia/omObject.h>


#define omOK_BUTTON 0
#define omCANCEL_BUTTON 1
#define omYES_BUTTON 2
#define omNO_BUTTON 3
#define omSAVE_BUTTON 4




/** @ingroup omgen
    OpenMAIA Dialog class.

    This class provides an interface to wxDialog class to use with 
    XRC resources file.
    The resource file containing the description of the dialog must be loaded before
    building omDialog class.
    The name of the dialog to load must be passed by the constructor of omDialog.<br>
    omDialog class handles buttons called:
    <ul>
      <li>buttonOk -- omOK_BUTTON</li>
      <li>buttonCancel -- omCANCEL_BUTTON</li>
      <li>buttonYes -- omYES_BUTTON</li>
      <li>buttonNo -- omNO_BUTTON</li>
      <li>buttonSave -- omSAVE_BUTTON</li>
    </ul>

    When a button is pressed an internal flag stores the name of the button that was pressed.
    This name can be queried using GetButtonPressed() method of the class.

    The following example shows how to use omDialog class.
    myresource.xrc contains the description of a dialog called dialog1 with yes and no buttons.
    <pre>

      ...

      wxXmlResource::Get()->InitAllHandlers();  
      wxXmlResource::Get()->Load("myresource.xrc") ;

      omDialog *myDialog = new omDialog(this, "Dialog title", "dialog1");
      myDialog->ShowModal();

      if (myDialog->ButtonPressed())
       {
         int button = myDialog->GetButtonPressed();
	 switch(button)
	  {
	    case: omYES_BUTTON:
	      // ... something
	    break;

	    case: omNO_BUTTON:
	      // ... something
	    break;
	  }
       }

       ...

    </pre>
*/


class omDialog: public wxDialog, public omObject
{
 public:
  omDialog(wxWindow *, wxString, wxString) ;
  int ShowModal();
  bool IsButtonPressed();
  int GetButtonPressed();
  bool Destroy();


  DECLARE_EVENT_TABLE()


 private:

  // variables
  bool isPressed;
  int buttonPressed;

  //void OnCloseDialog(wxCloseEvent& WXUNUSED(event));  
  void OnYesButton(wxCommandEvent& WXUNUSED(event));
  void OnNoButton(wxCommandEvent& WXUNUSED(event));
  void OnCancelButton(wxCommandEvent& WXUNUSED(event));
  void OnOkButton(wxCommandEvent& WXUNUSED(event));
  void OnSaveButton(wxCommandEvent& WXUNUSED(event));

  
} ;


#endif  // _omDialog_
