////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#ifndef _MaiaExecuteAction_
#define _MaiaExecuteAction_

#include <openmaia/commons.h>
#include <wx/wx.h>

#include <openmaia/omcommon.h>

#include <openmaia/omObject.h>
#include <openmaia/omConnector.h>
#include <openmaia/omFunctions.h>

#include <omemukeys.h>

#include <iostream>
#include <algorithm>
#include <cctype>
#include <sstream>




// constants

#define MOUSE_TIMER_ID 1
#define MOUSE_MOVEMENT_TIMER 100





/** @ingroup omgen
    This class provides a set of methods used to emulate keyboard and mouse.
    The class MUST be instantied before the use because the constructor initializes
    some internal variables.
*/

class omExecutor: public omObject, public wxEvtHandler
{
 public:
  // general methods
  omExecutor() ;
  void Stop() ;    // stop timers
  void Start() ;   // start timers if needed

  // emulate pression of char keys
  void RenderString( wxString) ;
  void PressKey( long) ;    // emulate special key press
  void ReleaseKey( long) ;  // emulate special key release
  // emulate mouse movements in x,y directions
  bool MoveMouse( wxString, wxString ) ;     // parameter passed by wxString "dx,dy"
  bool MoveMouse( int, int ) ;   // the right function
  void ClickMouse( int ) ;   // click a button of a mouse
  void DragOrDropMouse( int ) ;   // drag and then drop the mouse button
  long ExecuteApplication(wxString command);

  
 protected:
  // variables
  void *sdData ;   ///< System dependent data.
  long maiaKeyCode[MAXMAIACODES] ;  ///< Array of keyCodes used for keyboard emulation.

  // variables used in mouse simulations
  int dxMouseMovement ;        ///< Mouse movement X increment.
  int dyMouseMovement ;        ///< Mouse movement Y increment.
  bool isMouseInMovement ;     ///< True if the mouse timer is active.
  wxTimer mouseTimer ;         ///< Mouse move timer.
  bool isMouseButtonPressed[MAXMOUSEBUTTONS] ;  ///< Mouse button flags. start index from 1 :).
  
  // private functions
  void OnMouseTimer(wxTimerEvent &) ;   ///< Mouse Timer event handler.

  // system dependent functions
  void SDConstructor() ;      ///< System dependen constructor.
  void SDRenderChar(char) ;   ///< System dependent RenderChar function.
  void SDPressKey(long) ;  ///< System dependent (keyboard) PressKey function.
  void SDReleaseKey(long) ; ///< System dependent (keyboard) ReleaseKey function.
  void SDMoveMouse() ;    ///< System dependent MoveMouse function.
  void SDPressMouseButton( int but) ;  ///< System dependent PressMouseButton function.
                                    ///<     @param but Button number.
  void SDReleaseMouseButton( int but) ; ///< System dependent ReleaseMouseButton function.
                                     ///<     @param but Button number.

  // Use wx event table to catch timer events
  DECLARE_EVENT_TABLE()

} ;


#endif // _MaiaExecuteAction_
