// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omFunctions_
#define _omFunctions_

//#ifdef __WXMAC__
//  #include <sys/types.h>
//  #include <sys/socket.h>
//  #include <netinet/in.h>
//  #include <arpa/inet.h>
//  #include <netdb.h>
//#else  
//  #include <arpa/inet.h>
//  #include <netdb.h>
//  #include <unistd.h>
//#endif

#include <openmaia/commons.h>
#include <wx/wx.h>
#include <wx/dir.h>
#include <wx/file.h>
#include <iostream>

#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <pwd.h>

#include <openmaia/config.h>
#include <openmaia/omXMLTags.h>
#include <openmaia/omemukeys.h>

//#ifdef __WXMAC__
//  #include <sys/types.h>
//  #include <sys/socket.h>
//  #include <netinet/in.h>
//  #include <arpa/inet.h>
//  #include <netdb.h>
//#else  
//  #include <arpa/inet.h>
//  #include <netdb.h>
//  #include <unistd.h>
//#endif



/** @ingroup omgen
    This class contains some static functions used by the maia api system.<br><br>
    Functions are called with omFunctions::function_name(...).
*/

class omFunctions {
 public:
  static void InitializeSimulationChars( long *) ;  ///< This is a system dependent function

  // misc utilities
  static wxString GetUnixString(wxString) ;
  static wxString GetWindowsString(wxString) ;
  static wxString ConvertToLower(wxString) ;
  static wxString ConvertToUpper(wxString) ;
  static int ArrayFind(wxArrayString *, wxString);
  static int ArrayFindNoCase(wxArrayString *, wxString);

  // path handling
  static void SetBasePath(wxString);
  static bool TestPersonalInstallation(wxString, wxString, wxString) ;
  static bool CreateDefaultInstallation(wxString, wxString);
  static bool TestFolder(wxString);
  static void CreateFolder(wxString, wxString);
  static bool TestFile(wxString) ;
  static wxString GetCurrentPath();
  static wxString GetSharedPath(wxString);
  static wxString GetInstallationPath(wxString);
  static wxString GetUserDataPath(wxString);
  static void CopyFile(wxString,wxString, wxString);

  // message handling
  static wxString GetCurrentIP() ;
  static wxString Encode(wxString);
  static wxString Decode(wxString);
  static int FindFirstNBChar(wxString, int t);
  static int FindLastNBChar(wxString);
  static int FindNotEncodedChar(wxString, char, int);

  // conversion utilities
  static int Literal2Value(wxString);
  static long GetMaiaKeycodeFromXML( wxString) ;
  static wxArrayString SplitTextLine(wxString, wxString);
  static wxString Value2Literal(int);
  static bool String2Boolean(wxString);
  static wxString Boolean2String(bool);
  static wxString Boolean2String(int);
  static int String2Int(wxString);
  
  // check utilities
  static bool IsBoolean(wxString);
  static bool IsInt(wxString);


 private:
  static void SetBasePathSD(wxString);    
                             ///< Set base path in function of operating system
  static bool CreateDefaultInstallationSD(wxString,wxString);   
                            ///< Create installation sysyem dependent
  static bool TestPersonalInstallationSD(wxString, wxString, wxString);   
                            ///< GetPersonalInstallation system dependent part.
  static wxString GetSharedPathSD(wxString);
                            ///< GetSharedPath system dependent part.
  static wxString GetInstallationPathSD(wxString);
                            ///<  GetInstallationPath system dependent part.
  static wxString GetUserDataPathSD(wxString);
                            ///< GetUserDataPath system dependent part.
  static void CopyFileSD(wxString,wxString, wxString);
                            ///< CopyFile system dependent part.
  static void CreateFolderSD(wxString, wxString);
                            ///< CreateFolder system dependent part.
} ;

#endif // _omFunctions_
