////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////
#ifndef _omObject_
#define _omObject_



#include <openmaia/commons.h>
#include <iostream>
#include <string>
#include <wx/wx.h>

// include configuration file used in every class
#include <openmaia/config.h>

#include <openmaia/omTCPMessage.h>

/** @ingroup omgen
    Superclass for event Handling.

    This class is used as a superclass for every object in the OpenMAIA suite. <br>
    All the objects retrive event passed by the functions provided by the omObject class. <br>
    Every virtual method in the omObject send an event to the child class. <br>
    The child class must override the interested method to handle the event generated. <br>

 */

class omObject {
 public:
  // generic events
  virtual void Init() ;        // initialize class
  virtual void Reload();       // reload values

  // generic start and stop 
  virtual void Start() ;       // pass the Start event
  virtual void Stop() ;        // pass the Stop event

  // navigation events
  virtual void Tick() ;        
  virtual void Action() ;      
  virtual void GoDown() ;      
  virtual void GoUp();
  virtual void GoLeft() ;
  virtual void GoRight() ;

  // notify data eventes
  virtual void NotifyMessage(wxString, wxArrayString);
  virtual void NotifyXYData(int, int) ;
  virtual void NotifyIntData(int) ;
  virtual void NotifyFloatData(float) ;
  virtual void NotifyTCPMessage(omTCPMessage);

  // used by GrantFocus2User
  virtual bool IsYourWindow(void *); // pass the IsYourWindow check

  // used by some device drivers
  virtual void StopExecutor();      // stop the command executor
  virtual void StartExecutor() ;    // start the command executor

  // low level direction button events
  virtual void ActionButtonDown();
  virtual void ActionButtonUp();
  virtual void LeftButtonDown();
  virtual void LeftButtonUp();
  virtual void RightButtonDown();
  virtual void RightButtonUp();
  virtual void UpButtonDown();
  virtual void UpButtonUp();
  virtual void DownButtonDown();
  virtual void DownButtonUp();
} ;

#endif
