////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omSerialDriver_
#define _omSerialDriver_

#include <openmaia/commons.h>
#include <openmaia/omObject.h>

#include <wx/wx.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/termios.h>
#include <fcntl.h>

#define OMSERIALBUTTONNUMBER 4

/** @ingroup omdrivers
    OpenMAIA Serial Port Driver.

    This class is a driver for the OpenMAIA Serial Port interface.
    OpenMAIA Serial Port interface is a hardware interface that 
    enables the connection of up to 4 external buttons via the serial port 
    RS-232.<br>
    Buttons are numerated from 0 to 3.<br><br>

    This class generates events to the object specified in the constructor.<br>
    Using the AssignActionEvent, AssignUpEvent, AssignDownEvent, AssignRightEvent and
    AssignLeftEvent methods you can specify which event should be generated in 
    corrispodence to each button click.<br>
    By default every button generates an Action Event.<br>
    Event generation may be disabled using DisableButtonEvent() method.
*/

class omSerialDriver: public omObject, wxTimer
{
 private:

  long actionKey, upKey, downKey, leftKey, rightKey;
  bool actionOnPress;
  omObject *toInform ;             ///< Object to send messages.
  int serialFd;  ///< Serial port file descriptor.
  wxString serialPort;

  void (omObject::*fptr[OMSERIALBUTTONNUMBER])() ;  ///< function pointer per button
  bool enableButton[OMSERIALBUTTONNUMBER] ;   ///< enable flags

  bool errorStatus;   ///< flag.

  // port channel flags
  bool RIActivated; ///< Flag.
  bool CTSActivated; ///< Flag.
  bool DSRActivated; ///< Flag.
  bool CDActivated; ///< Flag.


  int tickTime ;  ///< Pooling interval (ms).

  void Notify() ;   ///< Timer notify.
  void OpenSerialPort();

 public:
  omSerialDriver(omObject *, int, wxString, bool) ;

  void Start() ;   // start the event catching
  void Stop() ;   // stop the event catching

  void SetOnPress(bool) ;
  bool GetError() ;

  void AssignActionEvent(int);
  void AssignUpEvent(int) ;
  void AssignDownEvent(int);
  void AssignLeftEvent(int);
  void AssignRightEvent(int);
  void DisableButtonEvent(int) ;

};


#endif // _omKeyboardDriver_
