////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _OMSMSMANAGER_H_
#define _OMSMSMANAGER_H_

#include <openmaia/commons.h>
#include <openmaia/omSms.h>
#include <wx/wx.h>

#include <iostream>
#include <fcntl.h>
#include <termios.h>
#include <signal.h>
#include <unistd.h>


/* ----------------- constants used for flow control ------ */
/* modem timeout in milliseconds */
#define SLEEPTIME 5

/* reading tries before error */
#define MAX_RETRY_NUMBER 1500

/* ------------------ constants used to manage messages -----*/
#define MAX_READMESS_BUFF_LENGHT 10000


/** @ingroup omgen

    This class is used to interface computer with a cellular phone via serial port or
    infra red connection.
    The class can be used to send or retrive SMS messages using a cellular phone.

  */

class omSmsManager
{
public:

  omSmsManager();
  virtual ~omSmsManager();

  wxString AutoDetect();  //Search the device checking /dev/ttyS_

  bool InitDevice(wxString port, int bouds=115200); //Initialize the device
  int CloseDevice();				    //Close the device

  wxString GetDevice();	

  int SendSMS(wxString phone_number, wxString message);
  int ReceiveMessages(omSmsArray *messages, wxString smstype);
  int DeleteMessage(int num);

private:
  wxString port;	///< Port to connect the device
  int device_init;	///< Flag for device initialization
  int device;		///< Device handler

private:
  int WriteString(char *s, int how);
  int WriteChar(char c, char *answ, int lansw);
  
  int SendString(char *dom, char *ris);

  int QueryPhone(char *qy, char *stop, char *answ);
  omSms *GetSmsHeader(wxString buf);

};

#endif // SMSMANAGER_H
