////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#ifndef _omTCPParser_
#define _omTCPParser_

#include <openmaia/commons.h>
#include <openmaia/omObject.h>
#include <openmaia/omTCPMessage.h>
#include <openmaia/omFunctions.h>

#include <wx/wx.h>

#define OM_NO_MESSAGE 0
#define OM_REGISTER_MESSAGE 1
#define OM_GENERIC_MESSAGE 2


/** @ingroup omgen
    TCP Message Parser.

    This class is used to parse TCP messages used by the agents.<br>
    Every character received from a TCP socket is analized by AnalizeChar() method of
    this class.
    This method compose and analize the message.
    The method return a code corresponing to the type of the message analized.<br>
    - OM_NO_MESSAGE: The message is not completed.
    - OM_REGISTER_MESSAGE: Registration Type Message.
    - OM_GENERIC_MESSAGE: Generic Type Message.

    If OM_REGISTER_MESSAGE is received method GetRegisterMessage() return message data.<br>
    If OM_GENERIC_MESSAGE is received method GetGenericMessage() return message data.
*/



class omTCPParser: public omObject
{
 private:
  // status flags
  bool messageReceived;             ///< True if message is received
  bool receivingMessage;            ///< True if in receiving message status
  bool registerMessage;             ///< True if register meessage type
  bool genericMessage;              ///< True if generic message type

  omTCPMessage genericMessageData;  ///< Received message buffer
  wxString agentName;               ///< Registered message agent name
  wxString serviceName;             ///< Registered message service name

  // parser flags and vars and methods
  bool receivingEscape;             ///< True if receiving an escape sequence
  bool receivingRegister;           ///< flag
  bool receivingGeneric;            ///< flag
  bool messageStarting;             ///< flag
  bool messageEnding;               ///< flag
  bool messageStarted;              ///< flag
  bool messageEnded;                ///< flag
  int charCounter;                  ///< generic character counterz
  wxString partialMessage;          ///< Partial message buffer

  bool GetParameterValue(wxString, wxString *);
  int GetDecodedParameters(wxString, wxArrayString *); 
  void ResetInitialStatus();

 public:
  omTCPParser();
  int AnalizeChar(char);
  bool GetRegisterMessage( wxString *, wxString *);
  bool GetGenericMessage( omTCPMessage *);

};

#endif 

