////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omUIActionQueue_
#define _omUIActionQueue_

#include <openmaia/commons.h>
#include <list>
#include <algorithm>
#include <string>

//#include <string.h>



#include <openmaia/omObject.h>
#include <openmaia/omUIApplication.h>
#include <openmaia/omCommand.h>
#include <openmaia/omUICommandParser.h>

#include <wx/wx.h>

/** @ingroup omui
    This object is used to queue the events generated by the user.
    The events are usually generated by the virtual keyboard and put in the
    queue using Put() method.
    Events than can be extracted from the queue with Pop() method.
    The queue has a timer that can be used for event pooling. If you use the event
    pooling when events are found are passed to the application commandParser object.
*/

class omUIActionQueue: public omObject, public wxTimer{
 private:
  std::list<omCommand > *l ;  ///< Events list.
  int timeTick ;   ///< Timer period in milliseconds.

  void Notify() ;

 public:
  omUIActionQueue(int) ;
  void Put( omCommand);
  bool Get(omCommand *);
  void Start() ;
  void Stop() ;
} ;


#endif // _omUIActionQueue_
