////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omUIApplication_
#define _omUIApplication_

#include <openmaia/commons.h>
#include <wx/wx.h>
#include <iostream>

//#include <unistd.h>

#include <openmaia/omFunctions.h>
#include <openmaia/omUIObjects.h>


/** @ingroup omui
This class specify a generic OpenMAIA User Interface Application. <br><br>

This class is used as a main class for an OpenMAIA User Interface Application.
A user interface is used by the disabled user to intercat with the OpenMAIA System.<br>
Generally a user interfac application is composed by some objects (see picture).
This class contains the pointers to the super objects used in the user interface application.

<img src="images/omUIApplication.png">

- The omUIKeyboard object is used to store keyboard data structure.<br>
  The keyboard is rapresented by a collection of buttons stored in a tree data structure.
  Every Node of the tree is a omUITreeNode object and every leaf is a omUIButton object.
- omUIGrantFocus2User object is used to grant that the OpenMAIA buttons don't steal the focus
  to the user.
- omUIActionQueue is used as an event queue.
  The keyboard generates events and pass them to the queue. The queue pass the events with
  FIFO method to the omUICommandParser object
- omUICommandParser is the object that analizes events coming from the queue, from the
  devices eventually handled and from the connector.
- omConnector is used to receive and send message to the agency via TCP/IP connection.
  When messages are received they are parsed and passed to the omUICommandParser object.
  Other objects can send messages to the agency using SendMessage method of the instantied 
  omConnector object.
- omUIOptions store all user interface application options.

All objects are instantiated in the omUIApplication::OnInit method.
omUIApplication::objects attribute is used to store the pointers to the instantiated
objects.<br>

Every Class that include <libopenmaiaui.h> header can obtain the pointer to an
object used in the user interface application with a command like the following one:<br><br>

omConnector *connector = (omConnector) wxGetApp().objects.connector;<br><br>

<b>NOTE!!</b> To link the correct wxApp pointer you need to implement the application 
using omUIApplication object.


*/
class omUIApplication: public wxApp
{
 public:
  /// maia application objects pointers
  omUIObjects objects ;   ///< Objects container.
                          ///< Used to store pointers to objects used in the application

  //  void GrabActionKeys() ;
  //  void UngrabActionKeys() ;

  virtual bool OnInit();   ///< This is the main of the user interface application.

 private:
  // --- private variables

  // set sistem paths
  void SetPath( wxString) ;
  void SetErrorHandlers() ;

  // system dependent methotds
  void SDSetErrorHandlers();   ///< System dependent SetErrorHandlers

};

DECLARE_APP(omUIApplication) 


#endif // _omUIApplication_
