////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////




#ifndef _omUICommandParser_
#define _omUICommandParser_

#include <openmaia/commons.h>
#include <iostream>
#include <string>
#include <wx/wx.h>
#include <wx/textfile.h>

#include <openmaia/omObject.h>
#include <openmaia/omUIApplication.h>
#include <openmaia/omCommand.h>
#include <openmaia/omExecutor.h>
#include <openmaia/omUIKeyboard.h>


/** @ingroup omui
    User Interface Application Command Parser.

    This class is a generic User Interface Application Command Parser.
    This class is used to handle events generated from the virtual keyboard, from
    the connector and eventually from the device drivers.
    This class parses some actions and contain an executor that make some device 
    emulations if the application need to emulate functions.
    The emulator may be disabled if the application is only a user interface and doesn't
    make any emulation.<br>
    This class handles executor start and stop. These methods starts and stops internal
    executor but also uses the connector object of the UI application to send the
    start and stop message through the TCP connections.
    This is the list of command parsed by the class:
    - EmulateString - string emulation
    - EnterClick - Enter emulation
    - CapsLockClick - Caps emulation
    - BackSpaceClick - BS emulation
    - EmulateSymbol - Symbol emulation
      - lt - < symbol
      - gt - > symbol
      - dq - " symbol
    - MoveMouseLeft, MoveMouseRight, MoveMouseUp, MoveMouseDown - mouse movement emulations
    - LeftClick, CentralClick, RightClick - mouse button click emulations
    - LeftDragOrDrop, CentralDragOrDrop, RightDragOrDrop - mouse button push or pop emulations
    <br><br>
    A class descendig from this class may reimplement ParseAction() method to parse
    specifi actions of the application and may call the parent method to parse generic actions.
    ParseAction return value true if the action is found and false if is not found.

    @todo
     - Implement start and stop execution through the TCP connections.
     - Implement the disable of the executor.

*/

class omUICommandParser: public omObject {
 protected:
  omExecutor *executor;     ///< Pointer to the executor.
  omUIKeyboard *keyboard;   ///< Pointer to the virtual keyboard object

 public:
  omUICommandParser() ;

  void Init();

  virtual bool ParseAction(omCommand);

  virtual void EndParsing();
  virtual void StartExecutor();
  virtual void StopExecutor() ;
} ;

#endif
