////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omUIKeyboard_
#define _omUIKeyboard_

#include <openmaia/commons.h>
#include <openmaia/omObject.h>
#include <openmaia/omUIApplication.h>
#include <openmaia/omUIOptions.h>
#include <openmaia/omUITreeNode.h>
#include <openmaia/omXMLParameters.h>
#include <openmaia/omXMLParser.h>
#include <openmaia/omUIButton.h>


#include <wx/wx.h>


/** @ingroup omui 

    This class rapresent a generic Open Maia virtual keyboard structure with the XML parser
    using the gnome libxml2 library.
    There are some virtual functions called by the parser. These functions must be
    overridden to handle application specific keyboard.
    See MaiaKeyboard as an example.
*/
class omUIKeyboard: public omXMLParser {
  protected:
  
  omUITreeNode *rootNode;   // Root of the buttons tree
  omUITreeNode *currentNode ;  // Current Node marked

  omUIOptions *opts;
  bool active ;    // true if the keyboard is active and the events will be passed to the tree
  

 public:  
  omUIKeyboard() ;  // constructor
  virtual void Init() ;
  virtual void Reload();
  void Show() ;  // show the keyboard
  bool IsYourWindow( void *) ;  // check if window is one of leafs
  virtual void LoadKeyboard(wxString ) ;
  virtual void Action() ;
  virtual void Tick() ;
  virtual void RepaintKeyboard();


} ;


#endif // _omUIKeyboard_
