////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omUIObjects_
#define _omUIObjects_

#include <openmaia/commons.h>
#include <openmaia/omObject.h>

#include <iostream>

/** @ingroup omui
    User Interface Application objects.

    This class is used to store  pointers of all objects used in a User Interface
    application.
    The pointers are stored in public attributes.
    All pointers are omObject and must be casted to use specific methods of the classes.

    @see omUIApplication for more informations.
*/

class omUIObjects
{
 public:
  // public application variables
  omObject *focusChecker ;  ///< Focus Checker.
  omObject *keyboard ;             ///< Virtual Keyboard.
  omObject *options;                ///< Application options.
  omObject *actionsQueue;               ///< Event Queue.
  omObject *commandParser ;               ///< Command parser.
  omObject *connector;               ///< TCP Connector.

  // public runtime variables
  omObject *lastEventWindow;         ///< The windows that generated last event.

  // public methods
  omUIObjects();
  void Init() ;
  void StartAllTimers() ;
  void StopAllTimers() ;
  void Reload();
} ;

#endif // _omUIObjects_
