////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omUIOptions_
#define _omUIOptions_

#include <openmaia/commons.h>
#include <openmaia/omObject.h>
#include <openmaia/omOptions.h>
#include <openmaia/omXMLParser.h>
#include <openmaia/omFunctions.h>
#include <openmaia/omcommon.h>
#include <openmaia/omemukeys.h>

#include <wx/wx.h>


/** @ingroup omui

    This class contains all generic options in a maialike application.
    The class can be extended and a subclass can be used instead of it.

    All generic objects in the api structure use this class to store options.

*/

class omUIOptions: public omOptions{
  
 public:  
  omUIOptions() ;
  virtual void Init() ;

  bool SetParameter(wxString, wxString);
  wxString GetParameter(wxString);
  wxString GenerateOptionsXMLString();

  // ----------------------- INTERNAL OPTIONS ----------------------

  // ------ Display options ---------------

  int screenWidth;                      ///< Current screen width in pixel
  int screenHeight;                     ///< Current screen height in pixel

  wxString keyboardName;                ///< name of current keyboard

  bool scanReverseDirection ;           ///< Direction of scan:
                                        ///< false - scan from left to right (default)
                                        ///< true - scan from right to left


  // ----- GENERAL ---
  /** key code of the action key */
  int actionKey ;
  /** direction key codes */
  int leftKey, 
    rightKey, 
    upKey, 
    downKey ;

  /** true if command keys are grabbed and not repeated to the
      operating system */
  bool grabKeys ;   

  /** true, the action is taken on button press
      false, the action is taken on button release
  */
  bool actionOnPress;

  bool soundEnabled;                          ///< flag (default: true)

  /** listen to serial port flag */
  bool serialEnabled ;
  /** serial port */
  wxString serialPort ;
  /** listen to game joystick check */
  bool joystickEnabled ;

  /** listen to keyboard as action keys */
  bool keyboardEnabled ;

  /** autoscan flag */
  bool autoscan ;
  /** joystick flag */
  bool joystick ;
  /** direct pointing flag */
  bool direct ;

  // ------ SCAN ---
  // ------ Automatic scan options --------
  /** Scan timing in milliseconds */
  int scanTime;

  /** number of scan loops before exit */
  int scanLoopsNumber;
  /** True if you want a little delay after the key is pressed */
  bool delayAfterClick ;
  /** True if you want a little delay after a key is actioned
      this means that after you press the action key on a leaf you got a delay
  */
  bool delayAfterAction;

  /** True if you want a time delay before exit from a scan block */
  bool delayAfterEndBlock ;
  /** which node resume when go to up block?
      You can RESUMEFROMFIRST, RESUMEFROMTHIS, RESUMEFROMNEXT
  */
  int scanResumeFrom;
  /** True if you want to immediatly exit after user click the button */
  bool exitBlockAfterClick ;

  int joyMaxX,           ///< Maximum X value of joystick (-1 if not initialized)
    joyMinX,             ///< Minimum X value of joystick (-1 if not initialized)
    joyMaxY,             ///< Maximum Y value of joystick (-1 if not initialized)
    joyMinY;             ///< Minimum Y value of joystick (-1 if not initialized)
} ;


#endif // _omUIOptions_
