////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#ifndef _omXMLParameters_
#define _omXMLParameters_

#include <openmaia/commons.h>
#include <openmaia/config.h>
#include <openmaia/omXMLParameter.h>
#include <openmaia/omFunctions.h>

#include <wx/wx.h>
#include <sstream>
#include <iostream>
#include <vector>


/** @ingroup omgen 
    This class handle a list of parameters used by the XML parser.
    Some methods let you insert and retrive parameters, one by one or searching by name.
*/


class omXMLParameters {
 protected:
  // -- variables
  std::vector<omXMLParameter > parameters ;   // parameters of the key
  int num;   // number of stored parameterss
  int cur ;  // current parameter in automatic mode

 public:
  // -- methods

  // constructor
  omXMLParameters() ;
  // add a new parameter
  bool AddParameter( wxString, wxString) ;
  // get parameter value by name
  wxString GetValue( wxString ) ;
  // gete parameter value as integer by name
  int GetIntValue( wxString ) ;
  // get parameter name by id
  wxString GetName( int ) ;
  // get parameter value by id
  wxString GetValue( int ) ;
  // get parameter value in lowercase by id
  wxString GetLowerValue( int) ;
  // get parameter value in lowercase by name
  wxString GetLowerValue( wxString) ;
  // get parameter name in lowercase by id
  wxString GetLowerName( int ) ;
  // get number of parameters
  int GetParsNumber() ;
  // get next parameter
  omXMLParameter GetNextParameter() ;
  // reset parameter counter
  void ResetCounter() ;
  
} ;


#endif 
