////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omXMLParser_
#define _omXMLParser_

#include <openmaia/commons.h>
#include <openmaia/omObject.h>
#include <openmaia/omFunctions.h>
#include <openmaia/omXMLParameters.h>


#include <libxml/parser.h>
#include <libxml/parserInternals.h>


#include <wx/wx.h>


/** @ingroup omgen
    This class rapresent a generic Maia keyboard structure with the XML parser
    using the gnome libxml2 library.
    There are some virtual functions called by the parser. These functions must be
    overridden to handle application specific keyboard.
    See MaiaKeyboard as an example.
*/
class omXMLParser: public omObject {
 private:
  xmlSAXHandler *omSAXHandler ;

 public:
  // constructor
  omXMLParser() ;
  // load new keyboard in the structure
  bool Parse( wxString ) ;
  
  // these functions are public because must to be called by the C functions of the parser.
  // the user must not call the functions directly!
  // called by the parser at the start of the document
  virtual void AtStartDocument() ;
  // called by the parser at the end of the document
  virtual void AtEndDocument() ;
  //called by the parser at the start of a tag
  virtual void AtStartTag( wxString n, omXMLParameters p) ;
  // called by the parser at the end of a tag
  virtual void AtEndTag( wxString n) ;

  
  // utility functions
  static wxString GetXMLOptionsHeader();
} ;


#endif // _omXMLParser_
