////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include "keyringCommandParser.h"

/** Build the class */
keyringCommandParser::keyringCommandParser()
{
}

/** Process NotifyTCPMessage event.
    A TCP Message is coming from an other agent in the federation and is notified by 
    the connector object of the application.<br>
    This method handles messages regarding options communication from maia and maiaconfig.
    @param message message received.
*/
void keyringCommandParser::NotifyTCPMessage(omTCPMessage message)
{
#ifdef OLD
 wxString command = message.GetCommand();

  // switch command ------------------------------------

  // ----------------------------------------------------- GetParValue --
  // get parameter value from options and send a message
  // to maiaconfig containing parameter value
  if (command.CmpNoCase(_T("GetParValue")) == 0)
    {
      wxString parName = message.GetParameter(0);
      wxString parValue = opts->GetParameter(parName) ;

      // build reply message
      omTCPMessage reply ;
      
      reply.toAgent = _T("maiaconfig");
      reply.command = _T("ParValue");
      reply.AddParameter(parName);
      reply.AddParameter(parValue);
      
      // send message
      connector->SendMessage(reply) ;
    }

  // ----------------------------------------------------- SetParValue --
  // get parameter value from options and send a message
  // to maiaconfig containing parameter value
  if (command.CmpNoCase(_T("SetParValue")) == 0)
    {
      wxString parName = message.GetParameter(0);
      wxString parValue = message.GetParameter(1);

      opts->SetParameter(parName, parValue);
      

      wxGetApp().objects.Reload();
    }

  if (command.CmpNoCase(_T("SaveParameters")) == 0)
    {
      opts->SaveOptions(_T("maia.conf"));
    }
  
  // ---------------------------------------------------------- Drive Commands
  // commands used to drive maia cursor
  // UISelect, UIGoUp, UIGoDown, UIGoLeft and UIGoRight

  // --- UISelect
  if (command.CmpNoCase(_T("UISelect")) == 0)
    {
      (wxGetApp().objects.commandParser)->Action();
    }

  // --- UIGoUp
  if (command.CmpNoCase(_T("UIGoUp")) == 0)
    {
      int i;
      // number of repeat
      wxString ns = message.GetParameter(0);
      int n = omFunctions::String2Int(ns);
      
      for (i=0; i<n; i++)
	{
	  (wxGetApp().objects.commandParser)->GoUp();
	}
    }


  // --- UIGoDown
  if (command.CmpNoCase(_T("UIGoDown")) == 0)
    {
      int i;
      // number of repeat
      wxString ns = message.GetParameter(0);
      int n = omFunctions::String2Int(ns);
      
      for (i=0; i<n; i++)
	{
	  (wxGetApp().objects.commandParser)->GoDown();
	}
    }

  // --- UIGoLeft
  if (command.CmpNoCase(_T("UIGoLeft")) == 0)
    {
      int i;
      // number of repeat
      wxString ns = message.GetParameter(0);
      int n = omFunctions::String2Int(ns);
      
      for (i=0; i<n; i++)
	{
	  (wxGetApp().objects.commandParser)->GoLeft();
	}
    }

  // --- UIGoRight
  if (command.CmpNoCase(_T("UIGoRight")) == 0)
    {
      int i;
      // number of repeat
      wxString ns = message.GetParameter(0);
      int n = omFunctions::String2Int(ns);
      
      for (i=0; i<n; i++)
	{
	  (wxGetApp().objects.commandParser)->GoRight();
	}
    }


#endif  
   

}

/** Initialize the class.
    This initialization is done after the construction.
*/
void keyringCommandParser::Init() 
{
  // initialize object pointers
  keyboard = (keyringKeyboard *) wxGetApp().objects.keyboard ;
  opts = (keyringOptions *) wxGetApp().objects.options ;
  // initialize keyboard options
  if (opts->keyboardEnabled)
    kbDriver = new omKeyboardDriver(this, this, 30, opts->actionKey, opts->upKey ,
				  opts->downKey, opts->leftKey, opts->rightKey, 
				  opts->actionOnPress, opts->grabKeys) ;
  else
    kbDriver = NULL ;

  if (opts->joystickEnabled)
    jDriver = new omJoystickDriver(this, 30, wxJOYSTICK1, 0.2, 0.8, 0.2, 0.8, opts->actionOnPress) ;
  else
    kbDriver = NULL ;

  if (opts->serialEnabled)
    sDriver = new omSerialDriver(this, 30, opts->serialPort, opts->actionOnPress) ;
  else
    kbDriver = NULL ;

  connector = (omConnector *) wxGetApp().objects.connector ;

  // store actual values of parameters
  actionKey = opts->actionKey;
  downKey = opts->downKey;
  upKey = opts->upKey;
  leftKey = opts->leftKey;
  rightKey = opts->rightKey;
  actionOnPress = opts->actionOnPress;
  keyboardEnabled = opts->keyboardEnabled;
  joystickEnabled = opts->joystickEnabled;
  serialEnabled = opts->serialEnabled;
  serialPort = opts->serialPort;
}


/** Reload options after parameters change. */
void keyringCommandParser::Reload()
{
  opts = (keyringOptions *) wxGetApp().objects.options ;

  // check if drivers values has changed

  // keyboard driver
  if (keyboardEnabled != opts->keyboardEnabled ||
      actionKey != opts->actionKey ||
      upKey != opts->upKey ||
      downKey != opts->downKey ||
      leftKey != opts->leftKey ||
      rightKey != opts->rightKey ||
      actionOnPress != opts->actionOnPress ||
      grabKeys != opts->grabKeys )
    {
      delete kbDriver;
      if (opts->keyboardEnabled)
	{
	  kbDriver = new omKeyboardDriver(this, this, 30, opts->actionKey, opts->upKey ,
				  opts->downKey, opts->leftKey, opts->rightKey, 
				  opts->actionOnPress, opts->grabKeys) ;
	  kbDriver->Start();
	}
      else
	kbDriver = NULL ;
    }

  // joy driver
  if (joystickEnabled != opts->joystickEnabled)
    {
      delete jDriver;
      if (opts->joystickEnabled)
	{
	  jDriver = new omJoystickDriver(this, 30, wxJOYSTICK1, 0.2, 0.8, 0.2, 0.8, opts->actionOnPress);
	  jDriver->Start();
	}
      else
	jDriver = NULL ;
    }

  // serial driver
  if (serialEnabled != opts->serialEnabled ||
      serialPort.CompareTo(opts->serialPort) != 0)
    {
      delete sDriver;
      if (opts->serialEnabled)
	{
	  sDriver = new omSerialDriver(this, 30, serialPort, opts->actionOnPress) ;
	  sDriver->Start();
	}
      else
	sDriver = NULL ;
    }


  // store actual values of parameters
  actionKey = opts->actionKey;
  downKey = opts->downKey;
  upKey = opts->upKey;
  leftKey = opts->leftKey;
  rightKey = opts->rightKey;
  actionOnPress = opts->actionOnPress;
  keyboardEnabled = opts->keyboardEnabled;
  joystickEnabled = opts->joystickEnabled;
  serialEnabled = opts->serialEnabled;
  serialPort = opts->serialPort;
}

/** Some drivers pools the device to get its status.
    This method start all drivers timers.
*/
void keyringCommandParser::Start() 
{
  kbDriver->Start() ;
  jDriver->Start();
  sDriver->Start() ;
}

/** Stop all drivers timers */
void keyringCommandParser::Stop() 
{
  kbDriver->Stop() ;
  jDriver->Stop();
  sDriver->Stop() ;
}


// message handlers ------




/** Parse Action coming from other object.
    @param m Action received.
*/
bool keyringCommandParser::ParseAction( omCommand m)
{
  //  std::cout << "parse action\n" ;
  bool parsed = omUICommandParser::ParseAction(m) ;


#ifdef OLD
  wxString action = m.GetCommand() ;
  wxArrayString parameters = m.GetParameters() ;

  //std::cout << "azione = " << action << " parametri= " << parameters << std::endl ;

  // da sistemare perche' dovrebbe inizializzarlo con la funzione Init()
  // ma non va
  keyringKeyboard *mkeyboard = (keyringKeyboard *) wxGetApp().objects.keyboard ;

  // ---------------------------------------------INTERNAL CODES ------------
  // ------------------------------------------------------------------------
  if ( action.CmpNoCase(_T("exitfrommaia")) == 0)
    exit(0) ;
  // -------------------------------------------- Scan Handling ----
  if ( action.CmpNoCase(_T("automaticscanon")) == 0)
    {
      if ( mkeyboard != NULL )
	mkeyboard->Start() ;
    }

  if ( action.CmpNoCase(_T("automaticscanoff")) == 0)
    {
      if ( mkeyboard != NULL )
	mkeyboard->Stop() ;
    }
  // --------------------------------------------- keyboard handling --------
  if (action.CmpNoCase(_T("loadkeyboard")) == 0)
    {
      // retrive keyboard from application
      std::cout << "entrato\n" ;
      wxGetApp().objects.StopAllTimers() ;
      mkeyboard->LoadKeyboard(parameters[0]) ;
      mkeyboard->Show();
      wxGetApp().objects.StartAllTimers() ;
    }

#endif

  return false ;


}


void keyringCommandParser::LeftButtonDown()
{
  std::cout << "left down" << std::endl;
  keyboard->LeftButtonDown();
}
  
void keyringCommandParser::LeftButtonUp()
{
  std::cout << "left up" << std::endl;
  keyboard->LeftButtonUp();
}

void keyringCommandParser::RightButtonDown()
{
  keyboard->RightButtonDown();
}

void keyringCommandParser::RightButtonUp()
{
  keyboard->RightButtonUp();
}


void keyringCommandParser::UpButtonDown()
{
  keyboard->UpButtonDown();
}

void keyringCommandParser::UpButtonUp()
{
  keyboard->UpButtonUp();
}

void keyringCommandParser::DownButtonDown()
{
  keyboard->DownButtonDown();
}

void keyringCommandParser::DownButtonUp()
{
  keyboard->DownButtonUp();
}

