////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#ifndef _keyringCommandParser_
#define _keyringCommandParser_

#include <openmaia/libopenmaia.h>
#include <openmaia/libopenmaiaui.h>
#include <openmaia/libopenmaiadrivers.h>

#include "keyringKeyboard.h"
#include "keyringOptions.h"

#include <wx/wx.h>
#include <wx/joystick.h>

#include <iostream>

/** @ingroup keyring
    This class extends functionalities of omUICommandParser.
    This object is the main event handling object of the maia application, 
    events are received from periperals drivers (keyboard, serial and joy) and
    from other agents via communication object (omConnector). <br>

    All the periperal objects are instantiated from keyringCommandParser.
*/

class keyringCommandParser: public omUICommandParser{
 private:
  // --- variables to store actual value of options

  // keyboard drivers options
  int actionKey, upKey, downKey, leftKey, rightKey;  ///< Keyboard driver parameter.
  bool actionOnPress, grabKeys, keyboardEnabled; ///< Keyboar driver Flag.

  // joy driver
  bool joystickEnabled;    ///< Joystick driver flag.

  // serial driver
  bool serialEnabled;      ///< Serial driver flag.
  wxString serialPort ;    ///< Serial driver parameter (in unix style, es /dev/ttyS0)


  omKeyboardDriver *kbDriver;  ///< Keyboard driver object
  omJoystickDriver *jDriver;   ///< Joystick driver object
  omSerialDriver *sDriver;     ///< Serial driver object
  keyringKeyboard *keyboard;      ///< Pointer to virtual keyboard object
  keyringOptions *opts ;          ///< Pointer to application options object
  omConnector *connector;      ///< Pointer to connector

 public:
  keyringCommandParser();
  void Init() ;
  void Reload();
  void Start() ;
  void Stop() ;

  void NotifyTCPMessage(omTCPMessage);
  bool ParseAction(omCommand);


  void LeftButtonDown();
  void LeftButtonUp();
  void RightButtonDown();
  void RightButtonUp();
  void UpButtonDown();
  void UpButtonUp();
  void DownButtonDown();
  void DownButtonUp();
};

#endif
