////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include "keyringKeyboard.h"


BEGIN_EVENT_TABLE( keyringKeyboard, wxFrame) 
  EVT_TIMER(TIMER_ID, keyringKeyboard::OnMovieTimerEvent)
END_EVENT_TABLE()


/** Call the constructor of the parent class 
    @param p Parent window of all the buttons (may be null)
    @param tick Scan interval in milliseconds
*/
keyringKeyboard::keyringKeyboard()
  : omUIKeyboard(),
  wxFrame(NULL,-1, wxString(_T("Keyring")), wxPoint(0,100), wxSize(1024,32), wxDEFAULT_FRAME_STYLE)
{
  curAngle=-0.001;
  rEye=6.0;

  newDir1=0;
  newDir2=0;

  oldDir1=0;
  oldDir2=0;

  busy=false;
  resetRing=false;


  movieTimer=new wxTimer(this,TIMER_ID);
}

/** Do post constructor initialization */
void keyringKeyboard::Init()
{
  omUIKeyboard::Init() ;
  opts = (keyringOptions *) wxGetApp().objects.options ;

  //wxXmlResource::Get()->LoadFrame(this, NULL, wxT("loop.xrc"));
  //wxXmlResource *resFile = new wxXmlResource(wxT("loop.xrc"));
  wxXmlResource::Get()->Load(wxT("loop.xrc"));
  
  int i;
  for (i=1;i<opts->sectors+1;i++)
    {
      wxString panelName = wxT("P");
      panelName << i;
      wxXmlResource::Get()->LoadPanel(this,panelName);
    }

  
  movieTimer->Start(50);
			 
  //wxTimerEvent gino;
  //this->OnMovieTimerEvent(gino);


}

/** Reload options after parameters change */
void keyringKeyboard::Reload()
{
  omUIKeyboard::Reload() ;
}


/** This method implement the virtual function of MaiaGenericKeyboard class. 
    The method creates the root node of the tree
*/
void keyringKeyboard::AtStartDocument()
{
}


/** This method implement the vitual function of omUIKeyboard class.
    The method activate the keyboard to let possible to catch events and transmit
    them to the root
*/
void keyringKeyboard::AtEndDocument() 
{
}

/** This method implement the vitual function of MaiaGenericKeyboard class.
    The method parse the tag name to fill correctly the tree.
    @param name tag name
    @param p parameters of the tag
*/
void keyringKeyboard::AtStartTag(wxString name, omXMLParameters p)
{
}

/** This method implement the vitual function of MaiaGenericKeyboard class.
    The method parse the tag and if necessary close open jobs (ex insert a button in the
    structure).
    @param name tag name
*/
void keyringKeyboard::AtEndTag( wxString name)
{
} 



/** Load the keyboard using MaiaGenericKeyboard function */
void keyringKeyboard::LoadKeyboard(wxString keyb)
{
}

void keyringKeyboard::Show()
{
  wxFrame::Show(TRUE);
}


void keyringKeyboard::OnMovieTimerEvent(wxTimerEvent &event)
{
  if (busy)
    return;

  busy=true;

  switch(newDir1)
    {
    case 1:
      rEye=opts->rRing+(rEye-opts->rRing)/1.1;
      break;

    case 2:
      rEye=opts->rRing+(rEye-opts->rRing)*1.1;
      break;
    }

  float rotSpeed=opts->dAngle*((opts->dAngleBaseAdd/10)+rEye-opts->rRing)*opts->sectors/100000;

  switch(newDir2)
    {
    case 3:
      curAngle=curAngle+rotSpeed;
      break;
    case 4:
      curAngle=curAngle-rotSpeed;
      break;
    }

  if (rEye>opts->MAXrEye)
    {
      // backspace emulation
      rEye=opts->rEyeOldRing;
    }

  if (!(newDir1 != 0 || newDir2 != 0 || !resetRing)) {
    busy=false;
    return;
  }

  // else
  resetRing=true;

  float vSpace = opts->rRing*fabs(sin(acos(opts->rRing/rEye))-sin(-1*acos(opts->rRing/rEye)));
  int i;
  for (i=0;i<opts->sectors;i++)
    {
      wxString panelName = wxT("P");
      panelName << (i+1);

      float a1 = curAngle+(2*M_PI/opts->sectors)*i;
      float a2 = curAngle+(2*M_PI/opts->sectors)*(i+1);

      if (sin(a1) < sin(-1*acos(opts->rRing/rEye)) && cos(a1) > 0 && sin(a2) > sin(acos(opts->rRing/rEye)) &&
	  cos(a2) > 0 )
	{
	  // code simulation
	  std::cout << i << std::endl;

	  rEye = opts->rEyeNewRing;
	}

      if (cos(a1) <= (opts->rRing/rEye) && cos(a2) <= (opts->rRing/rEye))
	{
	  // hide sector
	  
	  (XRCCTRL(*this, panelName, wxPanel))->SetPosition(wxPoint(0,32+1));
	} else 
	  {
	    // show sector
	    if (cos(a1) <= (opts->rRing/rEye))
	      a1 = -1*acos(opts->rRing/rEye);
	    
	    if (cos(a2) <= (opts->rRing/rEye))
	      a2 = acos(opts->rRing/rEye);
	    
	    float xp = rEye - (opts->rRing*opts->rRing)/rEye;
	    float yp1 = xp * (opts->rRing*sin(a1))/(rEye-opts->rRing*cos(a1));
	    float yp2 = xp * (opts->rRing*sin(a2))/(rEye-opts->rRing*cos(a2));

	    (XRCCTRL(*this, panelName, wxPanel))->SetPosition(wxPoint( lrint(1024/2+(1024/vSpace)* yp1) ,0));
	    (XRCCTRL(*this, panelName, wxPanel))->SetSize(wxSize(lrint((1024/vSpace)*fabs(yp2-yp1)+1),32));
	  }
    }

  busy=false;
}


void keyringKeyboard::LeftButtonDown()
{
  newDir2=3;
}
  
void keyringKeyboard::LeftButtonUp()
{
  if (newDir2==3)
    newDir2=0;
}

void keyringKeyboard::RightButtonDown()
{
  newDir2=4;
}

void keyringKeyboard::RightButtonUp()
{
  if (newDir2==4)
    newDir2=0;
}


void keyringKeyboard::UpButtonDown()
{
  newDir1=1;
}

void keyringKeyboard::UpButtonUp()
{
  if (newDir1==1)
    newDir1=0;
}

void keyringKeyboard::DownButtonDown()
{
  newDir1=2;
}

void keyringKeyboard::DownButtonUp()
{
  if (newDir1==2)
    newDir1=0;
}
