////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#ifndef _keyringKeyboard_
#define _keyringKeyboard_


#include <openmaia/libopenmaia.h>
#include <openmaia/libopenmaiaui.h>

#include <omcommon.h>

#include "keyringOptions.h"

#include <unistd.h>
#include <math.h>

#include <wx/wx.h>


#define TIMER_ID 1

/** @ingroup keyring

    This class handles a maia keyboard with some MaiaButtons objects stored
    in a tree structure.
    The class is an implementation of MaiaGenericKeyboard class that is gives
    general methods and structures.
    Virtual methods of MaiaGenericKeyboard are implemented here.
*/

class keyringKeyboard: public omUIKeyboard, public wxFrame{
 private:

  keyringOptions *opts;          ///< Application options

  float curAngle,
    rEye;

  int newDir1,
    newDir2,
    oldDir1,
    oldDir2;

  bool resetRing,
    busy;

  wxTimer *movieTimer;

  // ------------------------ private functions
  void Notify() ;
  void LinkConnectedButtons();

 public:
  keyringKeyboard() ;
  void Init();
  void Reload();
  // methods used by the parser
  void AtStartDocument() ;  
  void AtEndDocument() ;
  void AtStartTag( wxString, omXMLParameters ) ;
  void AtEndTag( wxString ) ;
  // method used by the automatic scan system

  void LoadKeyboard(wxString) ;  // load the keyboard
  void Show();
  void OnMovieTimerEvent(wxTimerEvent &event);



  void LeftButtonDown();
  void LeftButtonUp();
  void RightButtonDown();
  void RightButtonUp();
  void UpButtonDown();
  void UpButtonUp();
  void DownButtonDown();
  void DownButtonUp();

  DECLARE_EVENT_TABLE()
} ;



#endif // _keyringKeyboard_
