////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omCommand.h>

/** Initialize a void omCommand class.
*/
omCommand::omCommand()
{
  command = _T("") ;
}


/** Initialize the class with a command and a single parameter.
    @param c Command code.
    @param p First parameter.
*/
omCommand::omCommand(wxString c, wxString p)
{
  command = c ;
  parameters.Add(p) ;
}

/** Initialize the class with a command and a some parameters.
    @param c Command code.
    @param pars Parameters list.
*/
omCommand::omCommand(wxString c, wxArrayString pars)
{
  command = c ;
  parameters = pars;
}


/** Initialize the class with a command.
    @param c Command code.
*/
omCommand::omCommand(wxString c)
{
  command = c ;
}


/** Get stored command.
    @return string containing the command
*/
wxString omCommand::GetCommand()
{
  return command ;
}

/** Get a stored parameter.
    @param idx index of the parameter to retrive
    @return string containing the parameter.
*/
wxString omCommand::GetParameter( int idx)
{
  if ( idx < parameters.GetCount())
    return parameters[idx] ;
  else
    return (_T("")) ;
}

/** Get the first stored parameter.
    @return string containing the parameter.
*/
wxString omCommand::GetParameter()
{
  if ( parameters.GetCount() > 0)
    return parameters[0] ;
  else
    return (_T("")) ;
}

/** Get all stored parameters as a wxArrayString.
    @return All stored parameters.
*/
wxArrayString omCommand::GetParameters()
{
  return parameters;
}

/** Get the number of stored parameters.
    @return parameters number.
*/
int omCommand::GetParCount()
{
  return parameters.GetCount();
}

/** Set command.
    @param c string containing the command
*/
void omCommand::SetCommand(wxString c)
{
  command = c ;
}

/** Set parameters
    @param p string containing the parameters
*/
void omCommand::AddParameter(wxString p)
{
  parameters.Add(p) ;
}
