////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omConnector.h>

enum
{
SOCKET_ID=1000
} ;
 
BEGIN_EVENT_TABLE(omConnector, wxEvtHandler)
  EVT_SOCKET(SOCKET_ID, omConnector::OnSocketEvent)
END_EVENT_TABLE()


/** Builds the omConnector object using defaults:
    - hostname = localhost
    - port = 60201
    - notify messages = false
*/
omConnector::omConnector()
{
  incomingMessage = wxString(_T("")) ;
  hostname = wxString(_T("localhost"));
  service = 60201;
  connected = false ;
  notifyMessages=false;
}

/** Builds the omConnector object for only message trasmission.
    @param host Host to connect.
    @param port Port.
*/
omConnector::omConnector(wxString host, unsigned short port)
{
  incomingMessage = wxString(_T("")) ;
  hostname = host;
  service = port;
  connected = false ;
  notifyMessages=false;
  currentIP = _T("");
}



/** Builds the omConnector object for only receiving.
    @param tn object to sent NotifyMessage event
*/
omConnector::omConnector(omObject *tn)
{
  incomingMessage = wxString(_T("")) ;
  hostname = wxString(_T("localhost"));
  service = 60201;
  connected = false ;
  toNotify = tn;
  notifyMessages=true;
  currentIP = _T("");
}

/** Builds the omConnector object for sending and receiving.
    @param tn object to send NotifyMessage event
    @param hn hostname
    @param p  port
*/
omConnector::omConnector(omObject *tn, wxString hn, unsigned short p)
{
  incomingMessage=wxString(_T(""));
  connected = false ;
  toNotify = tn ;
  hostname = hn;
  service=p;
  notifyMessages=true;
  currentIP = _T("");
}

/** Connect and register the agent 
    @return true if the connection was ok
*/
bool omConnector::Connect()
{
  sock = new wxSocketClient();
//   sock->SetEventHandler(*this, SOCKET_ID);
//   sock->SetNotify( wxSOCKET_CONNECTION_FLAG | wxSOCKET_INPUT_FLAG | wxSOCKET_LOST_FLAG ) ;

  sock->SetEventHandler(*this, SOCKET_ID);
//   sock->SetNotify(wxSOCKET_CONNECTION_FLAG |
//                     wxSOCKET_INPUT_FLAG |
//                     wxSOCKET_LOST_FLAG);
  sock->SetNotify( wxSOCKET_INPUT_FLAG ) ;
  if(notifyMessages){
    sock->Notify(TRUE);
  }


//   sock->SetNotify( wxSOCKET_INPUT_FLAG ) ;
//   sock->Notify(TRUE);

  // connect to the server
  wxIPV4address addr;
  //addr.Hostname("localhost");
  //addr.Service(60201);

  addr.Hostname(hostname);
  addr.Service(service);

#ifdef MAIA_DEBUG
  std::cout << "omConnect::Connect - conneting to " << hostname << ":" << service << std::endl;
#endif

  bool success = sock->Connect(addr, FALSE) ;

  // get client side socket informations
  wxIPV4address addrClient;
  sock->GetLocal(addrClient);
  currentHost = addrClient.Hostname();
  currentPort = addrClient.Service();

  #ifdef MAIA_DEBUG
  std::cout << "omConnect::Connect - Service side address" << currentHost << ":" << currentPort << std::endl;
#endif


  if (success)
    {
      connected = true ;
      currentIP = omFunctions::GetCurrentIP() ;
    }

  return success;

}


/** Send a message to the server 
    @param m Message to send.
*/
void omConnector::SendMessage(omTCPMessage m)
{
  wxString message = m.GetFormattedMessage();
  sock->Write(message.c_str(), message.Length());
}


/** Send the click event to the connected server */
void omConnector::SendClick()
{
  //  this->SendMessage(omMessage(wxString(_T("Action")),wxString(_T(""))));
}


// ---------------------------------- private methods --------------------------------

/// Called by the event handler.
void omConnector::OnSocketEvent(wxSocketEvent &event)
{
  if (event.GetSocketEvent() == wxSOCKET_CONNECTION)
    {
      connected = true;
#ifdef MAIA_DEBUG
  std::cout << "omConnector: Socket connection accepted" << std::endl;
#endif
    }

  if (event.GetSocketEvent() == wxSOCKET_INPUT)
    {
      char c;
      int tcpParserStatus=-1;
      do {
	sock->Read(&c,1);
	tcpParserStatus = tcpParser.AnalizeChar(c);
      } while (sock->IsData() && tcpParserStatus!=OM_GENERIC_MESSAGE);

      if (tcpParserStatus==OM_GENERIC_MESSAGE)
	{
	  omTCPMessage tmpmess;
	  tcpParser.GetGenericMessage(&tmpmess);

	  if(notifyMessages)
	    toNotify->NotifyTCPMessage(tmpmess);
	}

    }


//   char c;
  
//   // read from IO channel until data is present
//   do {
//     sock->Read(&c,1);
//     incomingMessage << c;
//   } while (sock->IsData());

//   if ( c== '\n')       // discard characters after \n mmm!!! ... to modify!! 
//     sock->Discard();


//   if ( c == '\n')     // presume end of the message
//     {
//       int i,j;
//       // parse the message
//       wxString message, parameters;

//       i = incomingMessage.Find(MAIACOMMAND);
//       i = i+strlen(MAIACOMMAND);                            // start of command name
//       j = incomingMessage.Find(MAIASEPARATOR);
//       message = incomingMessage.Mid(i,j-i);

//       i = incomingMessage.Find(MAIASEPARATOR);
//       i = i + strlen(MAIASEPARATOR) + strlen(MAIAPARAMETERS);
//       j = incomingMessage.Find(MAIAENDMESSAGE);
//       parameters = incomingMessage.Mid(i,j-i);

//       // extrapolate single parameters from parameter list

//       // count parameters number
//       int npars = 1 ;
//       wxString partialPars = parameters;
//       // single parameters array
//       wxArrayString parsList;
//       i = 0 ;  // inizial counter
//       j = partialPars.Find(MAIAPARSEPARATOR);
//       while ( j != -1 )
// 	{
// 	  wxString currentParameter = partialPars(0, j) ;
// 	  parsList.Add(currentParameter);
// 	  // remove store parameter
// 	  partialPars.Remove(0, currentParameter.Len() + sizeof(MAIAPARSEPARATOR)) ;
// 	  j = partialPars.Find(MAIAPARSEPARATOR);
// 	}
//       // add the last element
//       wxString currentParameter = partialPars(0, partialPars.Len()) ;
//       parsList.Add(currentParameter) ;

//       incomingMessage = wxString(_T("")) ;
      
//       // notify message to the connected object
//       //toNotify->NotifyMessage(wxString(_T(message.c_str())), wxString(_T(parameters.c_str())));
//       toNotify->NotifyMessage(wxString(_T(message.c_str())), parsList);
//     }

}

