////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include "omDialog.h"



/// Build omDialog class.
///
/// @param parent parent window of the dialog.
/// @param title Title of the dialg.
/// @param dialogName Name of the dialog in the loaded XRC file
omDialog::omDialog(wxWindow *parent, wxString title, wxString dialogName )
  //  : wxDialog(parent, -1, title ,wxDefaultPosition,wxDefaultSize, wxDEFAULT_DIALOG_STYLE, title   )
{
  // load resource file
  wxXmlResource::Get()->LoadDialog(this, NULL, dialogName);
  this->SetTitle(title);
  
  // initialize variables
  buttonPressed = -1;
  isPressed = false;
}

/// Override ShowModal method of wxDialog class.
int omDialog::ShowModal()
{
  isPressed = false;
  buttonPressed = -1;
  return wxDialog::ShowModal();
}

/// Return true if a button was pressed
bool omDialog::IsButtonPressed()
{
  return isPressed;
}

/// Return the code of the button pressed or -1 if no button was pressed.
int omDialog::GetButtonPressed()
{
  return buttonPressed;
}

/// Call wxDialog::Destroy()
bool omDialog::Destroy()
{
  return wxDialog::Destroy();
}


// -------------------------------------------------------- Events Handlers -----------

/// Ok Button handler.
void omDialog::OnOkButton(wxCommandEvent& WXUNUSED(event))
{
  this->Hide();
  isPressed = true;
  buttonPressed = omOK_BUTTON;
}

/// Cancel Button handler
void omDialog::OnCancelButton(wxCommandEvent& WXUNUSED(event))
{
  this->Hide();
  isPressed = true;
  buttonPressed = omCANCEL_BUTTON;
}


/// Yes Button handler
void omDialog::OnYesButton(wxCommandEvent& WXUNUSED(event))
{
  this->Hide();
  isPressed = true;
  buttonPressed = omYES_BUTTON;
}

/// No Button handler
void omDialog::OnNoButton(wxCommandEvent& WXUNUSED(event))
{
  this->Hide();
  isPressed = true;
  buttonPressed = omNO_BUTTON;
}

/// No Button handler
void omDialog::OnSaveButton(wxCommandEvent& WXUNUSED(event))
{
  this->Hide();
  isPressed = true;
  buttonPressed = omSAVE_BUTTON;
}

// ---------------------------------------------------------------------  event table

BEGIN_EVENT_TABLE(omDialog, wxDialog)
  EVT_BUTTON(XRCID(_T("buttonOk")), omDialog::OnOkButton)
  EVT_BUTTON(XRCID(_T("buttonCancel")), omDialog::OnCancelButton)
  EVT_BUTTON(XRCID(_T("buttonYes")), omDialog::OnYesButton)
  EVT_BUTTON(XRCID(_T("buttonNo")), omDialog::OnNoButton)
  EVT_BUTTON(XRCID(_T("buttonSave")), omDialog::OnSaveButton)
END_EVENT_TABLE()
