// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omFunctions.h>

/** Get a unix style string from a generic string.
    This means that all \ chars are converted into / chars 
    @param gs general string
    
    @return unix style string
*/

wxString omFunctions::GetUnixString(wxString gs)
{
  wxString newString = gs ;
  newString.Replace(_T("\\"),_T("/")) ;
  return(newString);
} 

/** Get a windows style string from a generic string.
    This means that all / chars are converted into \ chars 
    @param gs general string
    
    @return unix style string
*/

wxString omFunctions::GetWindowsString(wxString gs)
{
  wxString newString = gs ;
  newString.Replace(_T("/"),_T("\\")) ;
  return(newString);
} 


/** convert a string to lowercase
    @param s source string
    @return lowercase string
*/
wxString omFunctions::ConvertToLower(wxString s) 
{
//   std::transform(s.begin(), s.end(), s.begin(), ToLower()) ;
//   return s ;
  wxString newString = s ;
  newString.MakeLower();
  return(newString);

}

/** convert a string to uppercase
    @param s source string
    @return uppercase string
*/
wxString omFunctions::ConvertToUpper(wxString s) 
{
//   std::transform(s.begin(), s.end(), s.begin(), ToUpper()) ;
//   return s ;
  wxString newString = s ;
  newString.MakeUpper();
  return(newString);
}

/** Find a position af a string in an array conserving the case.

    @param a Pointer to source array.
    @param s String to find.

    @return Index of the string or -1 if not found.
*/
int omFunctions::ArrayFind(wxArrayString *a, wxString s)
{
  int i;
  int found = -1;
  for ( i=0; i < a->GetCount(); i++)
    {      
      if ( a->Item(i).CompareTo(s) == 0) 
	{
	  found = i;
	  i = a->GetCount()-1;   // go to the end of the array
	}
    }

  return found;
}

/** Find a position af a string in an array ignoring the case.

    @param a Pointer to source array.
    @param s String to find.

    @return Index of the string or -1 if not found.
*/
int omFunctions::ArrayFindNoCase(wxArrayString *a, wxString s)
{
  int i;
  int found = -1;
  for ( i=0; i < a->GetCount(); i++)
    {      
      if ( a->Item(i).CmpNoCase(s) == 0) 
	{
	  found = i;
	  i = a->GetCount()-1;   // go to the end of the array
	}
    }

  return found;
}

/** Sets the path of the application moving to the top of the user data dir
    @param binPath - path of binary file, from argv
*/
void omFunctions::SetBasePath(wxString binPath)
{
  // change the path in unix format always
  wxString currentPath = omFunctions::GetUnixString(binPath);

#ifdef MAIA_DEBUG
  std::cout << "omFunctions::SetBasePath - currentPath= " << currentPath << std::endl ;
#endif

  omFunctions::SetBasePathSD(currentPath);

  // to continue ...
  // set path to developement tree home

  // -- obsolete --wxString newPath=_T("../../data") ;
  

  //int pippo=chdir("../../data");
//   int pippo=chdir("./~");
//   std::cout << "pippo= "<< pippo <<std::endl;
//   std::cout << "errno= " << errno << std::endl ;

  // change to ~/.openmaia/profiles path
  //wxString newPath=_T("~/.openmaia/profiles");

  //chdir((char *)newPath.c_str()) ;

//   int pippo=chdir ("~");
//   std::cout << "errno= " << errno << std::endl ;
//   // get path information from argv[0]
//   // this is different under windows and unix.
//   // In windows the path is absolute, in unix generally is relative

//   wxString op(av[0]);
  
//   // change the path in unix format always
//   wxString currentPath = omFunctions::GetUnixString(op);
  
//   // path format is something like c:/yourbasepath/bin/application under windows
//   // and ./application or application under unix
//   // find /bin substring to check if use relative or absolute path
//   // I would advice to use absolute path under windows

//   wxString newPath ;
//   int pos = currentPath.Find(_T("/bin"));
//   if ( pos == -1 )   // substring not found, assume to use relative path 
//     {
//       newPath = wxString(_T("..")) ;   // move to the top of the installation
//     }
//   else  // use absolute path
//     {
//       newPath = currentPath.SubString(0,pos);
//     }
  
//   // change to base path
//   chdir((char *)newPath.c_str()) ;
}


/** This function return the Maia Key code from its definition in 
    the maia xml codify according omXMLTags.h header file
    @param xmlCode xml codify of the key
    @return maia virtual code codify of the key, -1 if key code was not found

    @todo Implement all special keys conversion from XML to MaiaKeycode
*/
long omFunctions::GetMaiaKeycodeFromXML( wxString xmlCode) 
{
    
  // check for special keys ... to complete
  if ( xmlCode.CmpNoCase(OMXMLCONTROLVALUE )==0)
    return CONTROLKEY ;

  if ( xmlCode.CmpNoCase(OMXMLSHIFTVALUE)==0)
    return SHIFTKEY ;

  if ( xmlCode.CmpNoCase(OMXMLUPVALUE)==0)
    return UPARROWKEY ;

  if ( xmlCode.CmpNoCase(OMXMLDOWNVALUE) == 0)
    return DOWNARROWKEY ;

  if ( xmlCode.CmpNoCase(OMXMLLEFTVALUE) == 0)
    return LEFTARROWKEY ;

  if ( xmlCode.CmpNoCase(OMXMLRIGHTVALUE) == 0)
    return RIGHTARROWKEY ;

  // if arrived here the code is presumed to be some standard ascii char.
  // in case of a string first char will be used.

  
  char c = xmlCode.GetChar(0);   // return the first char of the string
  return (long) c;
  
  // return error
  return -1 ;

}


/** Test for data installation path.<br>
    The position of the user data files is different if you are using windows or un*x system.<br>
    - Under Windows systems data files are installed in folder 
    %PROFILE_NAME\\openmaia\\openmaia_profile\\application\\ if windows profiles are activated<br>
    If profiles are not activated data files are stored in
    %OPENMAIA_INSTALLATION_PATH\\profiles\\openmaia_profile\\application.<br>
    %PROFILE_NAME folder is found by 
    HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\User Shell Folders\\AppData
    registry key.
    
    - Under Un*x system data files are stored in ~/.openmaia/openmaia_profile/application

    @param profile OpenMAIA profile name.
    @param appname OpenMAIA application name.
    @param binPath Path of binary file

    @return true if data path is found, false otherwhise.
*/
bool omFunctions::TestPersonalInstallation(wxString profile, wxString appname, wxString binPath)
{
  return omFunctions::TestPersonalInstallationSD(profile, appname, binPath) ;
}

/** This function copy a default data installation from the template
    directory into the user personal directory.<br>
    The function is lounched the first time that an OpenMAIA application is lounched.
    @param appName OpenMAIA application name.
    @param binPath Path of binary (from argv)

    @return true if succeded, false if error.
*/
bool omFunctions::CreateDefaultInstallation(wxString appName, wxString binPath)
{
  // call system dependent method
  return omFunctions::CreateDefaultInstallationSD(appName, binPath);
}

/** This function test for folder existence 
    @param folder folder name

    @return true if folder exist.
*/
bool omFunctions::TestFolder(wxString folder)
{
  return wxDir::Exists(folder);

//   int rvalue=0, i;
//   folder = omFunctions::GetUnixString(folder);
//   wxArrayString components = omFunctions::SplitTextLine(folder, _T("/"));
//   int ncomponents = components.Count();
  
//   // check if path starts with / symbol not returned by splitter function
//   if (folder.GetChar(0) == '/')
//     {
//       components.Insert(_T("/"),0);
//       ncomponents++;
//     }

//   for (i=0; i<ncomponents; i++)
//     {
//       wxString component = components[i];
//       rvalue = chdir( (char *) component.c_str());
//       if (rvalue == -1)
// 	break;
//     }

//   if (rvalue == -1)
//     {
//       switch(errno)
// 	{
// 	case ENOENT:    // path not found
// 	  return false;
// 	  break;
// 	}
//     }

//   return true;
}

/** Create a new folder using mkdir command.
    Under Windows is used mkdir.exe provided with openmaia distribution
    @param folder Folder to create
    @param binPath Path of the folder contining binaries of mkdir (windows)
*/
void omFunctions::CreateFolder(wxString folder,wxString binPath)
{
  omFunctions::CreateFolderSD(folder,binPath);
}


/** Test for file presence.
    @param fname File name.

    @return True if file is present
*/
bool omFunctions::TestFile(wxString fname)
{
  return wxFile::Exists(fname);
//   int rvalue = chdir((const char *) fname.c_str()) ;

//     if (rvalue == -1)
//     {
//       switch(errno)
// 	{
// 	case ENOENT:
// 	  return false;
// 	  break;
// 	}
//     }

//   return true;

}

/** Get Current IP of the computer connected to a network
    @return IP in dot format
*/
wxString omFunctions::GetCurrentIP()
{
//   char szHostName[128];
//   wxString str;
//   struct hostent * pHost;
//   int i = 0, j = 0;
//   int addCount = 0;

//   wxArrayString StrAddresses;
//   wxUint32 longAddresses[12];

//   if(gethostname(szHostName, 128) == 0 )
//     {
//       pHost = gethostbyname(szHostName);
//       for( i = 0; pHost!= NULL && pHost->h_addr_list[i]!= NULL && i < 6; i++ )
// 	{
// 	  for( j = 0; j < pHost->h_length; j++ )
// 	    {
// 	      if( j > 0 )
// 		{
// 		  str += _T(".");
// 		}
// 	      wxString tmpString = _T("");
// 	      tmpString.Printf("%d",(unsigned int)((unsigned char*)pHost->h_addr_list[i])[j]);
// 	      str += tmpString;
// 	    }

// 	  // Do not store an address if it is localhost
// 	  if (str != _T("127.0.0.1"))
// 	    {
// 	      // Now that we have an ip address, store it
// 	      StrAddresses.Add(str);
// 	      addCount++ ;
// 	    } str.Clear();
// 	}
//     }
//   if (addCount > 0)
//     return StrAddresses[0];
//   else
//     return _T("127.0.0.1");

  return _T("127.0.0.1");
}

/** Encode a string for TCP trasmission according the protocol.
    In this encoding ; $ # and \ symbols are translated into escape sequences
    \; \# \$, and \\.
    Carriege return is translated into \\r symbol and New line in \\n symbol.
    @param original Original string to encode.

    @return Encoded string.
*/
wxString omFunctions::Encode(wxString original)
{
  int i;
  wxString returnString = _T("");
  for (i=0; i<original.Length(); i++)
    {
      char c = original.GetChar(i);
      if ( c == ';' || c == '$' || c == '#' || c == '\\' || c == '\n')
	{
	  if ( c == ';')
	    returnString += _T("\\;");
	  if ( c == '$')
	    returnString += _T("\\$");
	  if ( c == '#')
	    returnString += _T("\\#");
	  if ( c == '\\') {
	    returnString += _T("\\");
	    returnString += _T("\\");
	  }
	  if ( c == '\n')
	    returnString += _T("\\n");
	  if ( c == '\r')
	    returnString += _T("\\r");
	}
      else
	{
	  wxString tmpString;
	  tmpString.Printf("%c",c);
	  returnString += tmpString ;
	}
    }
  return returnString;
}

/** Decode a string encoded with omFunctions::Encode method.
    @param original Original string to decode.

    @return Decoded string.
*/
wxString omFunctions::Decode(wxString original)
{
  wxString returnString=_T("");
  wxString tmpString;
  int i=0;
  
  for (i=0; i<original.Length(); i++)
    {
      char c=original.GetChar(i);
      if ( c == '\\' )    // get next char
	{
	  i++;
	  c=original.GetChar(i);
	  if ( c == 'n') 
	    returnString += _T("\n");
	  if ( c == 'r') 
	    returnString += _T("\r");
	  else
	    {
	      tmpString.Printf("%c",c);
	      returnString += tmpString;
	    }
	}
      else
	{
	  tmpString.Printf("%c",c);
	  returnString += tmpString;
	}
    }
  return returnString;
}

/** Find the first not blank position of a char in a string.
    Blank chars are spaces and \n symbols.
    @param s source string
    @param sp start position

    @return position of not blank char after start position; -1 if not blank char is not found
*/
int omFunctions::FindFirstNBChar(wxString s, int sp)
{
  int i;
  for (i=sp; i<s.Length();i++)
    if (s[(unsigned int)i] != ' ' && s[(unsigned int)i] != '\n' && s[(unsigned int)i] != '\r')
      return i;

  // char not found
  return -1;
}


/** Find the last not blank position of a char in a string.
    Blank chars are spaces and \n symbols.
    @param s source string

    @return position of not blank char after start position; -1 if not blank char is not found
*/
int omFunctions::FindLastNBChar(wxString s)
{
  int i;
  char c ;
  for (i=s.Length()-1; i>=0; i--)
    {
      c = s[(unsigned int)i];
      if (c != ' ' && c != '\n' && c != '\r')
	return i;
    }
  // char not found
  return -1;
}

/** Find the position of the first not encoded char in a string
    @param s source string
    @param c char to find	
    @param sp start position

    @return position shearced char; -1 if not blank char is not found
*/
int omFunctions::FindNotEncodedChar(wxString s, char c, int sp)
{
  int findPosition=-1;
  int i=sp;
  int endposition=s.Length();
  char tmpChar;
  do {
    tmpChar=s.GetChar(i);
    if ( tmpChar == c && i > sp)  // check if not an escape char
      {
	char tmpChar2=s.GetChar(i-1);
	if ( tmpChar2 != '\\' )
	  {
	    findPosition=i;
	    break;
	  }
      }
    i++;
  } while ( i!=endposition);

  return findPosition;
}


/** Split a line of text composed by elements and separator and return 
    an array containing all the elements without white spaces.
    @param str string to split
    @param sep separator
    
    @return an array containing all components of the string
*/
wxArrayString omFunctions::SplitTextLine(wxString str, wxString sep)
{
  int sepPos = str.Find(sep);
  int startpos, endpos;
  wxString tmp;   // temporary string
  wxString lineRest = str;   // contain the rest of the line

  wxArrayString result;

  while ( sepPos != -1)     // last part
    {
      tmp = lineRest.SubString(0,sepPos-1);
      // tail white spaces
      startpos = omFunctions::FindFirstNBChar(tmp,0);
      endpos = omFunctions::FindLastNBChar(tmp);
      tmp=tmp.SubString(startpos,endpos);

      // tail the rest of the line
      lineRest=lineRest.SubString(sepPos + sep.Length(), lineRest.Length());
      sepPos = lineRest.Find(sep);

      // add result to the array
      if (tmp.Cmp(_T("")) != 0)
	result.Add(tmp);
    }

  // check if lineRest contains last element
  startpos = omFunctions::FindFirstNBChar(lineRest,0);
  endpos = omFunctions::FindLastNBChar(lineRest);
  lineRest = lineRest.SubString(startpos, endpos);
  if (lineRest.Cmp(_T("")) != 0)
    result.Add(lineRest);
  
  return result;
}

/** Get current path.
    This method uses getcwd() C function and fix some problems in
    path returned from cygwin.

    @return current path in unix format.
*/
wxString omFunctions::GetCurrentPath()
{
  char buf[512];
  getcwd(buf,512);   // get current path

  wxString currentPath(buf);
  currentPath = omFunctions::GetUnixString(currentPath);

  // search for '/cygdrive/<drive>' used by cygwin
  size_t pos;
  pos = currentPath.First(_T("/cygdrive/")) ; 
  if ( pos != -1)
    {
      char drive = currentPath.GetChar(pos+10);
      std::cout << "drive = " << drive << std::endl;
      wxString tmpString = _T("");
      tmpString << drive << _T(":") << currentPath.Mid(pos+11);
      currentPath = tmpString;
    }

  return currentPath;
}


/** Return OpenMAIA constant literal given its value.
    @param value Constant value.
    
    @return Constant literal as integer.

    @see omemukeys.h
*/
wxString omFunctions::Value2Literal(int value)
{

  wxString literal = _T("");

  switch (value)
    {
    case 30:
      literal = _T("RESUMEFROMFIRST");
      break;

    case 31:
      literal = _T("RESUMEFROMTHIS");
      break;
      
    case 32:
      literal = _T("RESUMEFROMNEXT");
      break;


    case 256:
      literal = _T("BACKSPACEKEY");
      break;

    case 257:
      literal = _T("TABKEY");
      break;

    case 258:
      literal = _T("CLEARKEY");
      break;

    case 259:
      literal = _T("ENTERKEY");
      break;
      
    case 260:
      literal = _T("SHIFTKEY");
      break;

    case 261:
      literal = _T("CONTROLKEY");
      break;

    case 262:
      literal = _T("ALTKEY");
      break;

    case 263:
      literal = _T("PAUSEKEY");
      break;

    case 264:
      literal = _T("CAPSLOCKKEY");
      break;

    case 265:
      literal = _T("ESCKEY");
      break;

    case 266:
      literal = _T("SPACEKEY");
      break;

    case 267:
      literal = _T("PAGEUPKEY");
      break;

    case 268:
      literal = _T("PAGEDOWNKEY");
      break;

    case 269:
      literal = _T("ENDKEY");
      break;

    case 270:
      literal = _T("HOMEKEY");
      break;

    case 271:
      literal = _T("LEFTARROWKEY");
      break;

    case 272:
      literal = _T("UPARROWKEY");
      break;

    case 273:
      literal = _T("RIGHTARROWKEY");
      break;

    case 274:
      literal = _T("DOWNARROWKEY");
      break;

    case 275:
      literal = _T("SELECTKEY");
      break;

    case 276:
      literal = _T("PRINTSCREENKEY");
      break;

    case 277:
      literal = _T("EXECUTEKEY");
      break;

    case 278:
      literal = _T("SNAPSHOTKEY");
      break;
      
    case 279:
      literal = _T("INSKEY");
      break;

    case 280:
      literal = _T("DELKEY");
      break;

    case 281:
      literal = _T("HELPKEY");
      break;
      
    case 282:
      literal = _T("NUMLOCKKEY");
      break;

      // ----------------------------------------- FUNCTION KEYS ----

    case 400:
      literal = _T("F1KEY");
      break;

    case 401:
      literal = _T("F2KEY");
      break;

    case 402:
      literal = _T("F3KEY");
      break;

    case 403:
      literal = _T("F4KEY");
      break;

    case 404:
      literal = _T("F5KEY");
      break;

    case 405:
      literal = _T("F6KEY");
      break;

    case 406:
      literal = _T("F7KEY");
      break;

    case 407:
      literal = _T("F8KEY");
      break;

    case 408:
      literal = _T("F9KEY");
      break;

    case 409:
      literal = _T("F10KEY");
      break;

    case 410:
      literal = _T("F11KEY");
      break;

    case 411:
      literal = _T("F12KEY");
      break;

    case 412:
      literal = _T("F13KEY");
      break;

    case 413:
      literal = _T("F14KEY");
      break;

    case 414:
      literal = _T("F15KEY");
      break;

    case 415:
      literal = _T("F16KEY");
      break;


    default:
      literal = _T("");
      break;
    }
  
  return literal;

}

/** Return OpenMAIA constant value given its literal.
    @param literal Literal value.
    
    @return Constant value as integer or -1 if litaral was not found.

    @see omemukeys.h
*/
int omFunctions::Literal2Value(wxString literal)
{
  int value = -1;

  // parse literal string (caseSensitive)

  if (literal.CompareTo(_T("RESUMEFROMFIRST")) == 0 )
    value = RESUMEFROMFIRST;
  
  if (literal.CompareTo(_T("RESUMEFROMTHIS")) == 0 )
    value = RESUMEFROMTHIS;

 if (literal.CompareTo(_T("RESUMEFROMNEXT")) == 0 )
    value = RESUMEFROMNEXT;

  // ---------------------------------------------------------- SPECIAL KEYS ---
  if (literal.CompareTo(_T("BACKSPACEKEY")) == 0 )
    value = BACKSPACEKEY;

  if (literal.CompareTo(_T("TABKEY")) == 0 )
    value = TABKEY;

  if (literal.CompareTo(_T("CLEARKEY")) == 0 )
    value = CLEARKEY ;

  if (literal.CompareTo(_T("ENTERKEY")) == 0 )
    value = ENTERKEY;

  if (literal.CompareTo(_T("SHIFTKEY")) == 0 )
    value = SHIFTKEY;

  if (literal.CompareTo(_T("CONTROLKEY")) == 0 )
    value = CONTROLKEY;

  if (literal.CompareTo(_T("ALTKEY")) == 0 )
    value = ALTKEY;

  if (literal.CompareTo(_T("PAUSEKEY")) == 0 )
    value = PAUSEKEY;

  if (literal.CompareTo(_T("CAPSLOCKKEY")) == 0 )
    value = CAPSLOCKKEY;

  if (literal.CompareTo(_T("ESCKEY")) == 0 )
    value = ESCKEY;

  if (literal.CompareTo(_T("SPACEKEY")) == 0 )
    value = SPACEKEY;

  if (literal.CompareTo(_T("PAGEUPKEY")) == 0 )
    value = PAGEUPKEY;

  if (literal.CompareTo(_T("PAGEDOWNKEY")) == 0 )
    value = PAGEDOWNKEY;

  if (literal.CompareTo(_T("ENDKEY")) == 0 )
    value = ENDKEY;

  if (literal.CompareTo(_T("HOMEKEY")) == 0 )
    value = HOMEKEY;

  if (literal.CompareTo(_T("LEFTARROWKEY")) == 0 )
    value = LEFTARROWKEY;

  if (literal.CompareTo(_T("UPARROWKEY")) == 0 )
    value = UPARROWKEY;

  if (literal.CompareTo(_T("RIGHTARROWKEY")) == 0 )
    value = RIGHTARROWKEY;

  if (literal.CompareTo(_T("DOWNARROWKEY")) == 0 )
    value = DOWNARROWKEY;

  if (literal.CompareTo(_T("SELECTKEY")) == 0 )
    value = SELECTKEY;

  if (literal.CompareTo(_T("PRINTSCREENKEY")) == 0 )
    value = PRINTSCREENKEY;

  if (literal.CompareTo(_T("EXECUTEKEY")) == 0 )
    value = EXECUTEKEY;

  if (literal.CompareTo(_T("SNAPSHOTKEY")) == 0 )
    value = SNAPSHOTKEY;

  if (literal.CompareTo(_T("INSKEY")) == 0 )
    value = INSKEY;

  if (literal.CompareTo(_T("DELKEY")) == 0 )
    value = DELKEY;

  if (literal.CompareTo(_T("HELPKEY")) == 0 )
    value = HELPKEY;

  if (literal.CompareTo(_T("NUMLOCKKEY")) == 0 )
    value = NUMLOCKKEY;

  // --------------------------------------------------------- FUNCTION KEYS ---

  if (literal.CompareTo(_T("F1KEY")) == 0 )
    value = F1KEY;

  if (literal.CompareTo(_T("F2KEY")) == 0 )
    value = F2KEY;

  if (literal.CompareTo(_T("F3KEY")) == 0 )
    value = F3KEY;

  if (literal.CompareTo(_T("F4KEY")) == 0 )
    value = F4KEY;

  if (literal.CompareTo(_T("F5KEY")) == 0 )
    value = F5KEY;

  if (literal.CompareTo(_T("F5KEY")) == 0 )
    value = F5KEY;

  if (literal.CompareTo(_T("F6KEY")) == 0 )
    value = F6KEY;

  if (literal.CompareTo(_T("F7KEY")) == 0 )
    value = F7KEY;

  if (literal.CompareTo(_T("F8KEY")) == 0 )
    value = F8KEY;

  if (literal.CompareTo(_T("F9KEY")) == 0 )
    value = F9KEY;

  if (literal.CompareTo(_T("F10KEY")) == 0 )
    value = F10KEY;

  if (literal.CompareTo(_T("F11KEY")) == 0 )
    value = F11KEY;

  if (literal.CompareTo(_T("F12KEY")) == 0 )
    value = F12KEY;

  if (literal.CompareTo(_T("F13KEY")) == 0 )
    value = F13KEY;

  if (literal.CompareTo(_T("F14KEY")) == 0 )
    value = F14KEY;

  if (literal.CompareTo(_T("F15KEY")) == 0 )
    value = F15KEY;

  if (literal.CompareTo(_T("F16KEY")) == 0 )
    value = F16KEY;

  return value;

}


/** Convert a string that spacifies a boolean value into a boolean type.
    String may contain 0,1, false, true, FALSE, true values.

    @param s String.

    @return boolean rapresentation of the string.
    @see IsBoolean() method to test if the format of string is correct. 
*/
bool omFunctions::String2Boolean(wxString s)
{
  if ( (s.CmpNoCase("FALSE") == 0) || (s.CmpNoCase("0") == 0))
    return false;
  else
    return true;   // by default. See IsBoolean Method to test string format.
}

/** Check if string format is compatible with boolean type.
    A string compatible with boolean type may contain false,true, FALSE, TRUE, 0, 1 values.
    
    @param s Source String.

    @return true if string is compatible, false if not
*/
bool omFunctions::IsBoolean(wxString s)
{
  if ( (s.CmpNoCase("FALSE")==0) || (s.CmpNoCase("0")==0) ||
       (s.CmpNoCase("TRUE")==0) || (s.CmpNoCase("1")==0))
    return true;

  return false;
}

/** Convert a boolean value into an uppercase string.

    @param b Boolean value.

    @return String rapresentation of the boolean value as wxString.
*/
wxString omFunctions::Boolean2String(bool b)
{
  if (b)
    return _T("TRUE");
  else
    return _T("FALSE");
}

/** Convert a boolean value into an uppercase string.

    @param b Boolean value.

    @return String rapresentation of the boolean value as wxString.
*/
wxString omFunctions::Boolean2String(int b)
{
  if (b)
    return _T("TRUE");
  else
    return _T("FALSE");
}


/** Check if string format is compatible with integer type.
    
    @param s Source String.

    @return true if string is compatible, false if not
*/
bool omFunctions::IsInt(wxString s)
{
  long longVal=0;
  return s.ToLong(&longVal) ;
}

/** Convert a string that spacifies an integer value into an integer type.

    @param s String.

    @return integer rapresentation of the string.
    @see IsInt() method to test if the format of string is correct. 
*/
int omFunctions::String2Int(wxString s)
{
  long longVal=0;  // assume 32 bit os, so int and long are the same!

  s.ToLong(&longVal);
  return (int) longVal;
}


/** Return the path of shared part of the OpenMAIA application.
    OpenMAIA save some informations in a shared path.<br>
    Under Un*x system shared path is tipically /usr/local/share/openmaia or PREFIX/share/openmaia
    if the tarball was configured with --prefix options.<br>
    Under Windows shared path is INSTALLATION_PATH/share and is calculated using
    the path of the binary file.

    @param binPath Path of the binary file.

    @return Shared path as wxString.
*/
wxString omFunctions::GetSharedPath(wxString binPath)
{
  return omFunctions::GetSharedPathSD(binPath);
}


/** Return the path of the directory where openmaia is installed.
    This method is used in windows implementations beacuse those OS does't handle
    relative path well.
    The path is taken from the binary path of the binary file running at this time.

    @param binPath Path of this binary file.
*/
wxString omFunctions::GetInstallationPath(wxString binPath)
{
  return omFunctions::GetInstallationPathSD(binPath);
}


/** Return the path of the directory where user data is stored.
    @param binPath Path of the binary file.

    @return user data path as wxString
*/
wxString omFunctions::GetUserDataPath(wxString binPath)
{
  return omFunctions::GetUserDataPathSD(binPath);
}


/** Copy original file into destination using cp command.
    Under un*x systems this method uses system copy command, under Windows
    it uses cp.exe program provided with the openmaia suite.

    @param source Source file.
    @param destination Destination file or folder
    @param binPath Path of the openmaia agent binary file.
           This parameter is used to find cp.exe program under Windows systems.
*/
void omFunctions::CopyFile(wxString source, wxString destination, wxString binPath)
{
  omFunctions::CopyFileSD(source,destination, binPath);
}
