////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omKeyboardDriver.h>



/** Initialize the class.
    @param object object to inform if event occoured.
    @param ec object that controls the executor (maybe null).
    @param tick time between check tics in milliseconds.
    @param action maia key code of the action key.
    @param up maia key code of the up key.
    @param down maia key code of the down key.
    @param left maia key code of the left key.
    @param right maia key code of the right key.
    @param onpress true if action on press false if action on release.
    @param grab true if keys must be grabbed by the application. (see SetGrabKeys)
*/
omKeyboardDriver::omKeyboardDriver(omObject *object, omObject* ec, int tick, 
				   long action, long up, long down, long left, long right,
				   bool onpress, bool grab)
{
  toInform = object ;
  executorController = ec;
  tickTime = tick ;
  actionKey = action;
  upKey = up;
  downKey = down ;
  leftKey = left;
  rightKey = right;
  actionOnPress = onpress;

  //grabKeys = false ;
  grabKeys = grab ;

  busy=false;

  this->SystemDependentConstructor() ;
}

/** The destructor releases grabbed keys of the keyboard for further use */
omKeyboardDriver::~omKeyboardDriver()
{
  std::cout << "distruttore tastiera" << std::endl;
  this->SDDestructor();
}

/** Set OnPress flag
    @param op value of the onpress flag
*/
void omKeyboardDriver::SetOnPress(bool op)
{
  actionOnPress = op ;
}

/** Set GrabKeys flag.<br>
    This class detects the state of the keys specified in the constructor.<br>
    When a key is pressed or released (according with the state of actionOnPress flag)
    an event is generated.
    If keys are not grabbed the pression even is propageted to the windows manager
    so the system can handle key event.
    If keys are grabbed by the OpenMAIA application the system is unable to handle
    key event so the key is useless by the system.
    @param grab value of the flag.
*/
void omKeyboardDriver::SetGrabKeys(bool grab)
{
  grabKeys = grab ;
}


/** Start event pooling with the time interval specified in the constructor*/
void omKeyboardDriver::Start()
{
  wxTimer::Start(tickTime) ;
}

/** Start event pooling with the time interval specified in the constructor*/
void omKeyboardDriver::Stop()
{
  wxTimer::Stop() ;
}


// ---------------------- private methods ---------

/// override the Notify method of the wxTimer parent
void omKeyboardDriver::Notify()
{
  if (busy)
    return;

  busy=true;
  this->SystemDependentNotify() ;
  busy=false;
}

/// This function is used to grab the control of all action keys.
/// <b>Avaiable only in windows.</b>
void omKeyboardDriver::GrabActionKeys()
{
  grabKeys = true;
  this->SDGrabActionKeys() ;
}

/// This function is used to ungrab the control of the action keys.
void omKeyboardDriver::UngrabActionKeys()
{
  grabKeys = false ;
  this->SDUngrabActionKeys() ;
}


/// Flush the state of control keys.<br>
/// This method is used to reset the state of the control keys.
void omKeyboardDriver::FlushKeysState()
{
  this->SDFlushKeysState();
}

