////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omObject.h>



/** Tick event is used in automatic scan mode to pass the scal clock signal to
    the class that implements automatic scan.
*/
void omObject::Tick() 
{
}

/** Action event is used in automatic and manual scan mode to tell to the object
    that the used selected ACTION function using keyboard, serial or other device. <br>
    The action function is used to select the current button in manual navigation mode
    and to select current scan block or current button in automatic scan mode.
*/
void omObject::Action()
{
}

/** GoLeft event is used in manual scan mode to tell to the object that the user
    selected LEFT function using keyboard, serial or other device. <br>
 */
void omObject::GoLeft()
{
}

/** GoRight event is used in manual scan mode to tell to the object that the user
    selected RIGHT function using keyboard, serial or other device. <br>
 */
void omObject::GoRight()
{
}

/** GoUp event is used in manual scan mode to tell to the object that the user
    selected UP function using keyboard, serial or other device. <br>
 */
void omObject::GoUp()
{
}

/** GoDown event is used in manual scan mode to tell to the object that the user
    selected DOWN function using keyboard, serial or other device. <br>
 */
void omObject::GoDown()
{
}

/** Start event tells to the object to Start all the eventual timers used by the object. 
*/
void omObject::Start()
{
}

/** Stop event tells to the object to Stop all the eventual timers used by the object. 
*/

void omObject::Stop()
{
}

/** IsYourWindow event is a request to the object that uses a window on the screen.
    This request is used to find out if the specified window is handled by the object.
    If so metohd return TRUE otherwhise return FALSE value.

    @return true or false
*/
bool omObject::IsYourWindow(void *)
{
}

/** The Init event is passed to initialize the class after the constructions. <br>
    In some cases you need to initizalize the class after the constructor call because
    the initialization needs that some objects are instantiated.
*/
void omObject::Init() 
{
}

/** This method causes the reload of the options of the class.
    If the class has a timer associated this method enforce the stopping and the starting of
    the timer.
*/
void omObject::Reload() 
{
}


/** Notify a message to the object. <br>
    This event is used to notify a message coming from the TCP/IP network or from 
    another object.
    The message is formed by a command and some parameters.
    The command is passed via <b>command</b> variablle and the parameters are passed via 
    parameters variable separated by MAIASEPARATOR ($$$$$) symbol.

    @param command command to notify
    @param parameters parameters to notify

    @see maiaTCPMessages.h
*/
void omObject::NotifyMessage(wxString command, wxArrayString parameters)
{
}

/** Notify a generic coordinate data to the object. <br>
    This event is used when you need to pass some coordinate data to che object.
    For example some OpenMAIA device drivers pass the state of a pointer using 
    NotifyXYData event.

    @param x x coodrinate
    @param y y coordinate
*/
void omObject::NotifyXYData(int x, int y)
{
}

/** Notify a generic integer data to the object. <br>
    This event is used when you need to pass a generic integer value to the class.

    @param d integer data
 */
void omObject::NotifyIntData(int d)
{
}

/** Notify a generic float data to the object.<br>
    This event is used when you need to pass a generic float value to the class.

    @param d fload data
 */
void omObject::NotifyFloatData(float d)
{
}


/** Notify the StopExecutor event to the object.<br>
    This causes the object that uses an OpenMAIA executor to stop it.
*/
void omObject::StopExecutor()
{
}

/** Notify the StartExecutor event to the object.<br>
    This causes the object that uses an OpenMAIA executor to start it.
 */
void omObject::StartExecutor()
{
}


/** Notify an incoming message from TCP stream.
    @param message Received message.
*/
void omObject::NotifyTCPMessage( omTCPMessage message)
{
}

/** Notify that the action button has been pressed */
void omObject::ActionButtonDown()
{
}

/** Notify that the action button has been released */
void omObject::ActionButtonUp()
{
}

/** Notify that the left button has been pressed */
void omObject::LeftButtonDown()
{
}

/** Notify that the left button has been released */
void omObject::LeftButtonUp()
{
}

/** Notify that the right button has been pressed */
void omObject::RightButtonDown()
{
}

/** Notify that the right button has been released */
void omObject::RightButtonUp()
{
}

/** Notify that the up button has been pressed */
void omObject::UpButtonDown()
{
}
/** Notify that the up button has been released */
void omObject::UpButtonUp()
{
}
 
/** Notify that the down button has been pressed */
void omObject::DownButtonDown()
{
}

/** Notify that the down button has been released */
void omObject::DownButtonUp()
{
}
