////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omOptions.h>
 
#include <wx/wx.h>


// Costructor for testing purposes
omOptions::omOptions()
{
  hostName = _T("localhost");
  port = 60201;
}


/** Initialize the class.
    This function is called after that all objects are istantiated
*/
void omOptions::Init()
{
}


/** Set a parameter value.
    @param name Parameter Name
    @param value Parameter value.

    @return true if this class has a parameter called 'name', false if not.
*/
bool omOptions::SetParameter(wxString name, wxString value)
{
  // --------------------------------------------- hostName --
  if ( name.CompareTo(_T("hostName")) == 0)
    {
      this->hostName = value;
      return true;
    }

  // --------------------------------------------- port  -------
  if ( name.CompareTo(_T("port")) == 0)
    {
      if (omFunctions::IsInt(value))
	this->port = omFunctions::String2Int(value);
      
      return true;
    }

  // if arrive here parameter was not found so return false
  return false;

}


/** Get value of a parameter specified by its name.
    @param name Name of the parameter.

    @return value of the parameter as wxString
*/
wxString omOptions::GetParameter(wxString name)
{
  wxString tmpString = _T("") ;
  // ------------------------------------------------ hostName --
  if (name.CompareTo(_T("hostName")) ==0)
    {
      return hostName;
    }

  // ------------------------------------------------ port -------
  if (name.CompareTo(_T("port")) ==0)
    {
      tmpString << port ;
      return tmpString;
    }

  // parameter was not found so return an empty string
  return _T("");

}


/** Generate a string containting an XML rapresantation of the options
    stored in the structure.
    
    @return The XML String
*/
wxString omOptions::GenerateOptionsXMLString()
{
  // open xml file
  wxString xmlString = omXMLParser::GetXMLOptionsHeader();


  xmlString << _T("<genericOptions>\n") ;
  
  xmlString << _T("\t<option name=\"hostName\" value=\"") << hostName << _T("\">\n");
  xmlString << _T("\t<option name=\"port\" value=\"") << port << _T("\">\n");

  xmlString << _T("<genericOptions>\n\n") ;

  return xmlString;
}
