////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omSmsManager.h>


/**
   Constructor
 */
omSmsManager::omSmsManager() {
  device_init=0;
};

/** Destructor.
    Close the device.
*/
omSmsManager::~omSmsManager() {
  close(device);
 };




/**
 * Autodetecting device
 *   try to get the device checking /dev/ttyS_ and sending ATZ
 */
wxString omSmsManager::AutoDetect(){
  const wxString prefix=wxString("/dev/ttyS");
  int x=12;
  wxString autoport=wxString();
  bool found=false;
  
  do{
    autoport.clear();
    autoport<<prefix<<x;
    x--;
    found=InitDevice(autoport);
  }while(!found && x!=0);

  CloseDevice();

  if(!found){
    std::cout<<"No device found... Sorry!"<<std::endl;
    return(wxString(""));
  }

  std::cout<<"Device found: "<<autoport<<std::endl;

  return(autoport);

};





/**
 * Initialize the device
 *	Opens serial connection on cellular phone
 *	return TRUE if open OK FALSE if error
 * 
 * @param	port 
 * @param	bouds 
 *
 * @return	Return a boolean integer value.
 *
 */
bool omSmsManager::InitDevice(wxString port, int bouds) {
  int n;//data byte
  char buf[1000] ;
  struct termios s_iniziale, tio, tmp ;
  int count=0;


  //Open the device
  std::cout<<"Initializing device "<<port<<" at speed "<<bouds<<std::endl;
  std::cout<<" Wait...";

  wxString process=wxString("stty -F ");
  process<<port<<" "<<bouds;
  ::wxExecute(process);
  usleep(5);

  device = open( port , O_RDWR |  O_NDELAY) ;
  if (device < 0){
    std::cout<<"Failed! ErrNo:"<<device<<std::endl;
    CloseDevice();
    return(false);	/* error */
  }
  else{
    /* disable O_NDELAY flag */
    n = fcntl(device, F_GETFL, 0 ) ;
    //    (void) fcntl(device, F_SETFL, n & ~O_NDELAY) ;
    std::cout<<"Success."<<std::endl;

  }

  /* save modem initial state */
  tcgetattr(device, &s_iniziale) ;
  tcgetattr(device, &tio) ;

  /* inizializza i flag della porta*/
  cfsetospeed(&tio, (speed_t) bouds) ;
  cfsetispeed(&tio, (speed_t) bouds) ;
  tio.c_cflag = ( tio.c_cflag & ~CSIZE) | CS8 ;
  tio.c_iflag = IGNBRK ;
  tio.c_lflag = 0 ;
  tio.c_oflag = 0 ;
  tio.c_cflag |= CLOCAL | CREAD ;
  tio.c_cflag &= ~CRTSCTS ;
  tio.c_cc[VMIN] = 1 ;
  tio.c_cc[VTIME] = 1 ;
  tio.c_iflag &= ~(IXON | IXOFF| IXANY) ;
  tio.c_cflag &= ~(PARENB | PARODD) ;
  tio.c_cflag &= ~CSTOPB ;

  tcsetattr( device, TCSANOW, &tio) ;

  /* toggle line to initialize modem */
  tcgetattr(device, &tmp) ;	/* salva in tmp le impostazioni attuali */
  cfsetospeed(&tmp, B0);
  cfsetispeed(&tmp, B0);
  tcsetattr(device, TCSANOW, &tmp);
  sleep(1);
  tcsetattr(device, TCSAFLUSH, &tio); // reinposta le configurazioni precedenti

  std::cout<<"Initializing modem \n Wait..."<<std::endl;
  /* try to read another time modem answer */
  WriteString("~^M~ATZ^M~", 0);
  count=0;
  do {
    n=read(device,buf,999) ;
    usleep(1);
    if(n!=-1)
      std::cout<<buf<<std::endl;
    count++;
  } while( count<MAX_RETRY_NUMBER && n<0 && !wxString(buf).Contains(_T("OK")) && !wxString(buf).Contains(_T("ERROR")) );

  if (wxString(buf).Contains(_T("OK"))){
    std::cout<<"Success."<<std::endl;

    device_init=1;	// Device initializzation success

    return(true) ;	/* no error */
  }
  else{
    std::cout<<"ERROR to initialize the device: "<<port<<std::endl;
    CloseDevice();
    return(false);
  }

};




int omSmsManager::CloseDevice() {
  std::cout<<"Device close"<<std::endl;
  return(close(device));
};





/**
 * Write a char to the modem.
 */
int omSmsManager::WriteChar( char c, char *answ, int lansw) {
	int i = 0 ;
	int count=0;

	write(device, &c, 1) ;	/* send char to the modem*/



 // Without echo test
	//i = read(device, answ, lansw) ; //sometime is needed :-|
	*answ=c;
	*(answ+1)='\0';
	return(1);
/**/

/* //With ECHO test
	do {
	  i = read(device, answ, lansw) ;	//read from modem 
	  usleep(1);
	  count++;
//  std::cout<<"  "<<c<<" "<<*answ<<" "<<i<<std::endl;
	} while (i==-1 && count<MAX_RETRY_NUMBER) ;
	
	if (count>=MAX_RETRY_NUMBER){
	  return(-1);
	}

	return(i) ;
/**/
}




/**
 * Write a string to the modem.
 * If how == 0, '~'  sleeps 1 second.
 * If how == 1, "^~" sleeps 1 second.
 */
int omSmsManager::WriteString(char *s, int how)
{
char *dial_user;
char *dial_pass;

  char c;

  while(*s) {
  	if (*s == '^' && (*(s + 1))) {
  		s++;
  		if (*s == '^')
  			c = *s;
  		else if (how == 1 && *s == '~') {
			sleep(1);
			s++;
			continue;
		} else
  			c = (*s) & 31;
	} else if (*s == '\\' && (*(s + 1))) {
		s++;
		switch (toupper (*s)) {
			case '\\':
			  c = *s;
			  break;
			case 'U':
			  if (dial_user && *dial_user)
			    WriteString(dial_user, how);
			  s++;
			  continue;
			case 'P':
			  if (dial_pass && *dial_pass)
			    WriteString(dial_pass, how);
			  s++;
			  continue;
			default:
			  s++;
			  continue;
		}
  	} else
  		c = *s;
	if (how == 0 && c == '~')
  		sleep(1);
  	else
  		write(device, &c, 1);
  	s++;
  }
}




/**
 *Send a string to phone
 * tmout - timeout in milliseconds
 
 *return TRUE if the modem respons correctly FALSE elsewhere
 */
int omSmsManager::SendString( char *dom, char *ris) {
	char bd[1000],br[1000], buf[1000], *b ;
	int nd,nr,n, i, r, l ;

	bzero(buf,1000) ;	/* azzera il buffer */

	nd = strlen(dom)  ;
	nr = strlen(ris)  ;

	strncpy(bd,dom,nd) ;
	strncpy(br,ris,nr) ;

	bd[nd]='\r' ;		/* inserisce CR per terminare la riga */
	b = bd ;

	//sleep(SLEEPTIME) ;
	sleep(1);
	//try to send each character to modem
	for ( i=0; i<=nd; i++ ) {
		l = WriteChar( *b, buf, 999) ;
		usleep(SLEEPTIME) ;

		std::cout<<buf<<std::endl;;
		b++ ;	/* next char in the query buffer */
		if (l==-1)
		  return(FALSE);
	}


	i = 0 ;
	n = 0 ;
	while( i<MAX_RETRY_NUMBER ) {
		i++ ;		// tries counter 
		n = read(device, buf, 999) ;
		usleep(SLEEPTIME);

		buf[n] = '\0'; 
//if(n>0)
//  std::cout<<i<<"   "<<buf<<std::endl;

		if (wxString(buf).Contains(wxString(ris))){
		  std::cout<<buf<<std::endl;
		  return(TRUE);
		}
		if ( wxString(buf).Contains(_T("ERROR")) ){
		  std::cout<<buf<<std::endl;
		  return(FALSE);
		}
		if (i > MAX_RETRY_NUMBER){
		  std::cout<<"TIMEOUT!"<<std::endl;
		  return(FALSE);
		}
	}



	usleep(1000);

	/* try to read another time modem answer */

std::cout<<std::endl<<"Waiting for modem response..."<<std::endl;
	i = 0 ;
	n = 0 ;
	while( i<MAX_RETRY_NUMBER ) {
		i++ ;		// tries counter 
		n = read(device, buf, 999) ;
		usleep(SLEEPTIME);

		buf[n] = '\0'; 
//if(n>0)
//  std::cout<<i<<"   "<<buf<<std::endl;
		if ( n > nr ) {
		  buf[n] = 0; 
		}

		if (wxString(buf).Contains(wxString(ris))){
		  std::cout<<buf<<std::endl;
		  return(TRUE);
		}
		if ( wxString(buf).Contains(_T("ERROR")) ){
		  std::cout<<buf<<std::endl;
		  return(FALSE);
		}
		if (i > MAX_RETRY_NUMBER){
		  std::cout<<"TIMEOUT!"<<std::endl;
		  return(FALSE);
		}
	}
	std::cout<<"UNDEFINED BEHAVIOR:\n "<<buf<<std::endl;
	return(FALSE);

}








int omSmsManager::QueryPhone(char *qy, char *ris, char *answ) {
	char bd[1000],br[1000], buf[MAX_READMESS_BUFF_LENGHT], *b ;
	int nd,nr,n, i, r, l ;

	bzero(buf,MAX_READMESS_BUFF_LENGHT/2) ;	/* azzera il buffer */

	nd = strlen(qy)  ;
	nr = strlen(ris)  ;

	strncpy(bd,qy,nd) ;
	strncpy(br,ris,nr) ;

	bd[nd]='\r' ;		/* inserisce CR per terminare la riga */
	b = bd ;

	for ( i=0; i<=nd; i++ ) {
		l = WriteChar( *b, buf, MAX_READMESS_BUFF_LENGHT) ;

std::cout<<buf<<std::endl;;

		b++ ;	/* next char in the query buffer */
		if (l==-1)
		  return(FALSE);
	}

	sleep(SLEEPTIME) ;

	if (wxString(buf).Contains(_T("ERROR"))) 
	  return(FALSE);


	/* now buf contains the answer -- I think!! */
	if (wxString(buf).Contains(wxString(ris))){
	  strncpy(answ,buf,MAX_READMESS_BUFF_LENGHT);
	  return(TRUE) ;		/* answer found */
	}

	/* try to read another time modem answer */
std::cout<<std::endl<<"Waiting for modem response..."<<std::endl;
	i = 0 ;
	n = 0 ;
	while( n == 0 ) {
		i++ ;		// tries counter 
		n = read(device, buf, MAX_READMESS_BUFF_LENGHT) ;
std::cout<<wxString(buf)<<std::endl;
                if (wxString(buf).Contains(_T("ERROR")) ) 
		  return(FALSE);
		if (wxString(buf).Contains(wxString(ris)))
		  break;
		if (i > MAX_RETRY_NUMBER)
		  return(FALSE) ;
	}

	if ( i < MAX_RETRY_NUMBER){
	  strncpy(answ,buf,MAX_READMESS_BUFF_LENGHT);
	  return(TRUE) ;
	}
	else
	  return(FALSE) ;

}






/**
 * Get the port value 
 * @return the value of port
 */
wxString omSmsManager::GetDevice ( ) {
	return port;
}




/**
 * Send an SMS to the phone
 * @return	int
 * @param	phone_number 
 * @param	message 
 */
int omSmsManager::SendSMS(wxString phone_number, wxString message ) {
  wxString number, body;

  std::cout<<"Try to sending message."<<std::endl;
  std::cout<<"  Please wait..."<<std::endl;

  number=_T("AT+CMGS=\"")+phone_number+_T("\"");

  body=message;
  body<<(char) 26;	//CTRL-Z to terminate message body

  if( !SendString(_T("AT+CMGF=1"),_T("OK"))){	//text message format
    std::cout<<"ERROR setting TEXT MODE!!!"<<std::endl;
    //    return(FALSE);
  }

  if( !SendString((char *)number.c_str(),_T(">"))){  //command + recipient number

    std::cout<<"ERROR sending SMS!!! (setting phone number)"<<std::endl;
    return(FALSE);
  }

  if( !SendString((char *)body.c_str(),_T("OK"))){	// body of the message
    std::cout<<"ERROR sending SMS!!! (message body)"<<std::endl;
    return(FALSE);
  }

  std::cout<<"Message sent correctly. ;-)"<<std::endl;


  return(TRUE) ;	/* no problem in sending message */

}




omSms *omSmsManager::GetSmsHeader(wxString buffer){
  omSms *message;

  message=new omSms();

  //Get READ or UNREAD
  buffer.Mid(buffer.Find("+CMGL:")+6,2).ToLong(&(message->index));
  buffer=buffer.Mid(buffer.Find(_T(","))+1);
  if (buffer.Find(_T("REC UNREAD"))>0)
    message->unread=1;
  else
    message->unread=0;

  //Get Sender
  buffer=buffer.Mid(buffer.Find(_T(","))+2);
  message->sender=wxString(buffer.Mid(0,buffer.Find(_T("\""))));

  //Get Timestamp
  buffer=buffer.Mid(buffer.Find(_T(","))+1);
  buffer=buffer.Mid(buffer.Find(_T(","))+2);
  message->timestamp=wxString(buffer.Mid(0,buffer.Find(_T("\""))));


  return(message);
}






/**
 * Receive an SMS from the phone
 * 
 * @param	messages Pointer to message structure.
 *              smstype  type of messages to retrive
 *                 UNREAD
 *                 READ
 *                 UNSENT
 *                 SENT
 *
 * @return	Number of received messages.
 */
int omSmsManager::ReceiveMessages( omSmsArray *messages, wxString smstype ) {
  char buf[MAX_READMESS_BUFF_LENGHT];
  int n;
  wxString wxbuf,first_mes;
  omSms *received_mes;


  if( !SendString("AT+CMGF=1","OK")){	//text message format
    std::cout<<"ERROR setting TEXT MODE!!!"<<std::endl;
    return(FALSE);
  }


std::cout<<"-----------------Retriving message from phone--------------------------------"<<std::endl;
/*
 * CMGL=0 UNREAD
 * CMGL=1 READ
 * CMGL=2 UNSENT
 * CMGL=3 SENT
 */
 smstype=smstype.Upper();
 wxString query=wxString("AT+CMGL=\"REC ")<<smstype<<wxString("\"");
 n = QueryPhone((char*)query.c_str(),"OK",buf) ;
  if ( n == 0 )
    return(-1);
std::cout<<"-------------------------------------------------"<<std::endl;

  wxbuf=wxString(buf);
  n=0;


  while(wxbuf.Contains(_T("+CMGL:"))){
//std::cout<<"-------------------------------------------------"<<std::endl;
    wxbuf=wxbuf.Mid(wxbuf.Find(_T("+CMGL:")));//find first message

    //Extract a message header from buffer
    first_mes=wxbuf.Mid(0,wxbuf.Find("\n"));
    wxbuf=wxbuf.Mid(wxbuf.Find("\n")+1);

//std::cout<<first_mes<<std::endl;
    received_mes=GetSmsHeader(first_mes);
    if (wxbuf.Find(_T("+CMGL:"))>-1){
      received_mes->body=wxbuf.Mid(0,wxbuf.Find(_T("+CMGL:")));
      wxbuf=wxbuf.Mid(wxbuf.Find("+CMGL:"));
    }
    else{
      received_mes->body=wxbuf.Mid(0,wxbuf.Find(_T("\nOK")));
      wxbuf=wxbuf.Mid(wxbuf.Find("\nOK"));
    }
//std::cout<<received_mes->index<<": "<<received_mes->sender<<" "<<received_mes->body<<std::endl;


    messages->Add(received_mes);
    n++;

  }

  return(n);
}



/**
 * Receive an SMS from the phone
 * 
 * @param	num index number of the message to delete
 *
 * @return	true if message was deleted correctly
 */
int omSmsManager::DeleteMessage(int num) {
  wxString query=wxString(_T("AT+CMGD="))<<num;

  if( !SendString((char*)query.c_str(),"OK")){	//text message format
    std::cout<<"ERROR deleting message!!!"<<std::endl;
    return(FALSE);
  }
  return(TRUE);
}
