////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omTCPMessage.h>

/** Initialize the class.
*/
omTCPMessage::omTCPMessage()
{
  fromAgent = _T("");
  fromService = _T("");
  fromHost = _T("");
  toAgent = _T("");
  toService = _T("");
  toHost = _T("");
  command = _T("");
}

/** Get a stored parameter.
    @param idx index of the parameter to retrive
    @return string containing the parameter.
*/
wxString omTCPMessage::GetParameter( int idx)
{
  if ( idx < parameters.GetCount())
    return parameters[idx] ;
  else
    return (_T("")) ;
}

/** Get the number of stored parameters.
    @return parameters number.
*/
int omTCPMessage::GetParCount()
{
  return parameters.GetCount();
}


/** Set parameters
    @param p string containing the parameters
*/
void omTCPMessage::AddParameter(wxString p)
{
  parameters.Add(p) ;
}

/** Return message formatted to be sent via TCP/IP

    @return string containing formatted message
*/
wxString omTCPMessage::GetFormattedMessage()
{
  int i;
  wxString mess = _T("$$$$\nmessage\n") ;
  if (fromAgent.CmpNoCase(_T("")) != 0)
    mess << _T("FROM_AGENT:") << fromAgent << _T("\n");
  if (fromService.CmpNoCase(_T("")) != 0)
    mess << _T("FROM_SERVICE:") << fromService << _T("\n");
  if (fromHost.CmpNoCase(_T("")) != 0)
    mess << _T("FROM_HOST:") << fromHost << _T("\n");
  if (toAgent.CmpNoCase(_T("")) != 0)
    mess << _T("TO_AGENT:") << toAgent << _T("\n");
  if (toService.CmpNoCase(_T("")) != 0)
    mess << _T("TO_SERVICE:") << toService << _T("\n");
  if (toHost.CmpNoCase(_T("")) != 0)
    mess << _T("TO_HOST:") << toHost << _T("\n");
  if (command.CmpNoCase(_T("")) != 0)
    mess << _T("COMMAND:") << command << _T("\n");

  if (this->GetParCount()>0)
    mess << _T("PARAMETERS:") ;

  for (i=0; i<this->GetParCount();i++)
    {
      mess << omFunctions::Encode(parameters[i]) ;
      if ( i < this->GetParCount()-1)
	mess << _T(";") ;
      else
	mess << _T("\n") ;
    }

  mess << _T("####\n") ;
  return mess;
}

/** Return message command part.
 */
wxString omTCPMessage::GetCommand()
{
  return command;
}
