////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omTCPParser.h>

/** Build omTCPParser object initializing variables */
omTCPParser::omTCPParser()
{
  messageReceived=false;
  receivingMessage=false;
  registerMessage=false;
  genericMessage=false;

  receivingEscape=false;
  receivingRegister=false;
  receivingGeneric=false;
  messageStarting=false;
  messageEnding=false;
  messageStarted=false;
  messageEnded=false;
  charCounter=0;
  partialMessage=_T("");

}

/** Analize a character coming from TCP/IP connection.
    This method compose a message and parses it.
    @param c character to analiza
    
    @return OM_NO_MESSAGE if message is not completed,
      OM_REGISTER_MESSAGE if register message is completed,
      OM_GENERIC_MESSAGE if generic message is compleded.
      
    @see class documentation.
*/
int omTCPParser::AnalizeChar(char c)
{
  if (messageEnded)
    return OM_NO_MESSAGE;

  // check if message is already completed and then probably this
  // is a new message


//   if ( messageReceived )
//     {
//       // reset initial status
//       messageReceived = false;
//       receivingEscape=false;
//       receivingRegister=false;
//       receivingGeneric=false;
//       messageStarting=false;
//       messageEnding=false;
//       messageStarted=false;
//       messageEnded=false;
//       partialMessage = _T("");
//       genericMessageData = omTCPMessage();
//       agentName=_T("");
//       serviceName=_T("");
//       registerMessage=false;
//       genericMessage=false;
//       genericMessageData=omTCPMessage();
//     }



  // --------------------------------- check for message starting -------
  // message start with $$$$ symbol
  if (c=='$' && !receivingEscape && !messageStarted && !messageStarting)
    {
      messageStarting = true;
      charCounter = 1;
    }	
  // wrong char during starting of the message
  if (c!='$' && !receivingEscape  && !messageStarted && messageStarting)
    {
      // reset waiting status
      messageStarting = false;
      charCounter = 0;
    }

  // receiving 4 dollars indicating start of message
  if (c=='$' && !receivingEscape  && !messageStarted && messageStarting)
    {
      charCounter++;
    }
  // test if message is started
  if (messageStarting && charCounter == 4)
    {
      // the message is started
      messageStarting = false;
      messageStarted = true;
      charCounter=0;
      partialMessage=(_T(""));
    }

  // --------------------------------- check for message ending -------
  // message start with #### symbol
  if (c=='#' && !receivingEscape && !messageEnding)
    {
      messageEnding = true;
      charCounter = 1;
    }
  // wrong char during ending of the message
  if (c!='#' && !receivingEscape  && messageEnding)
    {
      // reset waiting status
      messageEnding = false;
      charCounter = 0;
    }

  // receiving 4 numbers indicating start of message
  if (c=='#' && !receivingEscape  && messageEnding)
    {
      charCounter++;
    }
  // test if message is started
  if (messageEnding && charCounter == 4)
    {
      // end of message reached
      messageEnded = true;
      messageEnding = false;
      charCounter = 0;
    }
  

  // if message ended fill data buffers and exit
  if ( messageEnded ) // ---------------------------------------------------------- END OF MESSAGE --
    {
#ifdef MAIA_DEBUG_VERBOSE
      if (registerMessage)
	{
	  std::cout << "omTCPParser::AnalizeChar- Agent Registered" << std::endl <<
	    "\tagentName=" << agentName << "." << std::endl <<
	    "\tserviceName=" << serviceName << std::endl;
	}
      if (genericMessage)
	{
	  std::cout << "omTCPParser::AnalizeChar- Agent Message" << std::endl <<
	    "\tfromAgent=" << genericMessageData.fromAgent << std::endl <<
	    "\tfromService=" << genericMessageData.fromService << std::endl <<
	    "\tfromHost=" << genericMessageData.fromHost << std::endl <<
	    "\ttoAgent=" << genericMessageData.toAgent << std::endl <<
	    "\ttoService=" << genericMessageData.toService << std::endl <<	
	    "\ttoHost=" << genericMessageData.toHost << std::endl <<
	    "\tcommand=" << genericMessageData.command << std::endl;
	  if (genericMessageData.GetParCount())
	    {
	      int i;
	      for (i=0;i<genericMessageData.GetParCount();i++)
		std::cout << "\tparameter " << i << "=" << genericMessageData.GetParameter(i) << std::endl;
	    }
	}
#endif
      ResetInitialStatus();
      // return a value
      if (registerMessage)
	return OM_REGISTER_MESSAGE;
      if (genericMessage)
	return OM_GENERIC_MESSAGE;

    } // ---------------------------------------------------------------------------------------------

  // add received char to partial message
  partialMessage << c;

  // analize the line if reached end of line
  if ( c == '\n')   // --------------------------------------------------------- PARSE THE LINE ----
    {
      wxString partialMessageUp=partialMessage;
      partialMessageUp = partialMessageUp.MakeUpper();   // uppercase of the line
      // check if registration or generic message
      if ( !registerMessage && !genericMessage )
	{
	  if (partialMessageUp.Find(_T("REGISTER")) != -1)
	    {
	      registerMessage=true;
	      partialMessage=(_T(""));
	    }

	  if (partialMessageUp.Find(_T("MESSAGE")) != -1)
	    {
	      genericMessage=true;
	      partialMessage=(_T(""));
	    }
	}

      // --------------------------------------------------------- REGISTRATION MESS --
      if (registerMessage)
	{
	  if (partialMessageUp.Find(_T("AGENT"))!=-1)	// agent name
	    {
	      // find value
	      GetParameterValue(partialMessage, &agentName);
	      // reset partial message buffer
	      partialMessage=_T("");
	    }

	  if (partialMessageUp.Find(_T("SERVICE"))!=-1)	// service name
	    {
	      // find value
	      GetParameterValue(partialMessage, &serviceName);
	      // reset partial message buffer
	      partialMessage=_T("");

	    }
  
	}
      // --------------------------------------------------------------- GENERIC MESS --
      if (genericMessage)
	{
	  if (partialMessageUp.Find(_T("FROM_AGENT"))!=-1)	// from agent
	    {
	      // find value
	      GetParameterValue(partialMessage, &(genericMessageData.fromAgent));
	      // reset partial message buffer
	      partialMessage=_T("");
	    }
	  
	  if (partialMessageUp.Find(_T("FROM_SERVICE"))!=-1)	// from service
	    {
	      // find value
	      GetParameterValue(partialMessage, &(genericMessageData.fromService));
	      // reset partial message buffer
	      partialMessage=_T("");
	    }

	  if (partialMessageUp.Find(_T("FROM_HOST"))!=-1)		// from ip
	    {
	      // find value
	      GetParameterValue(partialMessage, &(genericMessageData.fromHost));
	      // reset partial message buffer
	      partialMessage=_T("");
	    }

	  if (partialMessageUp.Find(_T("TO_AGENT"))!=-1)	// to agent
	    {
	      // find value
	      GetParameterValue(partialMessage, &(genericMessageData.toAgent));
	      // reset partial message buffer
	      partialMessage=_T("");
	    }

	  if (partialMessageUp.Find(_T("TO_SERVICE"))!=-1)	// to service
	    {
	      // find value
	      GetParameterValue(partialMessage, &(genericMessageData.toService));
	      // reset partial message buffer
	      partialMessage=_T("");
	    }

	  if (partialMessageUp.Find(_T("TO_HOST"))!=-1)		// to ip
	    {
	      // find value
	      GetParameterValue(partialMessage, &(genericMessageData.toHost));
	      // reset partial message buffer
	      partialMessage=_T("");
	    }

	  if (partialMessageUp.Find(_T("COMMAND"))!=-1)		// from agent
	    {
	      // find value
	      GetParameterValue(partialMessage, &(genericMessageData.command));
	      // reset partial message buffer
	      partialMessage=_T("");
	    }

	  if (partialMessageUp.Find(_T("PARAMETERS"))!=-1)    	// from agent
	    {
	      wxArrayString arrayBuffer;
	      int i,n;
	      // find value
	      n= GetDecodedParameters(partialMessage, &(arrayBuffer));
	      // store parameters into genericMessageData
	      for (i=0;i<n;i++)
		genericMessageData.AddParameter(arrayBuffer[i]);

	      // reset partial message buffer
	      partialMessage=_T("");
	    }
	}
    }
  


  return OM_NO_MESSAGE;
}


/**Get register message data.
   @param agent pointer to an agent name buffer.
   @param service pointer to a service name buffer.

   @return true if have register message information, false if not.
*/
bool omTCPParser::GetRegisterMessage(wxString *agent, wxString *service)
{
  if (!registerMessage)
    return false;

  *(agent) = this->agentName;
  *(service) = this->serviceName;
  messageEnded = false;
  agentName=_T("");
  serviceName=_T("");
  return true;
}

/**Get generic message data.
   @param message pointer to message buffer

   @return true if have generic message information, false if not
*/
bool omTCPParser::GetGenericMessage( omTCPMessage *message)
{      
  if (!genericMessage)
    return false;

  message->fromAgent=genericMessageData.fromAgent;
  message->fromService=genericMessageData.fromService;
  message->fromHost=genericMessageData.fromHost;
  message->toAgent=genericMessageData.toAgent;
  message->toService=genericMessageData.toService;
  message->toHost=genericMessageData.toHost;
  message->command=genericMessageData.command;

  // copy parameters
  int i;
  for (i=0;i<genericMessageData.GetParCount();i++)
    message->AddParameter(genericMessageData.GetParameter(i));

  messageEnded=false;
  genericMessageData=omTCPMessage();

  return true;
}





// ------------------------------------------------------------------- PRIVATE METHODS --

/// Find a parameter in a line of the TCP message
/// @param line line of the message
/// @param parValue buffer to insert parameter value
///
/// @return true if the line is correct, false if incorrect
bool omTCPParser::GetParameterValue(wxString line, wxString *parValue)
{
  int startpos, endpos;	

  // search separator char
  startpos=line.Find(_T(":"));

  // find first not blank char
  startpos=omFunctions::FindFirstNBChar(line,startpos+1);
  // find last not blank char
  endpos=omFunctions::FindLastNBChar(line);
  if ( startpos == -1 || endpos == -1)
    {
#ifdef MAIA_DEBUG
      std::cout << "omTCPParser::GetParameterValue- Error in message line" <<std::endl
		<< "\t" << line << std::endl;
#endif
      return false;
    }
  *parValue = line.SubString(startpos, endpos);
}



/// Find a list of parameters in a line of the TCP message, decode them and 
/// fill a wxArrayString strcuture.
/// @param line line of the message
/// @param parsValue array buffer in wich insert parameters
///
/// @return number of inserted parameters
int omTCPParser::GetDecodedParameters(wxString line, wxArrayString *parsValue)
{
  int parsCount = 0;
  wxString lineRest;   		// tailed string containing the part of the
                                // line to process
  wxString curParBuffer	;  	// temporary buffer
  int startpos, endpos;
  
  // tail fixed part of the string
  startpos=line.Find(_T(":"));
  startpos=omFunctions::FindFirstNBChar(line,startpos);
  if (startpos == -1)   // no parameters found
    return 0;


  lineRest=line.SubString(startpos+1,line.Length());
  
  // check parameters existence
  while(1)
    {
      startpos=0;
      // find separator or end of line
      //endpos=tmpString.Find(_T(";"));

      endpos=omFunctions::FindNotEncodedChar(lineRest, ';', 0);
      
      parsCount++;

      if (endpos==-1)   // reached end of the line
	break;
      
      // isolate single parameter
      curParBuffer=lineRest.SubString(startpos, endpos-1);
      // tail the parameter and the separator to the line
      lineRest=lineRest.SubString(endpos+1,lineRest.Length());

      //store start position for next cycle
      startpos=endpos;
      // tail white spaces
      endpos=omFunctions::FindLastNBChar(curParBuffer);
      curParBuffer=curParBuffer.SubString(0,endpos);

      // tail current parameter
      //startpos=omFunctions::FindFirstNBChar(lineRest, 0);
      //lineRest=lineRest.SubString(startpos,lineRest.Length());	
      

      // store isolated parameter
      curParBuffer=omFunctions::Decode(curParBuffer);
      parsValue->Add(curParBuffer);
    }

  // at the end of the cycle last parameter must be stored
  startpos=omFunctions::FindFirstNBChar(lineRest, 0);
  endpos=omFunctions::FindLastNBChar(lineRest);
  if (startpos != -1 && endpos!=-1)   // last parameter is not formed only by white spaces
    {
      curParBuffer=lineRest.SubString(startpos,endpos);
      curParBuffer=omFunctions::Decode(curParBuffer);
      parsValue->Add(curParBuffer);
    }
  else	
    parsCount--;    // erase last parameter count

  return parsCount;
}


void omTCPParser::ResetInitialStatus()
{
  messageReceived = false;
  receivingEscape=false;
  receivingRegister=false;
  receivingGeneric=false;
  messageStarting=false;
  messageEnding=false;
  messageStarted=false;
  partialMessage = _T("");
}
