////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omUIActionQueue.h>


/** The constructor. Create an empty list. 
    ATTENTION!! appKeyboard field in inter variable must be initialized to the
    correct value to work properly
    @param tick timer interval for event check in milliseconds
*/
omUIActionQueue::omUIActionQueue(int tick)
{
  l = new std::list<omCommand >() ;   // create an empty list

  timeTick = tick ;
  //  executor = new MaiaExecuteAction() ;
  //wxGetApp().objects.executor = executor ;
}

/** Put an element to the end of list.
    @param c  Maia Command code to add to the back of the queue.
*/
void omUIActionQueue::Put( omCommand c) {
  l->push_back(c) ;
}

/** Get an element from the start of list.
    @param rv pointer the omMessage object that will store the result
    @return The first element of the queue.
*/
bool omUIActionQueue::Get( omCommand *rv) {
  std::list<omCommand >::iterator it ;


  if (l->size() == 0)
    return false;   // NULL Pointer

  it = l->begin() ;  // begin of the list
  it++ ;             // it points to second element

  //MaiaAction rv ; 
  copy(l->begin(), it, rv) ;

  l->pop_front() ;

  return(true) ;
}

/** Start check events */
void omUIActionQueue::Start()
{
  wxTimer::Start(timeTick) ;
}

/** Stop check events */
void omUIActionQueue::Stop()
{
  wxTimer::Stop() ;
}


// ------------ private methods

/** Override the norify method of the wxTimer superclass
    Check for event in the queue and execute them
*/
void omUIActionQueue::Notify()
{
  omCommand a ;
  bool isElements = this->Get(&a) ;

  if ( isElements )
    {
      omUICommandParser *cp = (omUICommandParser *) wxGetApp().objects.commandParser ;
      cp->ParseAction(a) ;
    }
}
