////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omUICommandParser.h>

/** Build the class.
    Instantiate the executor and initialize to null the pointer to the virtual keyboard.
*/
omUICommandParser::omUICommandParser()
{
  executor = new omExecutor() ;
  keyboard = NULL;
}

/** Make post construction initialization.
    Initialize virtual keyboard pointer to application virtual keyboard.
*/
void omUICommandParser::Init()
{
  keyboard = (omUIKeyboard *) wxGetApp().objects.keyboard ;
}


/** Parse a message coming from agent's event queue
    @param m message to parse

    @return true if message was parsed, false otherwhise
*/

// -- to-port -- the functions must be ported to the new structure
bool omUICommandParser::ParseAction(omCommand m)
{
  wxString action = m.GetCommand() ;
  wxString pars = m.GetParameter() ;  // every command has a single parameter :)

  bool retval = false ;

  keyboard = (omUIKeyboard *) wxGetApp().objects.keyboard;

//   if (!options->actionOnPress)
//     // -- to-port -- checker->FlushKeysState();
//   else {
//     //this->ReleaseKey(options->actionKey);
//   }


  // -- to-port -- checker->Stop();

//   // retrive informations about serve connector object
//   connector = (MaiaConnector *) wxGetApp().objects.connector ;
//   // send the command to the connector if exists
//   if ( connector != NULL)
//     connector->SendMessage(MaiaMessage(ac,pars));

  //std::transform(ac.begin(), ac.end(), ac.begin(), ToLower()) ;
  action = omFunctions::ConvertToLower(action) ;

  // ---------------------------------------------- EMULATIONS CODES --------

  // -------------------------------------------- Character Emulation ------
  if ( action.CmpNoCase(_T("emulatestring")) == 0 ) 
    {
      executor->RenderString(pars) ;
      return true;
    }

  if ( action.CmpNoCase(_T("enterclick")) == 0) {
    executor->PressKey(ENTERKEY) ;
    executor->ReleaseKey(ENTERKEY) ;
    return true;
  }

  if ( action.CmpNoCase(_T("capslockclick")) == 0) {
    executor->PressKey(CAPSLOCKKEY) ;
    executor->ReleaseKey(CAPSLOCKKEY) ;
    return true;
  }

  if ( action.CmpNoCase(_T("backspaceclick")) == 0) {
    executor->PressKey(BACKSPACEKEY);
    executor->ReleaseKey(BACKSPACEKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("tabclick")) == 0) {
    executor->PressKey(TABKEY);
    executor->ReleaseKey(TABKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("escclick")) == 0) {
    executor->PressKey(ESCKEY);
    executor->ReleaseKey(ESCKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("arrowclick")) == 0) {
    if ( pars == wxString(_T("up"))) 
      {
	executor->PressKey(UPARROWKEY);
	executor->ReleaseKey(UPARROWKEY);
      }

    if ( pars == wxString(_T("down"))) 
      {
	executor->PressKey(DOWNARROWKEY);
	executor->ReleaseKey(DOWNARROWKEY);
      }

    if ( pars == wxString(_T("left"))) 
      {
	executor->PressKey(LEFTARROWKEY);
	executor->ReleaseKey(LEFTARROWKEY);
      }

    if ( pars == wxString(_T("right"))) 
      {
	executor->PressKey(RIGHTARROWKEY);
	executor->ReleaseKey(RIGHTARROWKEY);
      }
    return true ;
  }

  if ( action.CmpNoCase(_T("pageupclick")) == 0) {
    executor->PressKey(PAGEUPKEY);
    executor->ReleaseKey(PAGEUPKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("pagedownclick")) == 0) {
    executor->PressKey(PAGEDOWNKEY);
    executor->ReleaseKey(PAGEDOWNKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("endclick")) == 0) {
    executor->PressKey(ENDKEY);
    executor->ReleaseKey(ENDKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("insclick")) == 0) {
    executor->PressKey(INSKEY);
    executor->ReleaseKey(INSKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("insclick")) == 0) {
    executor->PressKey(INSKEY);
    executor->ReleaseKey(INSKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("delclick")) == 0) {
    executor->PressKey(DELKEY);
    executor->ReleaseKey(DELKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("homeclick")) == 0) {
    executor->PressKey(HOMEKEY);
    executor->ReleaseKey(HOMEKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("controlpress")) == 0) {
    executor->PressKey(CONTROLKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("controlrelease")) == 0) {
    executor->ReleaseKey(CONTROLKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("altpress")) == 0) {
    executor->PressKey(ALTKEY);
    return true ;
  }

  if ( action.CmpNoCase(_T("altrelease")) == 0) {
    executor->ReleaseKey(ALTKEY);
    return true ;
  }


  // -------------------------- function keys emulation -------------------
  if ( action.CmpNoCase(_T("FunctionKey")) == 0) {
    int n = omFunctions::String2Int(pars);
  
    int toEmulate = -1;    // find code of function key toemulate
    
    switch(n)
      {
      case 1:
	toEmulate = F1KEY;
	break;

      case 2:
	toEmulate = F2KEY;
	break;

      case 3:
	toEmulate = F3KEY;
	break;

      case 4:
	toEmulate = F4KEY;
	break;

      case 5:
	toEmulate = F5KEY;
	break;

      case 6:
	toEmulate = F6KEY;
	break;

      case 7:
	toEmulate = F7KEY;
	break;

      case 8:
	toEmulate = F8KEY;
	break;

      case 9:
	toEmulate = F9KEY;
	break;

      case 10:
	toEmulate = F10KEY;
	break;

      case 11:
	toEmulate = F11KEY;
	break;

      case 12:
	toEmulate = F12KEY;
	break;


      case 13:
	toEmulate = F13KEY;
	break;

      case 14:
	toEmulate = F14KEY;
	break;

      case 15:
	toEmulate = F15KEY;
	break;

      case 16:
	toEmulate = F16KEY;
	break;
      }
    executor->PressKey(toEmulate);
    executor->ReleaseKey(toEmulate);
    return true;
  }


  
  

  // some symbols cannot be handled as string because the XML parser
  // fails in reading them.
  // those symbols are codified with this special action and the parameters are
  // code -- symbol
  // lt   -- <
  // gt   -- >
  // dq   -- "
  // a grave -- `a
  if ( action.CmpNoCase(_T("emulatesymbol")) == 0) {
    if ( pars == wxString(_T("lt"))) {
      executor->RenderString(wxString(_T("<")));
      return true;
    }

    if ( pars == wxString(_T("gt"))) {
      executor->RenderString(wxString(_T(">")));
      return true;
    }

    if ( pars == wxString(_T("dq"))) {
      executor->RenderString(wxString(_T("\"")));
      return true;
    }

    if ( pars == wxString(_T("agrave"))) {
      executor->RenderString(wxString(_T("")));
      return true;
    }

    if ( pars == wxString(_T("ograve"))) {
      executor->RenderString(wxString(_T("")));
      return true;
    }

    if ( pars == wxString(_T("ugrave"))) {
      executor->RenderString(wxString(_T("")));
      return true;
    }

    if ( pars == wxString(_T("egrave"))) {
      executor->RenderString(wxString(_T("")));
      return true;
    }

    if ( pars == wxString(_T("eacute"))) {
      executor->RenderString(wxString(_T("")));
      return true;
    }

    if ( pars == wxString(_T("igrave"))) {
      executor->RenderString(wxString(_T("")));
      return true;
    }
  }

  // -------------------------------------------- Mouse Emulation ---------
  // mouse move
  if ( (action.CmpNoCase(_T("movemouseleft")) == 0) || 
       (action.CmpNoCase(_T("movemouseright"))== 0) ||
       (action.CmpNoCase(_T("movemouseup")) == 0) || 
       (action.CmpNoCase(_T("movemousedown")) == 0)) {
    keyboard->Stop() ;
    bool restartKeyboard = executor->MoveMouse( action, pars) ;
       if (restartKeyboard)
    keyboard->Start() ;
    return true;
  }

  // mouse click
  if ( action.CmpNoCase(_T("leftclick")) == 0) {
    executor->ClickMouse(LEFTBUTTON) ;
    return true;
  }

  if ( action.CmpNoCase(_T("centralclick")) == 0) {
    executor->ClickMouse(CENTRALBUTTON) ;
    return true;
  }

  if ( action.CmpNoCase(_T("rightclick")) == 0) {
    executor->ClickMouse(RIGHTBUTTON) ;
    return true;
  }

  // mouse push or pop (handle keyboard scan too)
  if (action.CmpNoCase(_T("leftdragordrop")) == 0) {
    executor->DragOrDropMouse(LEFTBUTTON) ;
    return true;
  }

  if ( action.CmpNoCase(_T("centraldragordrop")) == 0) {
    executor->DragOrDropMouse(CENTRALBUTTON) ;
    return true;
  }

  if ( action.CmpNoCase(_T("rightdragordrop")) == 0){
    executor->DragOrDropMouse(RIGHTBUTTON) ;
    return true;
  }







  // ------------------------------------------------------------
  // --------------------- Execute external application ---------
  // ------------------------------------------------------------

  //Execute application directly
  if ( action.CmpNoCase(_T("ExecuteApplication")) == 0) {
    executor->ExecuteApplication(pars) ;
    return true;
  }

  //Execute application using alias methods
  if ( action.CmpNoCase(_T("ExecuteAliasApplication")) == 0) {
    wxTextFile *aliasFile=new wxTextFile(pars);
    wxString command;

std::cout<<"File: "<<pars<<" -- Exists:"<<aliasFile->Exists()<<std::endl;
    if(aliasFile->Exists()){
      aliasFile->Open();
      
      command=aliasFile->GetFirstLine();
std::cout<<"Command: "<<command<<std::endl;
      executor->ExecuteApplication(command) ;

      while (!aliasFile->Eof()){
	command=aliasFile->GetNextLine();
std::cout<<"Command: "<<command<<std::endl;
	executor->ExecuteApplication(command) ;
      }

      aliasFile->Close();
      return true;
    }
    return false;
  }







//   if (options->actionOnPress)
//     //this->PressKey(options->actionKey);
//     this->ReleaseKey(options->actionKey);

// -- to-port -- checker->Start();

  return false ;
}


/// to implement
void omUICommandParser::EndParsing()
{
  // do nothing
}

/** Start the executor.
    @todo implement TCP/IP message sending
*/
void omUICommandParser::StartExecutor()
{
  if (executor != NULL)
    executor->Start() ;
}

/** Stop the executor.
    @todo implement TCP/IP message sending
*/
void omUICommandParser::StopExecutor()
{
  if (executor != NULL)
    executor->Stop() ;
}

