////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#include <openmaia/omUIKeyboard.h>

#include <unistd.h>

/** Construct the class by inizializating the XML parser.
    This class need gnome libxml2 library and headers.
*/
omUIKeyboard::omUIKeyboard()
  : omXMLParser()
{
 
  // initialize pointers to NULL
  rootNode = NULL ;
  currentNode = NULL ;

 

  // keyboard initially is not active
  active = false ;
 
}

/** Initialize the class.
    This function is called after that all objects are istantiated
*/
void omUIKeyboard::Init()
{
  opts = (omUIOptions *) wxGetApp().objects.options ;
}

/** Reload option values. */
void omUIKeyboard::Reload()
{
  opts = (omUIOptions *) wxGetApp().objects.options ;
}

/** Show the keyboard on the screen */
void omUIKeyboard::Show()
{
  if (rootNode != NULL) {
    rootNode->PaintWindows() ;  
    rootNode->Show() ;
  }
}


/** Check if the passed window handler belong to the associated leaf
    or to any child leaf.
    The handler is a void * so this method is not system dependent.
    @param wh window handler to check

    @return true if wh is handler of leaf or any child leaf, false otehrwise
*/
bool omUIKeyboard::IsYourWindow( void *wh)
{
  return rootNode->IsYourWindow(wh) ;
}

/** Load keyboard from XML file using omXMLParser to parse it .
    This functions stops all timers associated with the omUIApplication, remember
    to restart the timers if needed.
    @param keyb name of XML file describing the keyboard
*/
void omUIKeyboard::LoadKeyboard(wxString keyb)
{
  // go to keyboard path
  //string kp = opts->keyboardPath ;
  //  chdir((char *) _T("keyboard")) ;

  // stop all timers for safety purposes
  // remember to start timers if needed at the end of the
  // function
  wxGetApp().objects.StopAllTimers() ;
  if (rootNode != NULL )
    delete rootNode ;
    //   rootNode->DeleteChildren() ;  // delete old keyboard

  if (currentNode != NULL)
    currentNode = NULL ;

  // flush wxApp events
  wxGetApp().Yield() ;

  active = false ;
  omXMLParser::Parse(keyb) ;
  //wxGetApp().objects.StartAllTimers() ;

}

/** Pass the action signal to the tree structure */
void omUIKeyboard::Action()
{
   if (opts->autoscan)
    {
      this->Stop() ;
      rootNode->Action() ;
      this->Start() ;
    }

}


/** Force the Tick() signal from outside the class */
void omUIKeyboard::Tick() 
{
  if (rootNode == NULL)
    return;

  rootNode->Tick() ;
}

/** Repaint the keyboard */
void omUIKeyboard::RepaintKeyboard()
{
  if (rootNode == NULL)
    return ;

  rootNode->Repaint();
}
