////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org> 
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omUIObjects.h>

/** The constructor initialize to NULL all the pointers */
omUIObjects::omUIObjects()
{
  focusChecker = NULL ;
  keyboard = NULL ;
  options = NULL ;
  actionsQueue = NULL ;
  commandParser = NULL ;
  connector = NULL ;

  lastEventWindow = NULL ;
}


/** Initialize all the classes.
    This method invoce Init() method of all not null objects.
    This function is called after that all objects are istantiated.
*/
void omUIObjects::Init()
{
  if ( options != NULL )
    options->Init() ;

  if ( actionsQueue != NULL)
    actionsQueue->Init() ;

  if ( keyboard != NULL )
    keyboard->Init() ;

  if (focusChecker != NULL )
    focusChecker->Init() ;

  if (connector != NULL)
    connector->Init();

  if (commandParser != NULL)
    commandParser->Init() ;
}

/** Lounch Reload() signal to all objects of the application.
    This signal causes the reload of the options with the actual version.
*/
void omUIObjects::Reload()
{
  if ( options != NULL )
    options->Reload() ;

  if ( actionsQueue != NULL)
    actionsQueue->Reload() ;

  if ( keyboard != NULL )
    keyboard->Reload() ;

  if (focusChecker != NULL )
    focusChecker->Reload() ;

  if (connector != NULL)
    connector->Reload();

  if (commandParser != NULL)
    commandParser->Reload() ;
}


/** Start all the timers associated with the object.
    The timers must implement Start() methos.
*/
void omUIObjects::StartAllTimers()
{
  if (focusChecker != NULL)
    focusChecker->Start() ;
  if (keyboard != NULL)
    keyboard->Start() ;
  if (options != NULL)
    options->Start();
  if ( actionsQueue != NULL)
    actionsQueue->Start() ;
  if (commandParser != NULL)
    commandParser->Start() ;
  if (connector != NULL)
    connector->Start() ;
}


/** Stop all the timers associated with the object.
    The timers must implement Stop() method.
*/
void omUIObjects::StopAllTimers()
{
  if (focusChecker != NULL)
    focusChecker->Stop() ;
  if (keyboard != NULL)
    keyboard->Stop() ;
  if (options != NULL)
    options->Stop();
  if ( actionsQueue != NULL)
    actionsQueue->Stop() ;
    if (commandParser != NULL)
    commandParser->Stop() ;
    if (connector != NULL)
      connector->Stop() ;
}
