////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omUIOptions.h>
 
#include <wx/wx.h>


// Costructor for testing purposes
omUIOptions::omUIOptions()
{
  
  //actionKey =SHIFTKEY ;  // shift
  actionKey =CONTROLKEY ;

  leftKey = LEFTARROWKEY ;
  rightKey = RIGHTARROWKEY ;
  upKey = UPARROWKEY ;
  downKey = DOWNARROWKEY ;


  grabKeys = false ;
  actionOnPress = true ; 

  // ----- scan options default ---
  scanTime = 1200;
  scanLoopsNumber = 1 ;
  
  scanReverseDirection = DEFAULTREVERSEDIRECTIONS ;

  screenWidth=wxSystemSettings::GetMetric(wxSYS_SCREEN_X);   //Get current screen width
  screenHeight=wxSystemSettings::GetMetric(wxSYS_SCREEN_Y);  //Get current screen height

  //keyboardPath = "keyboards/extended" ;

  soundEnabled = true;
 
  serialEnabled = true ;
  serialPort = wxString(_T("/dev/ttyS0")) ;

  joystickEnabled = true ;

  keyboardEnabled = true ;

  autoscan = false ;
  joystick = true ;
  direct = true ;

  delayAfterClick = true ;
  delayAfterAction = true ;
  delayAfterEndBlock = true ;
  scanResumeFrom = RESUMEFROMTHIS ;
  exitBlockAfterClick = false ;
}


/** Initialize the class.
    This function is called after that all objects are istantiated
*/
void omUIOptions::Init()
{
}


/** Set a parameter value.
    @param name Parameter Name
    @param value Parameter value.

    @return true if this class has a parameter called 'name', false if not.
*/
bool omUIOptions::SetParameter(wxString name, wxString value)
{
  
  // check parent options
  if (omOptions::SetParameter(name, value))
    return true;

  // --------------------------------------------- actionKey  -------
  int intValue;
  bool boolValue = false;
  if ( name.CompareTo(_T("actionKey")) == 0)
    {
      intValue = omFunctions::Literal2Value(value);
      if (intValue != -1)
	this->actionKey = intValue;

      return true;
    }

  // --------------------------------------------- leftKey  -------
  if ( name.CompareTo(_T("leftKey")) == 0)
    {
      intValue = omFunctions::Literal2Value(value);
      if (intValue != -1)
	this->leftKey = intValue;

      return true;
    }

  // --------------------------------------------- rightKey  -------
  if ( name.CompareTo(_T("rigtKey")) == 0)
    {
      intValue = omFunctions::Literal2Value(value);
      if (intValue != -1)
	this->rightKey = intValue;

      return true;
    }

  // --------------------------------------------- upKey  -------
  if ( name.CompareTo(_T("upKey")) == 0)
    {
      intValue = omFunctions::Literal2Value(value);
      if (intValue != -1)
	this->upKey = intValue;

      return true;
    }

  // --------------------------------------------- grabKeys  -------
  if ( name.CompareTo(_T("grabKeys")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->grabKeys = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- actionOnPress  -------
  if ( name.CompareTo(_T("actionOnPress")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->actionOnPress = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- scanTime  -------
  if ( name.CompareTo(_T("scanTime")) == 0)
    {
      if ( omFunctions::IsInt(value))
	this->scanTime = omFunctions::String2Int(value) ;

      return true;
    }

  // --------------------------------------------- scanLoopsNumber  -------
  if ( name.CompareTo(_T("scanLoopsNumber")) == 0)
    {
      if ( omFunctions::IsInt(value))
	this->scanLoopsNumber = omFunctions::String2Int(value) ;

      return true;
    }

  // --------------------------------------------- scanReverseDirection  -------
  if ( name.CompareTo(_T("scanReverseDirection")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->scanReverseDirection = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- soundEnabled  -------
  if ( name.CompareTo(_T("soundEnabled")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->soundEnabled = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- serialEnabled  -------
  if ( name.CompareTo(_T("serialEnabled")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->serialEnabled = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- serialPort  -------
  if ( name.CompareTo(_T("serialPort")) == 0)
    {
      this->serialPort = value;

      return true;
    }

  // --------------------------------------------- joystickEnabled  -------
  if ( name.CompareTo(_T("joystickEnabled")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->joystickEnabled = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- keyboardEnabled  -------
  if ( name.CompareTo(_T("keyboardEnabled")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->keyboardEnabled = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- autoscan  -------
  if ( name.CompareTo(_T("autoscan")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->autoscan = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- joystick  -------
  if ( name.CompareTo(_T("joystick")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->joystick = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- direct  -------
  if ( name.CompareTo(_T("direct")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->direct = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- delayAfterClick  -------
  if ( name.CompareTo(_T("delayAfterClick")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->delayAfterClick = omFunctions::String2Boolean(value) ;

      return true;
    }

  // ---------------------------------------------  delayAfterAction -------
  if ( name.CompareTo(_T("delayAfterAction")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->delayAfterAction = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- delayAfterEndBlock  -------
  if ( name.CompareTo(_T("delayAfterEndBlock")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->delayAfterEndBlock = omFunctions::String2Boolean(value) ;

      return true;
    }

  // --------------------------------------------- exitBlockAfterClick  -------
  if ( name.CompareTo(_T("exitBlockAfterClick")) == 0)
    {
      if ( omFunctions::IsBoolean(value))
	this->exitBlockAfterClick = omFunctions::String2Boolean(value) ;

      return true;
    }


  // --------------------------------------------- scanResumeFrom -------
  if ( name.CompareTo(_T("scanResumeFrom")) == 0)
    {
      intValue = omFunctions::Literal2Value(value);
      if (intValue != -1)
	this->scanResumeFrom = intValue;

      return true;
    }

  // if arrive here parameter was not found so return false
  return false;

}


/** Get value of a parameter specified by its name.
    @param name Name of the parameter.

    @return value of the parameter as wxString
*/
wxString omUIOptions::GetParameter(wxString name)
{
  wxString tmpString = _T("") ;

  // check for parent GetParameter
  tmpString = omOptions::GetParameter(name) ;
  if (tmpString.CompareTo(_T("")) != 0)
    return tmpString;

  // ------------------------------------------------ actionKey --
  if (name.CompareTo(_T("actionKey")) ==0)
    {
      return omFunctions::Value2Literal(actionKey);
    }

  // ------------------------------------------------ leftKey  --
  if (name.CompareTo(_T("leftKey")) ==0)
    {
      return omFunctions::Value2Literal(actionKey);
    }

  //------------------------------------------------ rightKey  --
  if (name.CompareTo(_T("rightKey")) ==0)
    {
      return omFunctions::Value2Literal(actionKey);
    }

  // ------------------------------------------------  upKey --
  if (name.CompareTo(_T("upKey")) ==0)
    {
      return omFunctions::Value2Literal(upKey);
   } 

  // ------------------------------------------------ downKey  --
  if (name.CompareTo(_T("upKey")) ==0)
    {
      return omFunctions::Value2Literal(upKey);
   } 

  // ------------------------------------------------ grabKeys  --
  if (name.CompareTo(_T("grabKeys")) ==0)
    {
      return omFunctions::Boolean2String(grabKeys);
   } 

  // ------------------------------------------------ actionOnPress  --
  if (name.CompareTo(_T("actionOnPress")) ==0)
    {
      return omFunctions::Boolean2String(actionOnPress);
   } 

  // ------------------------------------------------ scanTime  --
  if (name.CompareTo(_T("scanTime")) ==0)
    {
      tmpString << scanTime ;
      return tmpString;
    }

  // ------------------------------------------------ scanLoopsNumber  --
  if (name.CompareTo(_T("scanLoopsNumber")) ==0)
    {
      tmpString <<scanLoopsNumber ;
      return tmpString;
    }

  // ------------------------------------------------  scanReverseDirection  --
  if (name.CompareTo(_T("scanReverseDirection")) ==0)
    {
      return omFunctions::Boolean2String(scanReverseDirection);
    } 

  // ------------------------------------------------  soundEnabled  --
  if (name.CompareTo(_T("soundEnabled")) ==0)
    {
      return omFunctions::Boolean2String(soundEnabled);
    } 

  // ------------------------------------------------  serialEnabled  --
  if (name.CompareTo(_T("serialEnabled")) ==0)
    {
      return omFunctions::Boolean2String(serialEnabled);
    } 

  // ------------------------------------------------  serialPort  --
  if (name.CompareTo(_T("serialPort")) ==0)
    {
      return serialPort;
    }

  // ------------------------------------------------  joystickEnabled  --
  if (name.CompareTo(_T("joystickEnabled")) ==0)
    {
      return omFunctions::Boolean2String(joystickEnabled);
    } 

  // ------------------------------------------------ keyboardEnabled   --
  if (name.CompareTo(_T("keyboardEnabled")) ==0)
    {
      return omFunctions::Boolean2String(keyboardEnabled);
    } 
  
  // ------------------------------------------------  autoscan  --
  if (name.CompareTo(_T("autoscan")) ==0)
    {
      return omFunctions::Boolean2String(autoscan);
    } 

  // ------------------------------------------------ joystick --
  if (name.CompareTo(_T("joystick")) ==0)
    {
      return omFunctions::Boolean2String(joystick);
    } 
  
  // -------------------------------------------------- direct --
  if (name.CompareTo(_T("direct")) ==0)
    {
      return omFunctions::Boolean2String(direct);
    } 

  // -------------------------------------------------- delayAfterClick --
  if (name.CompareTo(_T("delayAfterClick")) ==0)
    {
      return omFunctions::Boolean2String(delayAfterClick);
    } 

  // -------------------------------------------------- delayAfterAction --
  if (name.CompareTo(_T("delayAfterAction")) ==0)
    {
      return omFunctions::Boolean2String(delayAfterAction);
    } 

  // --------------------------------------------------  delayAfterEndBlock--
  if (name.CompareTo(_T("delayAfterEndBlock")) ==0)
    {
      return omFunctions::Boolean2String(delayAfterEndBlock);
    } 

  // -------------------------------------------------- exitBlockAfterClick --
  if (name.CompareTo(_T("exitBlockAfterClick")) ==0)
    {
      return omFunctions::Boolean2String(exitBlockAfterClick);
    } 
  
  // ------------------------------------------------ scanResumeFrom --
  if (name.CompareTo(_T("scanResumeFrom")) ==0)
    {
      return omFunctions::Value2Literal(scanResumeFrom);
    }
 
  // parameter was not found so return an empty string
  return _T("");

}


// Generate Options XML string.
// Documentation taken from parent.
wxString omUIOptions::GenerateOptionsXMLString()
{
  // get parent options
  wxString xmlString = omOptions::GenerateOptionsXMLString();
  
  // add specific options
  xmlString << _T("<userInterfaceOptions>\n") ;
  
  xmlString << _T("\t<option name=\"actionKey\" value=\"") << omFunctions::Value2Literal(actionKey) << _T("\">\n");
  xmlString << _T("\t<option name=\"leftKey\" value=\"") << omFunctions::Value2Literal(leftKey) << _T("\">\n");
  xmlString << _T("\t<option name=\"rightKey\" value=\"") << omFunctions::Value2Literal(rightKey) << _T("\">\n");
  xmlString << _T("\t<option name=\"upKey\" value=\"") << omFunctions::Value2Literal(upKey) << _T("\">\n");
  xmlString << _T("\t<option name=\"downKey\" value=\"") << omFunctions::Value2Literal(downKey) << _T("\">\n");
  xmlString << _T("\t<option name=\"grabKeys\" value=\"") << omFunctions::Boolean2String(grabKeys) << _T("\">\n");
  xmlString << _T("\t<option name=\"actionOnPress\" value=\"") << omFunctions::Boolean2String(actionOnPress) << _T("\">\n");
  xmlString << _T("\t<option name=\"scanTime\" value=\"") << scanTime << _T("\">\n");
  xmlString << _T("\t<option name=\"scanLoopsNumber\" value=\"") << scanLoopsNumber << _T("\">\n");
  xmlString << _T("\t<option name=\"scanReverseDirection\" value=\"") << omFunctions::Value2Literal(scanReverseDirection)  << _T("\">\n");
  xmlString << _T("\t<option name=\"soundEnabled\" value=\"") << omFunctions::Boolean2String(soundEnabled)  << _T("\">\n");
  xmlString << _T("\t<option name=\"serialEnabled\" value=\"") << omFunctions::Boolean2String(serialEnabled)  << _T("\">\n");
  xmlString << _T("\t<option name=\"serialPort\" value=\"") << serialPort  << _T("\">\n");
  xmlString << _T("\t<option name=\"joystickEnabled\" value=\"") << omFunctions::Boolean2String(joystickEnabled)  << _T("\">\n");
  xmlString << _T("\t<option name=\"keyboardEnabled\" value=\"") << omFunctions::Boolean2String(keyboardEnabled)  << _T("\">\n");
  xmlString << _T("\t<option name=\"autoscan\" value=\"") << omFunctions::Boolean2String(autoscan)  << _T("\">\n");
  xmlString << _T("\t<option name=\"joystick\" value=\"") << omFunctions::Boolean2String(joystick)  << _T("\">\n");
  xmlString << _T("\t<option name=\"direct\" value=\"") << omFunctions::Boolean2String(direct)  << _T("\">\n");
  xmlString << _T("\t<option name=\"delayAfterClick\" value=\"") << omFunctions::Boolean2String(delayAfterClick)  << _T("\">\n");
  xmlString << _T("\t<option name=\"delayAfterAction\" value=\"") << omFunctions::Boolean2String(delayAfterAction)  << _T("\">\n");
  xmlString << _T("\t<option name=\"delayAfterEndBlock\" value=\"") << omFunctions::Boolean2String(delayAfterEndBlock)  << _T("\">\n");
  xmlString << _T("\t<option name=\"scanResumeFrom\" value=\"") << omFunctions::Value2Literal(scanResumeFrom)  << _T("\">\n");
  xmlString << _T("\t<option name=\"exitBlockAfterClick\" value=\"") << omFunctions::Boolean2String(exitBlockAfterClick)  << _T("\">\n");


  xmlString << _T("</userInterfaceOptions>\n") ;

  return xmlString;
}
