////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#include <openmaia/omXMLParameters.h>


/** Construct the class */
omXMLParameters::omXMLParameters()
{
  num = 0 ;
  cur = 0 ;
}

/** Add new parameter in the structure.
    If there is another parameter with the same name the function return false and doesn't insert 
    the new parameter.
    @param n name of the parameter.
    @param v value of the parameter.

    @return true if parameter is added successfully.
*/
bool omXMLParameters::AddParameter( wxString n, wxString v)
{
  //n = MaiaFunctions::ConvertToLower(n) ;
  //v = MaiaFunctions::ConvertToLower(v) ;

  omXMLParameter np(n,v) ;
  
//   // check if the parameter alredy exist in the structure.
  wxString sv = this->GetValue(n) ;

  // if parameter exist with different value return false
  if ( ( sv.CmpNoCase(_T("")) != 0 ) && ( sv.CmpNoCase(v) != 0))
    return false ;
  // if parameter exist with the same value don't insert and return true
  if ( ( sv.CmpNoCase(_T("")) != 0 ) && ( sv.CmpNoCase(v) == 0 ))
    return true ;
       
  // elsewere insert the parameter and return true

  // insert the new parameter in the dinamic vector
  parameters.insert( parameters.end(), np) ;
  num++ ;

  return true ;
}

/** Get parameter value by name.
    @param n name of the parameter

    @return value of the parameter. If name is not found returns a void wxString.
*/
wxString omXMLParameters::GetValue( wxString n)
{
   wxString v = wxString(_T("")) ;
   
   // search the value sequentially!!!
   int i ;
   bool found = false ;

   for ( i=0; i < num && !found; i++) // for all stored parameters
     {
       omXMLParameter pi = parameters[i] ;
       if ( parameters[i].name.CmpNoCase(n) == 0 ){    // value found
	 v = parameters[i].value ;
	 found = true ;
       }
     }

   return v ;
}

/** Get parameter value in lowercase by name.
    @param n name of the parameter

    @return value of the parameter. If name is not found returns a void wxString.
*/
wxString omXMLParameters::GetLowerValue( wxString n)
{
  wxString v = this->GetValue(n) ;

  v = omFunctions::ConvertToLower(v) ;
   
  return v ;

}

/** Get integer value of parameter value by name.
    @param n name of the parameter

    @return value of the parameter. If name is not found -1.
*/
int omXMLParameters::GetIntValue( wxString n) 
{
  int intval = -1;
  long longval =0;
  
  wxString stringval = this->GetValue(n) ;

  if ( stringval.CmpNoCase(_T("")) != 0 ) 
    {
      stringval.ToLong(&longval) ;
      intval=(int)longval;
    }

  return intval ;
}

/** Return the name of the parameter by index.
    @param i index of the parameter.

    @return name of the parameter. If the parameter doesn't exist returns a void wxString.
*/
wxString omXMLParameters::GetName( int i)
{
  wxString n = wxString(_T("")) ;

  if ( i < num )//If i is a valid index number...
    n = parameters[i].name ;

  return n ;
}

/** Return the name of the parameter in lowercase by index.
    @param i index of the parameter.

    @return name of the parameter. If the parameter doesn't exist returns a void wxString.
*/
wxString omXMLParameters::GetLowerName( int i)
{
  wxString n = this->GetName(i) ;

  n = omFunctions::ConvertToLower(n) ;

  return n ;
}


/** Return the value of the parameter by index.
    @param i index of the parameter.

    @return value of the parameter. If the parameter doesn't exist returns a void wxString.
*/
wxString omXMLParameters::GetValue( int i)
{
  wxString v = wxString(_T("")) ;

  if ( i < num )
    v = parameters[i].value ;

  return v ;
}

/** Return the value of the parameter in lowercase by index.
    @param i index of the parameter.

    @return lowercase value of the parameter. If the parameter doesn't exist returns a void wxString.
*/
wxString omXMLParameters::GetLowerValue( int i)
{
  wxString v = this->GetValue(i) ;

  v = omFunctions::ConvertToLower(v) ;

  return v ;
}


/** Return the number of stored parameters

    @return number of stored parameters
*/
int omXMLParameters::GetParsNumber()
{
  return num ;
}

/** Reset the counter to 0.
    This function is used in automatic retrive mode for resetting keyboard .
*/

void omXMLParameters::ResetCounter()
{
  num = 0 ;
}
