////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <omExecutor.h>

#include <iostream>



#define NOMINMAX
#include <windows.h>
#include <wx/msw/winundef.h>

//#ifdef min
//#undef min
//#endif

//#ifdef max
//#undef max
//#endif






void omExecutor::SDConstructor()
{
  omFunctions::InitializeSimulationChars(maiaKeyCode) ;
}

/// Render the string using X11 api
void omExecutor::SDRenderChar(char c)
{
  
  short code = VkKeyScan(c) ;
  
  // least significant byte contains keycode
  int vkToRender = code & 0xFF ;
  // analize most significant byte to catch ALT,CTRL,SHIFT bits
  // in this order
  bool shiftPressed = false ;
  bool ctrlPressed = false ;
  bool altPressed = false ;


  if ( code & 0x0100 )    // SHIFT MASK 
    shiftPressed = true ;

  if ( code & 0x0200 )    // CONTROL MASK
    ctrlPressed = true ;

  if ( code & 0x0400 )    // ALT MASK
    altPressed = true ;

  // emulate transform keys press
  if ( shiftPressed) {
    keybd_event(VK_SHIFT, 0, 0, 0 ) ;
  }

  if ( ctrlPressed){
    keybd_event(VK_CONTROL, 0, 0, 0 ) ;
  }

  if ( altPressed) {
    keybd_event(VK_MENU, 0, 0, 0 ) ;
  }

  // emulate right key press and release
  keybd_event(vkToRender, 0, 0, 0 ) ;
  keybd_event(vkToRender, 0, KEYEVENTF_KEYUP, 0 ) ;

  // release transform keys
  if ( shiftPressed) {
    keybd_event(VK_SHIFT, 0, KEYEVENTF_KEYUP, 0 ) ;
  }

  if ( ctrlPressed) {
    keybd_event(VK_CONTROL, 0, KEYEVENTF_KEYUP, 0 ) ;
  }

  if ( altPressed) {
    keybd_event(VK_MENU, 0, KEYEVENTF_KEYUP, 0 ) ;
  }

}

// emulate a special key press using virtual key simulation
void omExecutor::SDPressKey( long code)
{
  // emulate key pressure
  keybd_event( maiaKeyCode[code], 0, 0, 0) ;
}

// emulate a special key release using virtual key simulation
void omExecutor::SDReleaseKey( long code)
{
  // emulate key pressure
  keybd_event( maiaKeyCode[code], 0, KEYEVENTF_KEYUP, 0) ;
}

void omExecutor::SDMoveMouse()
{
  POINT currPos ;
  GetCursorPos(&currPos) ;  // get current pointer position
  
  // calculate new position
  int newX = currPos.x + dxMouseMovement ;
  int newY = currPos.y + dyMouseMovement ;

  // set new cursor position
  SetCursorPos(newX, newY) ;

} 

void omExecutor::SDPressMouseButton( int but )
{
  int mAction ;

  // select virtual key for button
  switch(but)
    {
    case LEFTBUTTON:
      mAction = MOUSEEVENTF_LEFTDOWN ;
      break ;

    case CENTRALBUTTON:
      mAction = MOUSEEVENTF_MIDDLEDOWN ;
      break ;  

    case RIGHTBUTTON:
       mAction = MOUSEEVENTF_RIGHTDOWN ;
      break ;  

    
    }

  // emulate button press
  mouse_event( mAction, 0,0,0,0) ;

}
 
void omExecutor::SDReleaseMouseButton( int but )
{
  int mAction ;

  // select virtual key for button
  switch(but)
    {
    case LEFTBUTTON:
      mAction = MOUSEEVENTF_LEFTUP ;
      break ;

    case CENTRALBUTTON:
      mAction = MOUSEEVENTF_MIDDLEUP ;
      break ;  

    case RIGHTBUTTON:
      mAction = MOUSEEVENTF_RIGHTUP ;
      break ;  

    
    }

  // emulate button press
  mouse_event( mAction, 0,0,0,0) ;

}
 
