////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////
// disable microsoft min, max macros that causes troubles

#include <openmaia/omFunctions.h>
#include <openmaia/omemukeys.h>


#define NOMINMAX
#include <windows.h>
#include <wx/msw/winundef.h>


#define PROFILE_KEY (const char *) "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\User Shell Folders"


// ------------------------------------------------ local functions

// GetEnvVarPosition()
//
// Get the position of a windows environment variable inside a string.
// the environment variable is isolated with a % symbol at the beginning and a % symbol
// at the end.
//
// s - string to parse
// sp - start position
// varName - buffer to insert found variable name
// return - initial position of a plausible env var or -1 if not found
int GetEnvVarPosition(wxString s, int sp, wxString *varName);

// GetProfilePath()
//
// Get the path of personal profile data.
// This function checks the register key to get profile path.
// Under windows 9x if profiles are not actived openmaia profiles will
// be stored under OPENMAIA installation path.
wxString GetProfilePath(wxString);

// ProfileActived()
//
// Return true if user profiles are actived and must create .openmaia directory
// into user profile directory.
// Return false if user profiles are not actived so profiles directory must
// be created under installation directory
bool ProfileActived();

// GetRunPath()
//
// Get path used by running application
wxString GetRunPath(wxString) ;

// RunSyncProg()
//
// Run a program using syncronous mode. 
void RunSyncProg(wxString);


/** Initialize a vector containing the system specific simulation codes corresponding
    to the interal codes.
    The vector must have MAXMAIACODES dimension.
    You can find MAXMAIACODES in common.h.
    @param cv void vector that will contain system codes
*/
void omFunctions::InitializeSimulationChars( long *cv)
{
  // initialize maiaKeyCode array with the windows simulation codes

  // special keys
  cv[BACKSPACEKEY] =  VK_BACK ;
  cv[TABKEY] =  VK_TAB ;
  cv[CLEARKEY] =  VK_CLEAR ;
  cv[ENTERKEY] =  VK_RETURN ;
  cv[SHIFTKEY] =  VK_LSHIFT ;
  cv[CONTROLKEY] =  VK_CONTROL ;
  cv[ALTKEY] =  VK_MENU ;
  cv[PAUSEKEY] =  VK_PAUSE ;
  cv[CAPSLOCKKEY] =  VK_CAPITAL ;
  cv[ESCKEY] =  VK_ESCAPE ;
  cv[SPACEKEY] =  VK_SPACE ;
  cv[PAGEUPKEY] =  VK_PRIOR ;
  cv[PAGEDOWNKEY] =  VK_NEXT ;
  cv[ENDKEY] =  VK_END ;
  cv[HOMEKEY] =  VK_HOME ;
  cv[LEFTARROWKEY] =  VK_LEFT ;
  cv[UPARROWKEY] =  VK_UP ;
  cv[RIGHTARROWKEY] =  VK_RIGHT ;
  cv[DOWNARROWKEY] =  VK_DOWN ;
  cv[SELECTKEY] =  VK_SELECT ;
  cv[PRINTSCREENKEY] =  VK_PRINT ;
  cv[EXECUTEKEY] =  VK_EXECUTE ;
  cv[SNAPSHOTKEY] =  VK_SNAPSHOT ;
  cv[INSKEY] =  VK_INSERT ;
  cv[DELKEY] =  VK_DELETE ;
  cv[HELPKEY] =  VK_HELP ;
  cv[NUMLOCKKEY] =  VK_NUMLOCK ;

  // function keys
  cv[F1KEY] =  VK_F1 ;
  cv[F2KEY] =  VK_F2 ;
  cv[F3KEY] =  VK_F3 ;
  cv[F4KEY] =  VK_F4 ;
  cv[F5KEY] =  VK_F5 ;
  cv[F6KEY] =  VK_F6 ;
  cv[F7KEY] =  VK_F7 ;
  cv[F8KEY] =  VK_F8 ;
  cv[F9KEY] =  VK_F9 ;
  cv[F10KEY] =  VK_F10 ;
  cv[F11KEY] =  VK_F11 ;
  cv[F12KEY] =  VK_F12 ;
  cv[F13KEY] =  VK_F13 ;
  cv[F14KEY] =  VK_F14 ;
  cv[F15KEY] =  VK_F15 ;
  cv[F16KEY] =  VK_F16 ;


}


void omFunctions::SetBasePathSD(wxString currentPath)
{
  wxString path = GetProfilePath(currentPath) ;


  if (ProfileActived())
    {
      path << "\\openmaia\\profiles" ;
    }
  else
    {
      // tail bin from currentPath
      int sepPosition = path.Find('\\', TRUE) ;
      path = path.SubString(0,sepPosition-1);
      path << "\\Personal_data\\profiles" ;
    }

  std::cout << "profile path= " << path << std::endl ;

  chdir((char *) path.c_str());
  //system("dir");
}

// Creat personal data folder copying data from share
// binPath is the path of binary file lounched (argv[0])
bool omFunctions::CreateDefaultInstallationSD(wxString appName, wxString binPath)
{
  
  wxString homePath = omFunctions::GetUnixString(GetProfilePath(binPath)) ;
  wxString runPath = GetRunPath(binPath);
  runPath = omFunctions::GetWindowsString(runPath);

  int ret=0;

  wxString pathToCreate;

  wxString suffix;

  if (ProfileActived())
    suffix = _T("/openmaia");
  else
    {
      suffix = _T("/Personal_data");
      // tail /bin from homePath
      int pos = homePath.Find('/',TRUE);
      homePath = homePath.SubString(0,pos-1);
    }

  // test ~/openmaia
  if (!omFunctions::TestFolder(homePath + _T("/openmaia")))
    {
      pathToCreate = homePath + suffix ;
      pathToCreate = omFunctions::GetUnixString(pathToCreate);
      wxString command = runPath + _T("\\mkdir.exe \"") + pathToCreate + _T("\"");
      RunSyncProg(command);
      //command = omFunctions::GetUnixString(command);
      //system(command.c_str());
      //WinExec((char *)command.c_str(),SW_HIDE);
    }
  // test ~/.openmaia/profiles
  if (!omFunctions::TestFolder(homePath + suffix + _T("/profiles")))
    {
      pathToCreate = homePath + suffix + _T("/profiles") ;
      pathToCreate = omFunctions::GetUnixString(pathToCreate);
      wxString command = runPath + _T("\\mkdir.exe \"") + pathToCreate + _T("\"");
      //command = omFunctions::GetUnixString(command);
      //system(command.c_str());
      RunSyncProg(command);
      //WinExec((char *)command.c_str(),SW_HIDE);
    }
  
  // test ~/.openmaia/profiles/default
  if (!omFunctions::TestFolder(homePath + suffix + _T("/profiles/default")))
    {
      pathToCreate = homePath + suffix + _T("/profiles/default") ;
      pathToCreate = omFunctions::GetUnixString(pathToCreate);
      wxString command = runPath + _T("\\mkdir.exe \"") + pathToCreate + _T("\"");
      //command = omFunctions::GetUnixString(command);
      //system(command.c_str());
      RunSyncProg(command);
      //WinExec((char *)command.c_str(),SW_HIDE);
    }

  // create shared path from bin path assuming bin path is INSTALLATION\bin and
  // shared path is INSTALLATION\shared
  wxString sharePath = runPath.SubString(0,runPath.Find('\\',TRUE)-1);
  sharePath << _T("\\share") ;
  sharePath = omFunctions::GetUnixString(sharePath);

  wxString command = runPath + _T("\\cp.exe -r \"") + sharePath + _T("/profiles/default/") + 
    appName + _T("\" \"") + omFunctions::GetUnixString(homePath + suffix) + _T("/profiles/default\"") ;
#ifdef MAIA_DEBUG
  std::cout << "omFunctions::CreateDefaultInstallationSD; copy command: " << command << std::endl;
#endif

  //command = omFunctions::GetUnixString(command);
  //system(command.c_str());
  RunSyncProg(command);
  //WinExec((char *)command.c_str(),SW_HIDE);

  return true;
}

// set the presence of personal installation folder
// binPath is the path of binary file lounched (argv[0])
bool omFunctions::TestPersonalInstallationSD(wxString profile, wxString appname, wxString binPath)
{
  wxString homePath = GetProfilePath(binPath);
  wxString suffix;

  homePath = omFunctions::GetUnixString(homePath);

  if (ProfileActived())
    suffix = _T("/openmaia");
  else
    {
      suffix = _T("/Personal_data");
      // tail /bin from homePath
      int pos = homePath.Find('/',TRUE);
      homePath = homePath.SubString(0,pos-1);
    }


  wxString dataPath = homePath + suffix + _T("/profiles/") + profile + _T("/") + appname  ;
  std::cout << "omFunctions::TestPersonalInstallationSD - " << dataPath << std::endl ;
  return omFunctions::TestFolder(dataPath);
  
}


// Get path of shared files.
// binPath - path of the binary file (used in windows version)
wxString omFunctions::GetSharedPathSD( wxString binPath)
{
  wxString sp = omFunctions::GetInstallationPathSD(binPath);
  sp << _T("\\share") ;
  return sp;
}


// Get path of installation directory
wxString omFunctions::GetInstallationPathSD(wxString binPath)
{
  wxString path = GetRunPath(binPath) ;
  // tail "bin" from runpath
  int binPosition = path.Find("bin");

  // check for uppercase string if not found lowercase one
  if (binPosition == -1)
    binPosition = path.Find("BIN");

  if (binPosition == -1)
    binPosition = path.Find("Bin");


  if (binPosition != -1)
    {
      path = path.Left(binPosition-1);
    }
  return path;
}


wxString omFunctions::GetUserDataPathSD(wxString binPath)
{
  wxString homePath = GetProfilePath(binPath);
  wxString suffix;

  homePath = omFunctions::GetUnixString(homePath);

  if (ProfileActived())
    suffix = _T("/openmaia");
  else
    {
      suffix = _T("/Personal_data");
      // tail /bin from homePath
      int pos = homePath.Find('/',TRUE);
      homePath = homePath.SubString(0,pos-1);
    }

  homePath << suffix;
  return homePath;
}

void omFunctions::CopyFileSD(wxString source, wxString destination, wxString binPath)
{
  wxString runPath = GetRunPath(binPath);
  runPath = omFunctions::GetWindowsString(runPath);
  

  wxString copyCommand = runPath + _T("\\cp.exe -f \"")  + omFunctions::GetUnixString(source) + _T("\" \"") + omFunctions::GetUnixString(destination) + _T("\"");

#ifdef MAIA_DEBUG
  std::cout << "omFunctions::CopyFileSD- " << copyCommand << std::endl;
#endif

  
  //copyCommand = omFunctions::GetUnixString(copyCommand);
  //system(copyCommand.c_str());
  RunSyncProg(copyCommand);
  //WinExec((char *)copyCommand.c_str(),SW_HIDE);
}


void omFunctions::CreateFolderSD(wxString folder, wxString  binPath)
{
  wxString runPath = GetRunPath(binPath);
  runPath = omFunctions::GetWindowsString(runPath);

  wxString copyCommand = runPath + _T("\\mkdir.exe \"") + omFunctions::GetUnixString(folder) + _T("\"");

  RunSyncProg(copyCommand);
}

// ------------------------------------------------------- Functions ---------

int GetEnvVarPosition(wxString s, int sp, wxString *varName)
{
  int i=sp;
  int l = s.Length();
  int foundPos = -1;

  bool found=false;
  bool foundFirst = false;

  wxString varNameLocal = _T("");

  char c;

  while(i<l && !found)     // while not found and not at the end of the string 
    {
      c=s.GetChar(i);
      
      if ( c != '%' && foundFirst)
	varNameLocal << c ;

      if ( c == '%' && foundFirst)
	found=true;

      if ( c == '%' && !foundFirst)
	{
	  foundFirst = true;
	  foundPos = i;
	}

      i++;
    }

  if (found)
      *(varName) = varNameLocal;

  return foundPos;
}


wxString GetProfilePath(wxString currentPath)
{
  static char *keyName= PROFILE_KEY;
  char buffer[255];
  DWORD retValLength = 255;
  HKEY hKey;
  wxString path;

  // trasform currentPath into unix path
  currentPath = omFunctions::GetUnixString(currentPath);

  long res1 = RegOpenKeyEx(HKEY_CURRENT_USER, keyName, 0, KEY_READ, &hKey);
  long res2=-1;
  if (res1 ==ERROR_SUCCESS)
    res2 = RegQueryValueEx(hKey, "AppData", 0, NULL, (BYTE *) buffer, &retValLength);


  if (res1 != ERROR_SUCCESS || res2 != ERROR_SUCCESS)  // key was not present so use openmaia installation path
    {
      return GetRunPath(currentPath) ;
    }
  else


  path = wxString(buffer);
  wxString varName;

  if (res2 == ERROR_SUCCESS)    // key data read
    {
      // expand environment variables present in the path
      int startpos = 0 ;
      startpos = GetEnvVarPosition(path, 0, &varName);
      while ( startpos != -1)
      {
	char *expandVar = getenv((char *)varName.c_str());
	wxString expandString(expandVar);
	if (expandString.Cmp(_T("")) != 0)
	  {
	    wxString toChange = _T("%");
	    toChange << varName << _T("%");
	    path.Replace(toChange,expandString);
	    startpos=0;
	  }
	startpos = GetEnvVarPosition(path, startpos+1, &varName);
      } 
    }

  path = omFunctions::GetUnixString(path) ;

  return path;
  
}


// get the run path form run line
// ATTENTION it works with windows style path string
wxString GetRunPath(wxString cp)
{
  wxString currentPath = omFunctions::GetWindowsString(cp);

  wxString path = omFunctions::GetWindowsString(cp) ;
  // key is not found, in this case set the profile path to 
  // MAIA_INSTALLATION\profiles

  

  // use currentPath to set MAIA_INSTALLATION path
  int lastSepPosition = currentPath.Find('\\',TRUE) ;   // program name after separator
  // check for .exe after separator
  bool tailEndString = false;
  wxString lastPart = currentPath.SubString(lastSepPosition+1,currentPath.Length());
  // search for . in the string from the end
  int dotPos = lastPart.Find('.',TRUE) ;
  if (dotPos != -1)
    {
      lastPart = lastPart.SubString(dotPos+1,lastPart.Length()) ;
      int s = omFunctions::FindFirstNBChar(lastPart,0);
      int e = omFunctions::FindLastNBChar(lastPart);
      lastPart = lastPart.SubString(s,e);
      if ( lastPart.CmpNoCase(_T("exe")) == 0)
	tailEndString = true;
    }

  // tail program name from currentPath.
  if (tailEndString)
    path = currentPath.SubString(0,lastSepPosition-1);

  return path;
}

// Execute a program in syncronous mode
void RunSyncProg(wxString prog)
{
//   long retval;
//   PROCESS_INFORMATION proc;
//   STARTUPINFO startinf;

//   startinf.cb = sizeof(startinf);
//   // create programm session
//   retval = CreateProcessA(NULL, (char *) prog.c_str(), NULL, NULL, 1, 
// 			  NORMAL_PRIORITY_CLASS, NULL, NULL, &startinf, &proc);
//   retval = WaitForSingleObject(proc.hProcess, INFINITE);
//   retval = CloseHandle(proc.hProcess);

  //wxShell(prog);
  ::wxExecute(prog,wxEXEC_SYNC);

}

// Get Activation profile status
bool ProfileActived()
{
  // if profiles are actived the path is memoirzed in this registry key
  static char *keyName= PROFILE_KEY;
  char buffer[255];
  DWORD retValLength = 255;
  HKEY hKey;
  wxString path;

  long res1 = RegOpenKeyEx(HKEY_CURRENT_USER, keyName, 0, KEY_READ, &hKey);
  long res2 = -1;
  if (res1 ==ERROR_SUCCESS)
    res2 = RegQueryValueEx(hKey, "AppData", 0, NULL, (BYTE *) buffer, &retValLength);

  if (res1 != ERROR_SUCCESS || res2 != ERROR_SUCCESS)  // key was not present so use openmaia installation path
    {
      return false;
    }
  else
    return true;
}
