////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <omKeyboardDriver.h>
#include "omWinData.h"

#include <iostream>


// disable mircrosoft min,max macros that causes troubles :)
#define NOMINMAX
#include <windows.h>
#include <wx/msw/winundef.h>




// function prototipies
LRESULT CALLBACK  KeyboardHandler(int , WPARAM , LPARAM) ;
long CheckDouble(long) ;   


// static variables used to handle data in the keyboard procedure
static HHOOK handlerHook ;
static long *maiaCode ;
static omKeyboardDriver *checker ;
static omObject *ec ;
static bool echoKey, grk ;  
static long uk, dk, lk, rk, ak;


// structure used to retrive key information by the keyboard procedure
typedef struct stKBHOOKINFO
{
  long vkCode ;       // virtual key code, range 1 to 254
  long scanCode;      // hardware scan code for the key
  long flags ;        // specifies extended-key flag,
                      // event-injected flag, context code,
                      // and transition-state flag
  long time ;         // timespamp for this message
  long dwExtraInfo ;  // extra info associated
} KBHOOKINFO ;


// -----------

void omKeyboardDriver::SystemDependentConstructor()
{
  omWinData *data = new omWinData() ;

  data->lastActionKeyRetVal =0 ;
  this->systemDependentData = data ;
  omFunctions::InitializeSimulationChars(maiaKeyCode) ;
}

void omKeyboardDriver::SDDestructor()
{

}


// This method is called every tickTime mseconds.
// It checks if the action key was pressed, if so sends the Action event to
// the object specified in the constructor of the class.
void omKeyboardDriver::SystemDependentNotify()
{
  int actionKeyCode = maiaKeyCode[actionKey] ;
  int upKeyCode = maiaKeyCode[upKey] ;
  int downKeyCode = maiaKeyCode[downKey] ;
  int leftKeyCode = maiaKeyCode[leftKey] ;
  int rightKeyCode = maiaKeyCode[rightKey] ;

  
  // retrive last data
  omWinData *data = (omWinData *) systemDependentData ;

  // check for events
  unsigned short retval = GetAsyncKeyState( actionKeyCode) & 0x8000;

  ///// pressure events
  // Action key
  retval = GetAsyncKeyState( actionKeyCode) & 0x8000;

  if ( retval != 0 && data->lastActionKeyRetVal == 0 ) {
    this->toInform->ActionButtonDown();
    data->lastActionKeyRetVal = retval ; 
    if (actionOnPress)
      {
	if (executorController != NULL)
	  executorController->StopExecutor() ;
	this->toInform->Action() ;
	if (executorController != NULL)
	  executorController->StartExecutor() ;
      }
  }
  
  // Up key
  retval = GetAsyncKeyState( upKeyCode) & 0x8000;
  if ( retval != 0 && data->lastUpKeyRetVal == 0 ) {
    this->toInform->UpButtonDown();
    data->lastUpKeyRetVal = retval ;
    if (actionOnPress)
      {
	if (executorController != NULL)
	  executorController->StopExecutor() ;

	this->toInform->GoUp() ;
	if (executorController != NULL)
	  executorController->StartExecutor() ;
      }
  }
  
  // Down key
  retval = GetAsyncKeyState( downKeyCode) & 0x8000;
  if ( retval != 0 && data->lastDownKeyRetVal == 0 ) {
    this->toInform->DownButtonDown();
    data->lastDownKeyRetVal = retval ; 
    if (actionOnPress)
      {
	if (executorController != NULL)
	  executorController->StopExecutor() ;
	this->toInform->GoDown() ;
	if (executorController != NULL)
	  executorController->StartExecutor() ;	
      }
  }
  
  // left key
  retval = GetAsyncKeyState( leftKeyCode) & 0x8000;
  if ( retval != 0 && data->lastLeftKeyRetVal == 0 ) {
    this->toInform->LeftButtonDown();
    data->lastLeftKeyRetVal = retval ; 
    if (actionOnPress)
      {
	if (executorController != NULL)
	  executorController->StopExecutor() ;
	this->toInform->GoLeft() ;
	if (executorController != NULL)
	  executorController->StartExecutor() ;	
      }
  }
  
  // Right key
  retval = GetAsyncKeyState( rightKeyCode) & 0x8000;
  if ( retval != 0 && data->lastRightKeyRetVal == 0 ) {
    this->toInform->RightButtonDown();
    data->lastRightKeyRetVal = retval ; 
    if (actionOnPress)
      {
	if (executorController != NULL)
	  executorController->StopExecutor() ;
	this->toInform->GoRight() ;
	if (executorController != NULL)
	  executorController->StartExecutor() ;	
      }
  }
  


  // release events

  // Action key
  retval = GetAsyncKeyState( actionKeyCode) & 0x8000 ;
  if ( retval == 0 && data->lastActionKeyRetVal != 0 ) {
    this->toInform->ActionButtonUp();
    data->lastActionKeyRetVal = retval ; 
    if (!actionOnPress)
      {
	if (executorController != NULL)
	  executorController->StopExecutor() ;
	this->toInform->Action() ;
	if (executorController != NULL)
	  executorController->StartExecutor() ;	
      }
  }
  
  // Up key
  retval = GetAsyncKeyState( upKeyCode) & 0x8000;  
  if ( retval == 0 && data->lastUpKeyRetVal != 0 ) {
    this->toInform->UpButtonUp();
    data->lastUpKeyRetVal = retval ; 
    if (!actionOnPress)
      {
	if (executorController != NULL)
	  executorController->StopExecutor() ;
	this->toInform->GoUp() ;
	if (executorController != NULL)
	  executorController->StartExecutor() ;	
      }
  }
  
  // Down key
  retval = GetAsyncKeyState( downKeyCode) & 0x8000;
  if ( retval == 0 && data->lastDownKeyRetVal != 0 ) {
    this->toInform->DownButtonUp();
    data->lastDownKeyRetVal = retval ; 
    if (!actionOnPress)
      {
	if (executorController != NULL)
	  executorController->StopExecutor() ;
	this->toInform->GoDown() ;
	if (executorController != NULL)
	  executorController->StartExecutor() ;
      }
  }
  
  // Left key
  retval = GetAsyncKeyState( leftKeyCode) & 0x8000;
  if ( retval == 0 && data->lastLeftKeyRetVal != 0 ) {
    this->toInform->LeftButtonUp();
    data->lastLeftKeyRetVal = retval ; 
    if (!actionOnPress)
      {
	if (executorController != NULL)
	  executorController->StopExecutor() ;
	this->toInform->GoLeft() ;
	if (executorController != NULL)
	  executorController->StartExecutor() ;	
      }
  }
  
  // Right key
  retval = GetAsyncKeyState( rightKeyCode) & 0x8000;
  if ( retval == 0 && data->lastRightKeyRetVal != 0 ) {
    this->toInform->RightButtonUp();
    data->lastRightKeyRetVal = retval ; 
    if (!actionOnPress)
      {
	if (executorController != NULL)
	  executorController->StopExecutor() ;
	this->toInform->GoRight() ;
	if (executorController != NULL)
	  executorController->StartExecutor() ;	
      }
  }
  

//   if (actionOnPress)
//     {
//       retval = GetAsyncKeyState( actionKeyCode) & 0x8000;

//       if ( retval != 0 && data->lastActionKeyRetVal == 0 ) {
// 	if (executorController != NULL)
// 	  executorController->StopExecutor() ;
// 	this->toInform->Action() ;
// 	if (executorController != NULL)
// 	  executorController->StartExecutor() ;
//       }
//       data->lastActionKeyRetVal = retval ; 

//       retval = GetAsyncKeyState( upKeyCode) & 0x8000;
//       if ( retval != 0 && data->lastUpKeyRetVal == 0 ) {
// 	if (executorController != NULL)
// 	  executorController->StopExecutor() ;
// 	this->toInform->GoUp() ;
// 	if (executorController != NULL)
// 	  executorController->StartExecutor() ;
//       }
//       data->lastUpKeyRetVal = retval ;

//       retval = GetAsyncKeyState( downKeyCode) & 0x8000;
//       if ( retval != 0 && data->lastDownKeyRetVal == 0 ) {
// 	if (executorController != NULL)
// 	  executorController->StopExecutor() ;
// 	this->toInform->GoDown() ;
// 	if (executorController != NULL)
// 	  executorController->StartExecutor() ;
//       }
//       data->lastDownKeyRetVal = retval ; 

//       retval = GetAsyncKeyState( leftKeyCode) & 0x8000;
//       if ( retval != 0 && data->lastLeftKeyRetVal == 0 ) {
// 	if (executorController != NULL)
// 	  executorController->StopExecutor() ;
// 	this->toInform->GoLeft() ;
// 	if (executorController != NULL)
// 	  executorController->StartExecutor() ;
//       }
//       data->lastLeftKeyRetVal = retval ; 

//       retval = GetAsyncKeyState( rightKeyCode) & 0x8000;
//       if ( retval != 0 && data->lastRightKeyRetVal == 0 ) {
// 	if (executorController != NULL)
// 	  executorController->StopExecutor() ;
// 	this->toInform->GoRight() ;
// 	if (executorController != NULL)
// 	  executorController->StartExecutor() ;
//       }
//       data->lastRightKeyRetVal = retval ; 
//     }
//   else   // check release on buttons
//     {
//       retval = GetAsyncKeyState( actionKeyCode) & 0x8000 ;

//       if ( retval == 0 && data->lastActionKeyRetVal != 0 ) {
// 	if (executorController != NULL)
// 	  executorController->StopExecutor() ;
// 	this->toInform->Action() ;
// 	if (executorController != NULL)
// 	  executorController->StartExecutor() ;
//       }
//       data->lastActionKeyRetVal = retval ; 

//       retval = GetAsyncKeyState( upKeyCode) & 0x8000;
//       if ( retval == 0 && data->lastUpKeyRetVal != 0 ) {
// 	if (executorController != NULL)
// 	  executorController->StopExecutor() ;
// 	this->toInform->GoUp() ;
// 	if (executorController != NULL)
// 	  executorController->StartExecutor() ;
//       }
//       data->lastUpKeyRetVal = retval ; 

//       retval = GetAsyncKeyState( downKeyCode) & 0x8000;
//       if ( retval == 0 && data->lastDownKeyRetVal != 0 ) {
// 	if (executorController != NULL)
// 	  executorController->StopExecutor() ;
// 	this->toInform->GoDown() ;
// 	if (executorController != NULL)
// 	  executorController->StartExecutor() ;
//       }
//       data->lastDownKeyRetVal = retval ; 

//       retval = GetAsyncKeyState( leftKeyCode) & 0x8000;
//       if ( retval == 0 && data->lastLeftKeyRetVal != 0 ) {
// 	if (executorController != NULL)
// 	  executorController->StopExecutor() ;
// 	this->toInform->GoLeft() ;
// 	if (executorController != NULL)
// 	  executorController->StartExecutor() ;
//       }
//       data->lastLeftKeyRetVal = retval ; 

//       retval = GetAsyncKeyState( rightKeyCode) & 0x8000;
//       if ( retval == 0 && data->lastRightKeyRetVal != 0 ) {
// 	if (executorController != NULL)
// 	  executorController->StopExecutor() ;
// 	this->toInform->GoRight() ;
// 	if (executorController != NULL)
// 	  executorController->StartExecutor() ;
//       }
//       data->lastRightKeyRetVal = retval ; 

//     }


}


void omKeyboardDriver::SDGrabActionKeys()
{
  // set initialy false to echo 
  echoKey = false ;
  // use and undocumented hook to grab all keyboard messages
  maiaCode = this->maiaKeyCode ;
  // set the class pointer and the executor pointer used by the external keyboard procedure 
  checker = this ;  
  grk = grabKeys;
  uk = upKey;
  dk = downKey;
  lk = leftKey;
  rk = rightKey;
  ak = actionKey ;

  ec = executorController;
  handlerHook = SetWindowsHookEx( (long) 13, KeyboardHandler  , 
			 (HINSTANCE) GetModuleHandle(NULL), 
			 0) ;

}

//todo
void omKeyboardDriver::SDUngrabActionKeys()
{
}


// keygrabber function
// this function is called when any  key is pressed.
// The function process the key if it is an action key else return
// the control of the key to the operating system
LRESULT CALLBACK KeyboardHandler( int ncode,
			    WPARAM wParam, LPARAM lParam)
{
  if (echoKey) 
    {
      echoKey = false ;
      return 100 ;
    }

  echoKey = true ;

  if ( ncode == 0)
    {
      KBHOOKINFO info ;
      memcpy(&info, (void *) lParam, sizeof(KBHOOKINFO)) ;

      if (!grk)
	return CallNextHookEx(handlerHook, ncode, wParam, lParam) ;

      //int vk = CheckDouble(info.vkCode) ;
      int vk = info.vkCode ;
      
      if (vk == maiaCode[uk] || vk == maiaCode[dk] ||
	  vk == maiaCode[lk] || vk == maiaCode[rk] ||
	  vk == maiaCode[ak])
	{
	  // if a special key is pressed the hook repeat message to
	  // the system so I don't need to handle the message
	  if ( vk != VK_SHIFT && vk != VK_CONTROL &&
	       vk != VK_MENU)
	    {
	      // stop the execution
	      if (ec!=NULL)
		ec->StopExecutor() ;  
	      if (vk == maiaCode[uk])
		checker->toInform->GoUp() ;
	      if (vk == maiaCode[dk])
		checker->toInform->GoDown() ;
	      if (vk == maiaCode[rk])
		checker->toInform->GoRight() ;
	      if (vk == maiaCode[lk])
		checker->toInform->GoLeft() ;
	      if (vk == maiaCode[ak])
		checker->toInform->Action() ;
	      // restart the execution
	      if (ec!=NULL)
		ec->StartExecutor() ;
	    }
	  return 100;
	  
	}
      
    }

  return CallNextHookEx(handlerHook, ncode, wParam, lParam) ;
}

// check if passed virtual key is a double key, if then return the
// primary one used in maiaKeycodes.
long CheckDouble(long vk)
{
  if ( vk == VK_LSHIFT || vk == VK_RSHIFT)
    return VK_SHIFT ;

  if ( vk == VK_LCONTROL || vk == VK_RCONTROL)
    return VK_CONTROL ;

  if ( vk == VK_LMENU || vk == VK_RMENU)
    return VK_MENU ;

  // else return passed virtual key
  return vk ;
}


// 
void omKeyboardDriver::SDFlushKeysState()
{
  std::cout << "flush keystate" << std::endl ;
  // retrive data informations
  omWinData *data = (omWinData *) systemDependentData ;
  // set to 0 all keys flags
  data->lastActionKeyRetVal = 0;
  data->lastUpKeyRetVal = 0 ;
  data->lastDownKeyRetVal = 0 ;
  data->lastRightKeyRetVal = 0;
  data->lastLeftKeyRetVal = 0 ;
}
