////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omExecutor.h>
#include "omX11Data.h"


#include <iostream>
#include <X11/keysym.h>
#include <X11/extensions/XTest.h>

static Display *dpy;

static KeySym *syms;
static XModifierKeymap *mods;
static int keysyms_per_keycode, min_keycode, max_keycode;

struct modstate {
  KeyCode kc;
  int pressed;
};


void typechar(KeySym );
void reset_modifier_state(void *);
void *set_modifier_state(int );
void update_keymap(void);
void initkeymap(void);

void *xmalloc(size_t);
void *xrealloc(void *, size_t);

#define KEYSTATE(map, kc) (map[kc >> 3] & (1 << (kc & 7)))

static char *modname[] = { "Shift", "Lock", "Control", "Meta",
			   "Mod2", "Mod3", "Mod4", "Mod5" };


/// Initialize maia char emulations table by associating maia codes with
/// X symbolic codes
void omExecutor::SDConstructor()
{
  omFunctions::InitializeSimulationChars(maiaKeyCode) ;
  // open the display for future use
  Display *display ;
  Window root ;

  omX11Data *data = new omX11Data() ;

  char *display_name ;

  if (! (display_name = getenv("DISPLAY")) ){
    std::cerr << "environment variable DISPLAY must be set" << std::endl ;
    exit(-1);
  }
  if (! (display = XOpenDisplay(display_name)) ){
    std::cerr << "I can't open display" << std::endl ;
    exit(-1);
  }

  // open the x11 display
  int screen = DefaultScreen(display);
  root = RootWindow(display,screen);

  // store informations in an object structure for future retrival
  data->display = display ;
  data->rootWindow = root ;

  // store the data as a void pointer
  this->sdData = data ;

  int minkeycode, maxkeycode ;
  XDisplayKeycodes(display, &minkeycode, &maxkeycode) ;

  int keycodecount = maxkeycode-minkeycode+1;
  int keysymsXkeycode;
  KeySym * ksyms = XGetKeyboardMapping(display, minkeycode, keycodecount, &keysymsXkeycode);

//   std::cout << "min: " << minkeycode << "\nmax: " << maxkeycode << std::endl  <<
//     "keysyms per keycode= " << keysymsXkeycode << std::endl;

  
//   std::cout << "\n\n\nLISTA KEYSYMS\n\n" ;
//   int j,k ;
//   for (j=0; j<keycodecount; j++) {
//     std::cout << "\nkc: " << j+minkeycode << std::endl;
//     for(k=0; k<keysymsXkeycode; k++)
//       std::cout << "  " << ksyms[j+k]  ;
//   }

  dpy = display ;
  update_keymap();
  //initkeymap();

}

/// Render the string using X11 api
void omExecutor::SDRenderChar(char c)
{
  omX11Data *data = (omX11Data *) this->sdData ;
  Display *display = data->display ;
  Window root = data->rootWindow;
  
  unsigned int codice = (unsigned int) 0xFF & c ;
  unsigned char pippo = (unsigned char) c ;

  KeySym ksToRender = maiaKeyCode[ codice] ;

  int keycode ;   // retrive keykode from keysym
  if (!(keycode = XKeysymToKeycode(display, ksToRender))) 
    {
      std::cerr << "Cannot find matching keycode." << std::endl;
      //exit(-1);
    }

  //  std::cout << "keysym = " << ksToRender << "\nkeycode = " << keycode << std::endl ;

  if (ksToRender == XK_at)
    {
      XTestFakeKeyEvent(display, 113, True, CurrentTime) ;
      XTestFakeKeyEvent(display, 24, True, CurrentTime) ;
      XTestFakeKeyEvent(display, 24, False, CurrentTime) ;
      XTestFakeKeyEvent(display, 113, False, CurrentTime) ;
      XFlush(display) ;
      return;
    }

  if (ksToRender == XK_numbersign)
    {
      XTestFakeKeyEvent(display, 113, True, CurrentTime) ;
      XTestFakeKeyEvent(display, 48, True, CurrentTime) ;
      XTestFakeKeyEvent(display, 48, False, CurrentTime) ;
      XTestFakeKeyEvent(display, 113, False, CurrentTime) ;
      XFlush(display) ;
      return;
    }





  typechar(ksToRender);




  XFlush(display) ;
  //std::cout << "renderizza" << std::endl ; 
  //  this->SDPressKey( (long) c);
  //  this->SDReleaseKey( (long) c) ;
}


void omExecutor::SDPressKey(long c)
{
  // retrive X11 enviroment informations
  omX11Data *data = (omX11Data *) this->sdData ;
  Display *display = data->display ;
  Window root = data->rootWindow;
  

  KeySym ksToRender = maiaKeyCode[ (int) c] ;

//   char *pippo;
//   pippo = XKeysymToString(ksToRender);
//   std::cout << "stringa keycode= " << pippo << std::endl ;


  int keycode ;   // retrive keykode from keysym
  if (!(keycode = XKeysymToKeycode(display, ksToRender))) 
    {
      std::cerr << "Cannot find matching keycode." << std::endl;
      //exit(-1);
    }

  //  std::cout << "keysym = " << ksToRender << "\nkeycode = " << keycode << std::endl ;

//   XTestFakeKeyEvent(display, XKeysymToKeycode(display, XK_at), True, CurrentTime);
//   XTestFakeKeyEvent(display, XKeysymToKeycode(display, XK_at), False, CurrentTime);

//   Window w ;
//   int revert_to ;
//   XGetInputFocus(display, &w, &revert_to);

  XTestFakeKeyEvent(display, 24, True, CurrentTime) ;
  XTestFakeKeyEvent(display, 24, False, CurrentTime) ;


 
//   int ret = XTestFakeKeyEvent(display, keycode, True, CurrentTime) ;
//   std::cout << "codice di ritorno = " << ret << std::endl ;

  XFlush(display) ;  
}

void omExecutor::SDReleaseKey(long c)
{
 
  // retrive X11 enviroment informations
  omX11Data *data = (omX11Data *) this->sdData ;
  Display *display = data->display ;
  Window root = data->rootWindow;
  

  KeySym ksToRender = maiaKeyCode[ (int) c] ;

  int keycode ;   // retrive keykode from keysym
  if (!(keycode = XKeysymToKeycode(display, ksToRender))) 
    {
      std::cerr << "Cannot find matching keycode." << std::endl;
      //exit(-1);
    }


//   Window w ;
//   int revert_to ;
//   XGetInputFocus(display, &w, &revert_to);

 
  XTestFakeKeyEvent(display, keycode, False, CurrentTime) ;

  XFlush(display) ;
  
}

void omExecutor::SDMoveMouse()
{
  omX11Data *data = (omX11Data *) this->sdData ;
  Display *display = data->display ;
  Window root = data->rootWindow;

  XTestFakeRelativeMotionEvent( display, dxMouseMovement, dyMouseMovement, CurrentTime) ;
  XFlush(display);
} 

void omExecutor::SDPressMouseButton( int but )
{
  omX11Data *data = (omX11Data *) this->sdData ;
  Display *display = data->display ;
  Window root = data->rootWindow;

  XTestGrabControl(display, TRUE) ;
  XTestFakeButtonEvent( display, but, TRUE, CurrentTime) ;
  XFlush(display);
}
 
void omExecutor::SDReleaseMouseButton( int but )
{
  omX11Data *data = (omX11Data *) this->sdData ;
  Display *display = data->display ;
  Window root = data->rootWindow;

  XTestFakeButtonEvent( display, but, FALSE, CurrentTime) ;
  XFlush(display);
}
 
////////////////////

void initkeymap(void)
{
  XDisplayKeycodes(dpy, &min_keycode, &max_keycode);
  syms = XGetKeyboardMapping(dpy, min_keycode, max_keycode - min_keycode + 1,
			     &keysyms_per_keycode);
  mods = XGetModifierMapping(dpy);
}

void update_keymap(void)
{
  XFree(syms);
  XFreeModifiermap(mods);
  initkeymap();
}


void *set_modifier_state(int setmods)
{
  int nmods, modsize, mod, key, state, wantstate;
  struct modstate *old;
  char keys[32];
  KeyCode kc;

  nmods = 0;
  modsize = 10;
  old = (struct modstate *) xmalloc(modsize * sizeof(struct modstate));
  memset(old, 0, modsize * sizeof(struct modstate));

  XQueryKeymap(dpy, keys);

  /* For each modifier... */
  for (mod = 0; mod < 8; mod++)
    {
      wantstate = setmods & (1 << mod);

      /* For each key that generates that modifier... */
      for (key = 0; key < mods->max_keypermod; key++)
	{
	  kc = mods->modifiermap[mod * mods->max_keypermod + key];
	  if (!kc)
	    break;
	  state = KEYSTATE(keys, kc);

	  /* If the key isn't in the state we want, fix it and record
	   * that we changed it.
	   */
	  if (state != wantstate)
	    {
	      if (nmods == modsize - 1)
		{
		  old = (struct modstate *)xrealloc(old, modsize * 2 * sizeof(struct modstate));
		  memset(old + modsize * sizeof(struct modstate), 0,
			 modsize * sizeof(struct modstate));
		  modsize *= 2;
		}
	      old[nmods].kc = kc;
	      old[nmods].pressed = state;
	      nmods++;

	      if (wantstate) {
		//		std::cout << "modo " << mod << " attivato\n" ;
		XTestFakeKeyEvent(dpy, kc, True, CurrentTime);
	      }
	      else
		XTestFakeKeyEvent(dpy, kc, False, CurrentTime);
	    }

	  /* If we want the modifier to be set, we only need to look
	   * at the first key, since we'll have set it if it wasn't
	   * already set. (If we want it unset, we need to check each
	   * key and unset any that are set.)
	   */
	  if (wantstate)
	    break;
	}
    }
  return (void *)old;
}

void reset_modifier_state(void *oldstate)
{
  struct modstate *old = (struct modstate *)oldstate;
  int i;

  for (i = 0; old[i].kc; i++)
    {
      if (!old[i].pressed)
	XTestFakeKeyEvent(dpy, old[i].kc, False, CurrentTime);
      else
	XTestFakeKeyEvent(dpy, old[i].kc, True, CurrentTime);
    }
  free(old);
}

void typechar(KeySym ks)
{
  KeyCode kc;
  int mod, xmod = -1, control = 0, start;
  void *old;

  if (ks < 32)
    {
      control = ControlMask;
      ks += 96;
    }
  kc = XKeysymToKeycode(dpy, ks);
  start = (kc - min_keycode) * keysyms_per_keycode;

  for (mod = 0; mod < 8; mod++)
    {
      /* This sucks, but OpenWindows does this wrong. (It lists
       * the uppercase letter in the 0 position and doesn't list
       * the lowercase letter at all.)
       */
      if (isalpha(ks))
	{
	  if (mod == 0 && islower(ks) && tolower(syms[start]) == ks)
	    {
	      /* Looking at unshifted position, want a lowercase
	       * letter, and this is the letter we're looking for.
	       */
	      xmod = 0;
	    }
	  else if (mod == 1 && isupper(ks) &&
		   (syms[start + mod] == ks || syms[start] == ks))
	    {
	      /* Looking at shifted position, want an uppercase
	       * letter, and either this is the letter we're looking
	       * for or the unshifted one was.
	       */
	      xmod = 1;
	    }
	}
      else if (syms[start + mod] == ks)
	xmod = mod;

      if (xmod != -1)
	{
	  old = set_modifier_state(xmod + control);
	  XTestFakeKeyEvent(dpy, kc, True, CurrentTime);
	  XTestFakeKeyEvent(dpy, kc, False, CurrentTime);
	  reset_modifier_state(old);
	  break;
	}
    }
}

void *xmalloc(size_t size)
{
  void *ptr = malloc(size);
  if (!ptr)
    std::cerr << "Out of memory." << std::endl;;
  return ptr;
}

void *xrealloc(void *ptr, size_t size)
{
  void *nptr;

  nptr = realloc(ptr, size);
  if (!nptr)
    std::cerr << "Out of memory." << std::endl;;
  return nptr;
}

