////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////



#include <openmaia/omFunctions.h>

#include <X11/keysym.h>
#include <X11/extensions/XTest.h>

#include <openmaia/omemukeys.h>



// local functions prototipes
wxString GetDataHome();


/** Initialize a vector containing the system specific simulation codes corresponding
    to the interal codes.
    The vector must have MAXMAIACODES dimension.
    You can find MAXMAIACODES in common.h.
    @param cv void vector that will contain system codes
*/
void omFunctions::InitializeSimulationChars( long *cv)
{
  int i ;  // counter
  // maiacodes from 0 to 32 do nothing
  for ( i=0; i< 32; i++)
    cv[i] = XK_VoidSymbol ;

  // keysyms of ascii codes from 32 to 255 are ascii codes :)
  for( i=32; i<256; i++)
    cv[i] = i ;

  // overwrite special symbols
  cv[64] = XK_at;



   cv[BACKSPACEKEY] =  XK_BackSpace ;
   cv[TABKEY] =  XK_Tab;
   cv[CLEARKEY] = XK_Clear;
   cv[ENTERKEY] =  XK_Return ;
   cv[SHIFTKEY] =  XK_Shift_L ; ;
   cv[CONTROLKEY] = XK_Control_L ;
   cv[ALTKEY] =  XK_Alt_L;
   cv[PAUSEKEY] =  XK_Pause;
   cv[CAPSLOCKKEY] =  XK_Caps_Lock;
   cv[ESCKEY] =  XK_Escape;
   cv[SPACEKEY] =  XK_space;
   cv[PAGEUPKEY] =  XK_Page_Up;
   cv[PAGEDOWNKEY] =  XK_Page_Down;
   cv[ENDKEY] =  XK_End;
   cv[HOMEKEY] =  XK_Begin;
   cv[LEFTARROWKEY] =  XK_Left ;
   cv[UPARROWKEY] =  XK_Up ;
   cv[RIGHTARROWKEY] =  XK_Right ;
   cv[DOWNARROWKEY] =  XK_Down ;
   cv[SELECTKEY] =  XK_Select;
   cv[PRINTSCREENKEY] = XK_Print;
   cv[EXECUTEKEY] =  XK_Execute;
   cv[SNAPSHOTKEY] =  XK_VoidSymbol;
   cv[INSKEY] =  XK_Insert;
   cv[DELKEY] =  XK_Delete;
   cv[HELPKEY] =  XK_Help;
   cv[NUMLOCKKEY] =  XK_Num_Lock;

   // function keys
   cv[F1KEY] =  XK_F1;
   cv[F2KEY] =  XK_F2;
   cv[F3KEY] =  XK_F3;
   cv[F4KEY] =  XK_F4;
   cv[F5KEY] =  XK_F5;
   cv[F6KEY] =  XK_F6;
   cv[F7KEY] =  XK_F7;
   cv[F8KEY] =  XK_F8;
   cv[F9KEY] =  XK_F9;
   cv[F10KEY] =  XK_F10;
   cv[F11KEY] =  XK_F11;
   cv[F12KEY] =  XK_F12;
   cv[F13KEY] =  XK_F13;
   cv[F14KEY] =  XK_F14;
   cv[F15KEY] =  XK_F15;
   cv[F16KEY] =  XK_F16;

}


// set path to personal data folder
void omFunctions::SetBasePathSD(wxString currentPath)
{
  chdir((char *)GetDataHome().c_str());
  chdir(".openmaia");
  chdir("profiles");

}

// Creat personal data folder copying data from share
bool omFunctions::CreateDefaultInstallationSD(wxString appName, wxString currentPath)
{
  int ret=0;
  wxString homePath = GetDataHome();
  // test ~/.openmaia
  if (!omFunctions::TestFolder(homePath + _T("/.openmaia")))
    {
      wxString command = _T("mkdir ") + homePath + _T("/.openmaia") ;
      ret = system((char *) command.c_str()) ;
      if (ret == -1)
	return false;
    }
  // test ~/.openmaia/profiles
  if (!omFunctions::TestFolder(homePath + _T("/.openmaia/profiles")))
    {
      wxString command = _T("mkdir ") + homePath + _T("/.openmaia/profiles") ;
      ret = system((char *) command.c_str()) ;
      if (ret == -1)
	return false;
    }
  
  // test ~/.openmaia/profiles/default
  if (!omFunctions::TestFolder(homePath + _T("/.openmaia/profiles/default")))
    {
      wxString command = _T("mkdir ") + homePath + _T("/.openmaia/profiles/default") ;
      ret = system((char *) command.c_str()) ;
      if (ret == -1)
	return false;
    }


  // copy files from template
  wxString installPath;
  if ( wxString(_T(INSTALL_PATH)).Cmp(wxString(_T("NONE"))) == 0)
    installPath = _T("/usr/local") ;
  else
    installPath = _T(INSTALL_PATH) ;

  // tail white chars 
  int lastCharPos = omFunctions::FindLastNBChar(installPath);
  installPath=installPath.SubString(0,lastCharPos);
  // add / symbol at the end of install path if not exist yet
  if (installPath.GetChar(installPath.Length()) != '/' )
    installPath << _T("/") ;

  wxString copyCommand = _T("cp -r ") ;
  copyCommand << installPath << _T("share/openmaia/default/") << appName << _T(" ") 
	      << homePath << _T("/.openmaia/profiles/default");

  // execute command
  ret = system((char *) copyCommand.c_str());
    if ( ret == -1)
      return false ;

  return true;
}

// set the presence of personal installation folder
bool omFunctions::TestPersonalInstallationSD(wxString profile, wxString appname, wxString currentPath)
{
  wxString dataPath;
  wxString homePath = GetDataHome();
  dataPath = homePath + _T("/.openmaia/profiles/") + profile + _T("/") + appname  ;
  return omFunctions::TestFolder(dataPath);

}


// Get path of shared files.
// binPath - path of the binary file (used in windows version)
wxString omFunctions::GetSharedPathSD( wxString binPath)
{
  // get install path from configure.h
  wxString partial(_T(INSTALL_PATH));

  // if --prefix was not used, autoconf initializes INSTALL_PATH as NONE.
  // by default INSTALL_PATH is /usr/local
  if (partial.CmpNoCase(_T("none")) == 0)
    partial = _T("/usr/local") ;

  // add final part
  partial << _T("/share/openmaia") ;

  return partial;
}

// Get path of installation directory
wxString omFunctions::GetInstallationPathSD(wxString binPath)
{
  wxString path(_T(INSTALL_PATH));
  if ( path.CmpNoCase(_T("none")) == 0)
    path = _T("/usr/local");

  path = path + _T("/bin");
  return path;
}

wxString omFunctions::GetUserDataPathSD(wxString binPath)
{
  wxString homePath = GetDataHome();
  homePath << _T("/.openmaia") ;
  return homePath;
}

void omFunctions::CopyFileSD(wxString source, wxString destination, wxString binPath)
{
  wxString copyCommand = _T("cp ") + source + _T(" ") + destination;
  system(copyCommand.c_str());
}

void omFunctions::CreateFolderSD(wxString folder, wxString  binPath)
{
  wxString copyCommand = _T("\\mkdir ") + omFunctions::GetUnixString(folder);

  system(copyCommand);
}



// ------------------------------------------------------- functions ------

// Return the path of the home directory of the user  without last / symbol
// return home dir path.
wxString GetDataHome()
{
  wxString result=_T("");
    
  // Try using $HOME
  const char *h = getenv("HOME");
  if (h != 0) 
    result=wxString(h);
    

  // Try using passwd entry
  if (result.Cmp(_T("")) == 0)
    {
      struct passwd *pw = 0;

      const char *user = getenv("USER");
      if (user == 0)
	user = getenv("LOGNAME");

      if (user != 0)
	pw = getpwnam(user);
      if (pw == 0)
	pw = getpwuid(getuid());
      if (pw != 0)
	result=wxString(pw->pw_dir);
    }

  // Revert to current dir
  if (result.Cmp(_T("")) == 0)
    result = _T(".");
    
  // tail last / symbol of homePath
  int lastpos=omFunctions::FindLastNBChar(result);
  if (result.GetChar(lastpos) == '/')
    result = result.SubString(0,lastpos-1);


  return result;

}
