////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

// This is the X11 part of omKeyboardDriver class

#include <openmaia/omKeyboardDriver.h>

#include "omX11Data.h"

#include <iostream>
#include <X11/keysym.h>
#include <X11/extensions/XTest.h>


// Under X11 open the display and the root window and grab the action key
void omKeyboardDriver::SystemDependentConstructor()
{
  omFunctions::InitializeSimulationChars(maiaKeyCode) ;

  Display *display ;
  Window root ;

  omX11Data *data = new omX11Data() ;

  char *display_name ;

  if (! (display_name = getenv("DISPLAY")) ){
    std::cerr << "environment variable DISPLAY must be set" << std::endl ;
    exit(-1);
  }
  if (! (display = XOpenDisplay(display_name)) ){
    std::cerr << "I can't open display" << std::endl ;
    exit(-1);
  }


  // open the x11 display
  int screen = DefaultScreen(display);
  root = RootWindow(display,screen);

  // store informations in an object structure for future retrival
  data->display = display ;
  data->rootWindow = root ;
  data->actionEcho = false ;

  data->actionSemaphore = false ;
  data->upSemaphore = false ;
  data->downSemaphore = false ;
  data->leftSemaphore = false ;
  data->rightSemaphore = false;

  //  XGrabKey(display, AnyKey, AnyModifier, root, False, GrabModeAsync, GrabModeAsync) ;

  // store the data as a void pointer
  this->systemDependentData = data ;
}

// ungrab grabbed keys
void omKeyboardDriver::SDDestructor()
{
  // retrive data
  omX11Data *data = (omX11Data *) systemDependentData ;

  // retrive display and window
  Window root = data->rootWindow ;
  Display *display = data->display ;
  
  int actionKeycode, upKeycode, downKeycode, leftKeycode, rightKeycode ;  

  if (!(actionKeycode = XKeysymToKeycode(display, maiaKeyCode[actionKey])))
    {
      std::cerr << "Cannot find matching action keycode." << std::endl;
      exit(-1);
    }
  if (!(upKeycode = XKeysymToKeycode(display, maiaKeyCode[upKey])))
    {
      std::cerr << "Cannot find matching up keycode." << std::endl;
      exit(-1);
    }
  if (!(downKeycode = XKeysymToKeycode(display, maiaKeyCode[downKey])))
    {
      std::cerr << "Cannot find matching down keycode." << std::endl;
      exit(-1);
    }
  if (!(leftKeycode = XKeysymToKeycode(display, maiaKeyCode[leftKey])))
    {
      std::cerr << "Cannot find matching left keycode." << std::endl;
      exit(-1);
    }
  if (!(rightKeycode = XKeysymToKeycode(display, maiaKeyCode[rightKey])))
    {
      std::cerr << "Cannot find matching right keycode." << std::endl;
      exit(-1);
    }



  XUngrabKey(display, actionKeycode, AnyModifier, root);
  XUngrabKey(display, upKeycode, AnyModifier, root);
  XUngrabKey(display, downKeycode, AnyModifier, root);
  XUngrabKey(display, leftKeycode, AnyModifier, root);
  XUngrabKey(display, rightKeycode, AnyModifier, root);
}


// This method is called every tickTime mseconds.
// It checks if the action key was pressed, if so sends the Action event to
// the object specified in the constructor of the class.
void omKeyboardDriver::SystemDependentNotify()
{

  XEvent event ;

  // retrive data
  omX11Data *data = (omX11Data *) systemDependentData ;

  // retrive display and window
  Window root = data->rootWindow ;
  Display *display = data->display ;

  int actionKeycode, upKeycode, downKeycode, leftKeycode, rightKeycode ;   

  // retrive application options

  // find keycodes from keysym

  if (!(actionKeycode = XKeysymToKeycode(display, maiaKeyCode[actionKey])))
    {
      std::cerr << "Cannot find matching action keycode." << std::endl;
      exit(-1);
    }
  if (!(upKeycode = XKeysymToKeycode(display, maiaKeyCode[upKey])))
    {
      std::cerr << "Cannot find matching up keycode." << std::endl;
      exit(-1);
    }
  if (!(downKeycode = XKeysymToKeycode(display, maiaKeyCode[downKey])))
    {
      std::cerr << "Cannot find matching down keycode." << std::endl;
      exit(-1);
    }
  if (!(leftKeycode = XKeysymToKeycode(display, maiaKeyCode[leftKey])))
    {
      std::cerr << "Cannot find matching left keycode." << std::endl;
      exit(-1);
    }
  if (!(rightKeycode = XKeysymToKeycode(display, maiaKeyCode[rightKey])))
    {
      std::cerr << "Cannot find matching right keycode." << std::endl;
      exit(-1);
    }

  // check action key
  bool aechoState = data->actionEcho ;
  bool actionPressed =false;

  bool uechoState = data->upEcho ;
  bool upPressed = false ;

  bool dechoState = data->downEcho ;
  bool downPressed = false ;

  bool lechoState = data->leftEcho ;
  bool leftPressed = false ;

  bool rechoState = data->rightEcho ;
  bool rightPressed = false ;

 
  // grab keys
  XGrabKey(display, actionKeycode, AnyModifier, root, False, GrabModeAsync, GrabModeAsync) ;
  XGrabKey(display, upKeycode, AnyModifier, root, False, GrabModeAsync, GrabModeAsync) ;
  XGrabKey(display, downKeycode, AnyModifier, root, False, GrabModeAsync, GrabModeAsync) ;
  XGrabKey(display, leftKeycode, AnyModifier, root, False, GrabModeAsync, GrabModeAsync) ;
  XGrabKey(display, rightKeycode, AnyModifier, root, False, GrabModeAsync, GrabModeAsync) ;
  //  XGrabKey(display, AnyKey, AnyModifier, root, False, GrabModeAsync, GrabModeAsync) ;


  // check for event
  if (XPending(display)) 
    {
      //std::cout << "pending\n" ;
      XNextEvent(data->display, &event) ;
      int code = event.xkey.keycode ;
      bool *pecho ;
      int lastcode ;

//       // check for low level messages
//       if ( code == actionKeycode)
// 	{
// 	  if (event.xkey.type == KeyPress)
// 	    this->toInform->ActionButtonDown();

// 	  if (event.xkey.type == KeyRelease)
// 	    this->toInform->ActionButtonUp();
// 	}


//       if ( code == upKeycode)
// 	{
// 	  if (event.xkey.type == KeyPress)
// 	    this->toInform->UpButtonDown();

// 	  if (event.xkey.type == KeyRelease)
// 	    this->toInform->UpButtonUp();
// 	}

//       if ( code == downKeycode)
// 	{
// 	  if (event.xkey.type == KeyPress)
// 	    this->toInform->DownButtonDown();

// 	  if (event.xkey.type == KeyRelease)
// 	    this->toInform->DownButtonUp();
// 	}

//       if ( code == leftKeycode)
// 	{
// 	  if (event.xkey.type == KeyPress)
// 	    this->toInform->LeftButtonDown();

// 	  if (event.xkey.type == KeyRelease)
// 	    this->toInform->LeftButtonUp();
// 	}

//       if ( code == rightKeycode)
// 	{
// 	  if (event.xkey.type == KeyPress)
// 	    this->toInform->RightButtonDown();

// 	  if (event.xkey.type == KeyRelease)
// 	    this->toInform->RightButtonUp();
// 	}



      // now check if an action key was pressed or released
      if (actionOnPress)
	{

	  if ( code == actionKeycode)
	    {
	      if (event.xkey.type == KeyPress)
		actionPressed = true ;
	      lastcode = code ;
	      pecho = &aechoState ;
	    }


	  if ( code == upKeycode)
	    {
	      if (event.xkey.type == KeyPress && !data->upSemaphore) {
		upPressed = true ;
		data->upSemaphore = true;
	      }
	      if (event.xkey.type == KeyRelease && data->upSemaphore) 
		data->upSemaphore = false;


	      lastcode = code ;
	      pecho = &uechoState ;
	    }

	  if ( code == downKeycode)
	    {
	      if (event.xkey.type == KeyPress)
		downPressed = true ;
	      lastcode = code ;
	      pecho = &dechoState ;
	    }

	  if ( code == leftKeycode)
	    {
	      if (event.xkey.type == KeyPress)
		leftPressed = true ;
	      lastcode = code ;
	      pecho = &lechoState ;
	    }

	  if ( code == rightKeycode)
	    {
	      if (event.xkey.type == KeyPress)
		rightPressed = true ;
	      lastcode = code ;
	      pecho = &rechoState ;
	    }

	  if ( event.xkey.type == KeyRelease && code == lastcode)   // echo the key on release
	    SDFlushKey(code, pecho) ;

	}
      else     // check event on release
	{
	  std::cout << "codice = " << code << std::endl ;
	  if ( code == actionKeycode && event.xkey.type == KeyRelease)
	    {
	      actionPressed = true ;
	      pecho = &aechoState ;
	    }

	  if ( code == upKeycode && event.xkey.type == KeyRelease)
	    {
	      upPressed = true ;
	      pecho = &uechoState ;
	    }

	  if ( code == downKeycode && event.xkey.type == KeyRelease)
	    {
	      downPressed = true ;
	      pecho = &dechoState ;
	    }

	  if ( code == leftKeycode && event.xkey.type == KeyRelease)
	    {
	      leftPressed = true ;
	      pecho = &lechoState ;
	    }

	  if ( code == rightKeycode && event.xkey.type == KeyRelease)
	    {
	      rightPressed = true ;
	      pecho = &rechoState ;
	    }

	  if ( event.xkey.type == KeyRelease)
	    SDFlushKey(code, pecho) ;

	}

      //     if ( event.xkey.type == KeyRelease)
      //       SDFlushKey(code, pecho) ;
    }


  // store data for next session
  data->actionEcho = aechoState ;
  data->upEcho = uechoState ;
  data->downEcho = dechoState ;
  data->leftEcho = lechoState ;
  data->rightEcho = rechoState ;
  systemDependentData = data ;

  // inform the keyboard
  if (actionPressed)
    {
      std::cout << "evento passato" << std::endl;
      this->toInform->Action() ;
    }

  if (upPressed)
    this->toInform->GoUp() ;

  if (downPressed)
    this->toInform->GoDown() ;

  if (leftPressed)
    this->toInform->GoLeft() ;

  if (rightPressed)
    this->toInform->GoRight() ;

}


  
// Flush the events in queue
void omKeyboardDriver::SDFlushKey(long keycode, bool *echoState)
{
  // retrive data
  omX11Data *data = (omX11Data *) systemDependentData ;

  // retrive display and window
  Window root = data->rootWindow ;
  Display *display = data->display ;

  // if it is an echo disable semaphore and return
  if (*echoState)
    {
      // disable semaphore
      *echoState = false  ;
      return ;
    }


  if ( !grabKeys)
    {
      XUngrabKey(display, keycode, AnyModifier, root) ;
      XTestFakeKeyEvent(display, keycode, False, CurrentTime) ;

      XGrabKey(display, keycode, AnyModifier, root, False, GrabModeAsync, GrabModeAsync) ;
    }

  XFlush(display) ;
}

void omKeyboardDriver::SDGrabActionKeys()
{
  // do nothing under x11
}


void omKeyboardDriver::SDUngrabActionKeys()
{
  // do nothing under x11
}

void omKeyboardDriver::SDFlushKeysState()
{
  // to be implemented
}
