////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////



// in this file is implemented some  linux-dipendent stuff :))

#include <openmaia/config.h>

#include <X11/Xlib.h>
#include <X11/Xmd.h>
#include <X11/Xatom.h>

#ifdef USE_GTK
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkprivate.h>
#endif

#include <wx/wx.h>


// ---------------------- GTK code ------------------------------------------------
#ifdef USE_GTK

#define XA_WIN_SUPPORTING_WM_CHECK 	"_WIN_SUPPORTING_WM_CHECK"
#define XA_WIN_LAYER			"_WIN_LAYER"
#define XA_WIN_HINTS			"_WIN_HINTS"
#define XA_WIN_STATE			"_WIN_STATE"
#define XA_NET_STATE_STAYS_ON_TOP	"_NET_WM_STATE_STAYS_ON_TOP"
#define XA_NET_WM_STATE			"_NET_WM_STATE"
#define _NET_WM_STATE_REMOVE   0
#define _NET_WM_STATE_ADD      1
#define _NET_WM_STATE_TOGGLE   2
#define XA_NET_SUPPORTING_WM_CHECK	"_NET_SUPPORTING_WM_CHECK"


/* flags for the window layer */
typedef enum
{
	WIN_LAYER_DESKTOP = 0,
	WIN_LAYER_BELOW = 2,
	WIN_LAYER_NORMAL = 4,
	WIN_LAYER_ONTOP = 6,
	WIN_LAYER_DOCK = 8,
	WIN_LAYER_ABOVE_DOCK = 10
}
WinLayer;
#endif 
// --------------------------------------------------------------------------------



#include <omUIButton.h>


// gboolean net_wm_found(void)
// {
// 	Atom r_type, support_check;
// 	int r_format, prev_error, p;
// 	unsigned long count, bytes_remain;
// 	unsigned char *prop = NULL, *prop2 = NULL;
// 	gboolean ret = FALSE;

// 	prev_error = gdk_error_warnings;
// 	gdk_error_warnings = 0;
//         support_check = gdk_atom_intern(XA_NET_SUPPORTING_WM_CHECK, FALSE);

// 	p = XGetWindowProperty(GDK_DISPLAY(), GDK_ROOT_WINDOW(), support_check,
// 			       0, 1, False, XA_WINDOW, &r_type, &r_format,
// 			       &count, &bytes_remain, &prop);

// 	if (p == Success && prop && r_type == XA_WINDOW &&
// 	    r_format == 32 && count == 1)
// 	{
// 		Window n = *(Window *) prop;

// 		p = XGetWindowProperty(GDK_DISPLAY(), n, support_check, 0, 1,
// 				       False, XA_WINDOW, &r_type, &r_format,
// 				       &count, &bytes_remain, &prop2);
	
// 		if (p == Success && prop2 && *prop2 == *prop &&
// 		    r_type == XA_WINDOW && r_format == 32 && count == 1)
// 			ret = TRUE;
// 	}

// 	if (prop)
// 		XFree(prop);
// 	if (prop2)
// 		XFree(prop2);
// 	gdk_error_warnings = prev_error;
// 	return ret;
// }

// gboolean gnome_wm_found(void)
// {
// 	Atom r_type, support_check;
// 	int r_format, prev_error, p;
// 	unsigned long count, bytes_remain;
// 	unsigned char *prop = NULL, *prop2 = NULL;
// 	gboolean ret = FALSE;

// 	prev_error = gdk_error_warnings;
// 	gdk_error_warnings = 0;
// 	support_check = gdk_atom_intern(XA_WIN_SUPPORTING_WM_CHECK, FALSE);
	
// 	p = XGetWindowProperty(GDK_DISPLAY(), GDK_ROOT_WINDOW(), support_check,
// 			       0, 1, False, XA_CARDINAL, &r_type, &r_format,
// 			       &count, &bytes_remain, &prop);

// 	if (p == Success && prop && r_type == XA_CARDINAL &&
// 	    r_format == 32 && count == 1)
// 	{
// 		Window n = *(long *) prop;

// 		p = XGetWindowProperty(GDK_DISPLAY(), n, support_check, 0, 1,
// 				       False, XA_CARDINAL, &r_type, &r_format,
// 				       &count, &bytes_remain, &prop2);

// 		if (p == Success && prop2 && r_type == XA_CARDINAL &&
// 		    r_format == 32 && count == 1)
// 			ret = TRUE;
// 	}

// 	if (prop)
// 		XFree(prop);
// 	if (prop2)
// 		XFree(prop2);
// 	gdk_error_warnings = prev_error;
// 	return ret;


// }


// ---------------------- GTK code ------------------------------------------------
#ifdef USE_GTK
void net_wm_set_property(GtkWidget * window, char *atom, gboolean state)
{
#ifdef GINO
	XEvent xev;
	gint prev_error;
	int set = _NET_WM_STATE_ADD;
	Atom type, property;

	if (state == FALSE)
		set = _NET_WM_STATE_REMOVE;

	type = gdk_atom_intern(XA_NET_WM_STATE, FALSE);
	property = gdk_atom_intern(atom, FALSE);

	prev_error = gdk_error_warnings;
	gdk_error_warnings = 0;

	xev.type = ClientMessage;
	xev.xclient.type = ClientMessage;
	xev.xclient.window = GDK_WINDOW_XWINDOW(window->window);
	xev.xclient.message_type = type;
	xev.xclient.format = 32;
	xev.xclient.data.l[0] = set;
	xev.xclient.data.l[1] = property;
	xev.xclient.data.l[2] = 0;
	
	XSendEvent(GDK_DISPLAY(), GDK_ROOT_WINDOW(), False,
		   SubstructureNotifyMask, (XEvent *) &xev);

	gdk_error_warnings = prev_error;
#endif // GINO

}
#endif
// --------------------------------------------------------------------------------


// static void gnome_wm_set_window_always(GtkWidget * window, gboolean always)
// {
// 	XEvent xev;
// 	long layer = WIN_LAYER_ONTOP;
// 	Atom xa_win_layer = gdk_atom_intern("_WIN_LAYER", FALSE);

// 	if (always == FALSE)
// 		layer = WIN_LAYER_NORMAL;

// 	if (GTK_WIDGET_MAPPED(window))
// 	{
// 		xev.type = ClientMessage;
// 		xev.xclient.type = ClientMessage;
// 		xev.xclient.window = GDK_WINDOW_XWINDOW(window->window);
// 		xev.xclient.message_type = xa_win_layer;
// 		xev.xclient.format = 32;
// 		xev.xclient.data.l[0] = layer;

// 		XSendEvent(GDK_DISPLAY(), GDK_ROOT_WINDOW(), False,
// 			   SubstructureNotifyMask, &xev);
// 	}
// 	else
// 	{
// 		long data[1];

// 		data[0] = layer;
// 		XChangeProperty(GDK_DISPLAY(),
// 				GDK_WINDOW_XWINDOW(window->window),
// 				xa_win_layer, XA_CARDINAL, 32, PropModeReplace,
// 				(unsigned char *) data,	1);
// 	}
// }



// // you can find the original implementation of this class in ../common/graphics.cpp
// // put the window on top using X11 call to the window manager
// // I tried it on kde3



// //void omUIButton::SDPutOnTop()
// //{
// // //    XEvent xev ;
// // //    Atom type , proper;

// // //   char *display_name ;
// // //   Display *display ;

// // //   if (! (display_name = getenv("DISPLAY")) ){
// // //     cerr << "environment variable DISPLAY must be set" << endl ;
// // //     exit(-1);
// // //   }
// // //   if (! (display = XOpenDisplay(display_name)) ){
// // //     cerr << "I can't open display" << endl ;
// // //     exit(-1);
// // //   }


// //   //GtkWidget *h = (GtkWidget *) this->GetHandle() ;   // get the handle of this window
// //    GtkWidget *h =  this->GetHandle() ;   // get the handle of this window

// //    //gdk_window_raise( h->window);

// //    // copied from xmms sources :))
// // //    type = gdk_atom_intern("_NET_WM_STATE", FALSE);
// // //    proper = gdk_atom_intern("_NET_WM_STATE_STAYS_ON_TOP",FALSE) ;

// // //    xev.type = ClientMessage ;
// // //    xev.xclient.window = GDK_WINDOW_XWINDOW( h->window) ;
// // //    xev.xclient.message_type = type ;
// // //    xev.xclient.format = 32 ;
// // //    xev.xclient.data.l[0] = 1 ;
// // //    xev.xclient.data.l[1] = proper ;
// // //    xev.xclient.data.l[2] = 0 ;

   
// // //    XSendEvent( GDK_DISPLAY(), GDK_ROOT_WINDOW(), FALSE,
// // // 	SubstructureNotifyMask, (XEvent *) &xev ) ;


// // //    if ( gnome_wm_found())
// // //     {

// // 	    XEvent xev;
// // 	    int prev_error;
// // 	    long layer = WIN_LAYER_ONTOP;
// // 	    Atom xa_win_layer = gdk_atom_intern(XA_WIN_LAYER, FALSE);

	
// // 	    prev_error = gdk_error_warnings;
// // 	    gdk_error_warnings = 0;

// // 	    //std::cout << "gnome" << std::endl ;

// // 	    if (GTK_WIDGET_MAPPED(h->window))
// // 	      {
// // 		//std::cout << "uno" << std::endl ;
// // 		xev.type = ClientMessage;
// // 		xev.xclient.type = ClientMessage;
// // 		xev.xclient.window = GDK_WINDOW_XWINDOW( h->window);
// // 		xev.xclient.message_type = xa_win_layer;
// // 		xev.xclient.format = 32;
// // 		xev.xclient.data.l[0] = layer;

// // 		XSendEvent(GDK_DISPLAY(), GDK_ROOT_WINDOW(), False,
// // 			   SubstructureNotifyMask, (XEvent *) & xev);
// // 	      }
// // 	    else
// // 	      {
// // 		//std::cout << "due" << std::endl ;
// // 		long data[1];

// // 		data[0] = layer;
// // 		XChangeProperty(GDK_DISPLAY(),
// // 				GDK_WINDOW_XWINDOW( h->window) ,
// // 				xa_win_layer, XA_CARDINAL, 32, PropModeReplace,
// // 				(unsigned char *) data,	1);
// // 		}
// // 	    gdk_error_warnings = prev_error;

// // 	    //XFlush(display);

// // // }
// // // 	else if (net_wm_found())
// // // 	  {
// // // 	    std::cout << "wm" << std::endl ;
// // // 	    	net_wm_set_property(h, XA_NET_STATE_STAYS_ON_TOP, TRUE);
// // // 	  }

// // }


void omUIButton::SDPutOnTop()
{
// ---------------------- GTK code ------------------------------------------------
#ifdef USE_GTK
  GtkWidget *h = (GtkWidget *) this->GetHandle() ;

  //gnome_wm_set_window_always(h, true);
  net_wm_set_property(h, "_NET_WM_STATE_ABOVE", true);
#endif
// --------------------------------------------------------------------------------
}



// use X11 api to check if this window has wh as window handler
bool omUIButton::SDIsYourWindow( void *wh)
{
// ---------------------- GTK code ------------------------------------------------
#ifdef USE_GTK
  GtkWidget *h = (GtkWidget *) this->GetHandle() ;   // get the handle of this window
  
  Window *wwh = (Window *) wh ;
  Window toCheck = *wwh ;
  Window thisHandler = GDK_WINDOW_XWINDOW( h->window) ;

  if ( thisHandler == toCheck)
    return true ;
  else
    return false ;
#endif
// --------------------------------------------------------------------------------
}


// play a sound 
// wave - relative path of the sound to play
// isRelative - true if path is relative to keyboard root
void omUIButton::PlaySoundSD(wxString wave, bool isRelative)
{
  
//   wxString currentPath = omFunctions::GetCurrentPath();
//   currentPath = omFunctions::GetWindowsString(currentPath);

//   wxString toPlay = _T("");
//   if (isRelative)
//       toPlay << omFunctions::GetWindowsString(wxString(currentPath)) << _T("\\keyboard\\") ;

//   toPlay << omFunctions::GetWindowsString(wave);
//   std::cout << "sound to play= " << toPlay << std::endl;
//   ::PlaySound( toPlay.c_str(), NULL, SND_ASYNC | SND_FILENAME) ;
}


