// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

// Implement the X11 dependen check the window with focus

#include <openmaia/config.h>
#include <openmaia/omUIGrantFocus2User.h>

#include "omX11Data.h"


#include <iostream>
#include <X11/keysym.h>
#include <X11/extensions/XTest.h>

#ifdef USE_GTK
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkprivate.h>
#endif

void omUIGrantFocus2User::SDConstructor()
{
  Display *display ;
  Window root ;

  omX11Data *data = new omX11Data() ;

  char *display_name ;

  if (! (display_name = getenv("DISPLAY")) ){
    std::cerr << "environment variable DISPLAY must be set" << std::endl ;
    exit(-1);
  }
  if (! (display = XOpenDisplay(display_name)) ){
    std::cerr << "I can't open display" << std::endl ;
    exit(-1);
  }


  // store informations in an object structure for future retrival
  data->display = display ;
  

  // store the data as a void pointer
  this->sdData = data ;

  // lastFocusedWindow is NULL if no non maia window is focused
  lastFocusedWindow = NULL ;

  
}


void omUIGrantFocus2User::SDCheckFocusedWindow()
{
  // Call the X11 api to find wich windows is focused
  Window currFocus ;
  int revert_to ;

  // call back x11 data
  omX11Data *data = (omX11Data *) this->sdData ;
  Display *display = data->display ;

  XGetInputFocus(display, &currFocus, &revert_to) ;

  // check if passed handler belongs to maia tree by querying to the application 
  // keyboard if exist. Assume the window non belongs for default
  omUIKeyboard *keyb = (omUIKeyboard *) wxGetApp().objects.keyboard ;
  bool isMaiaWindow = false ;
  if ( keyb != NULL ) 
    isMaiaWindow = keyb->IsYourWindow(&currFocus) ;
  if (!isMaiaWindow) 
    {
      // use data->RootWindow to store the last focused window structure
    data->rootWindow = currFocus ;
    lastFocusedWindow = &(data->rootWindow) ;
    }
}

// set the focus to the last focused window
void omUIGrantFocus2User::SDSetLastFocusedWindow()
{ 
  // call back x11 data
  omX11Data *data = (omX11Data *) this->sdData ;
  Display *display = data->display ;

  //XGetInputFocus(display, &currFocus, &revert_to) ;
  // last focused window is stored a rootWindow in data
  
  
  if ( data->rootWindow != 0)
    XSetInputFocus(display, data->rootWindow, RevertToPointerRoot, CurrentTime) ;
   
}
