////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////



#include <iostream>
#include <string>

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <getopt.h>


#include <wx/wx.h>
#include <wx/image.h>

#include <openmaia/config.h>
#include <openmaia/libopenmaia.h>
#include <openmaia/libopenmaiaui.h>

#include "maia.h"



IMPLEMENT_APP(omUIApplication)

bool omUIApplication::OnInit()
{
  // options flags
  bool setDisableSerial=false;
  bool setHostName=false;
  bool setPort=false;
  bool setNoConnect=false;

  // optional parameters
  wxString hostName;
  int portnumber;
  wxString binPath(argv[0]);

  printf("argv prima %s\n",argv[0]) ;

  // profile name
  wxString profile = _T("default") ;

  int option_index = 0;
  while(1)
    {
      int c ;
      int this_option_optind = optind ? optind : 1;

      wxString cur_option, cur_parameter;

      static struct option long_options[] = {
	{"help", 0, 0, 0},
	{"disable-serial", 0, 0, 0},
	{"host",1, 0, 0},
	{"port",1, 0, 0},
	{"no-connect",0, 0, 0},
	{"profile",1, 0, 0},
	{0, 0, 0, 0}
      };

      c = getopt_long (argc, (char **)argv, "",
		       long_options, &option_index);
      if (c == -1)
	break;


      switch(c)
	{

	case 0:   // registered option found
	  cur_option = wxString(long_options[option_index].name) ;

	  if (optarg)
	    cur_parameter = wxString(optarg) ;

	  break;

	default:
	  std::cerr << "Option not found" << std::endl ;

	}

      // help option
      if (cur_option.CmpNoCase(_T("help")) == 0)
	{
	  std::cerr << "Open Maia " << VERSION << " Suite" << std::endl <<
	    "Maia User Interface Agent" << std::endl << std::endl <<
	    "Usage: maia [options] [<profile name>]" << std::endl <<
	    "--host=<host name>\t\tSet host name that run openmaia server" << std::endl <<
	    "--port=<port number>\t\tSet port number other than 60201" << std::endl <<
	    "--no-connect\t\t\tDon't connecto to openmaia server" << std::endl <<
	    "--disabile-serial\t\tDisable serial port handling" << std::endl <<
	    "--profile=<profile name>\t\tStart with specified profile" << std::endl <<
	    std::endl ;
	  exit(-1) ;
	}

      // disble-serial option
      if (cur_option.CmpNoCase(_T("disable-serial")) == 0)
	setDisableSerial = true ;

      // host option
      if (cur_option.CmpNoCase(_T("host")) == 0)
	if (cur_parameter.CmpNoCase(_T("")) != 0)
	  {
	    setHostName = true ;
	    hostName = wxString(cur_parameter) ;
	  }

      // port option
      if (cur_option.CmpNoCase(_T("port")) == 0)
	if (cur_parameter.CmpNoCase(_T("")) != 0)
	  {
	    setPort=true ;
	    long p;
	    cur_parameter.ToLong(&p);
	    portnumber = (int) p;
	  }

      if (cur_option.CmpNoCase(_T("profile")) == 0)
	if (cur_parameter.CmpNoCase(_T("")) != 0)
	  {
	    profile = wxString(cur_parameter) ;
	  }

    }

  if ( optind < argc ) {
    wxString parameter = _T(argv[option_index+1]) ;
    if (parameter.CmpNoCase(_T("moo")) == 0)
      {
#ifdef LINUX_SYSTEM
	std::cerr << "         (__)\n         (oo)\n   /------\\/\n  / |    ||\n *  /\\---/\\\n    ~~   ~~\n...Have you mooed today?\n" ;
#endif
#ifdef WINDOWS_SYSTEM
	std::cerr << "\n        ((oo))\n   /------\\/\n  / |    ||\n *  /\\---/\\\n    ~~   ~~\n...Have you mooed today?\n" ;
#endif
	exit(-2); 
      }
    else
      profile = parameter ;
  }

  // add handlers to graphics file formats
  wxImage::AddHandler( new wxBMPHandler ) ;
  wxImage::AddHandler( new wxJPEGHandler) ;
  wxImage::AddHandler( new wxPNGHandler) ;
  wxImage::AddHandler( new wxGIFHandler) ;
  wxImage::AddHandler( new wxTIFFHandler) ;

  printf("argv prima %s\n",argv[0]) ;

  bool inDeveloperTree = false;
  // check for development tree
  if ( omFunctions::TestFolder(_T("../../data/default/maia")))
    {
      chdir("../../data/default/maia") ;
      inDeveloperTree=true;
    }
  else
    {
      // check for presence of default maia profile
      bool personalInstallation = false;

      personalInstallation = omFunctions::TestPersonalInstallation(_T("default"),_T("maia"), binPath) ;

      if (!personalInstallation)
	omFunctions::CreateDefaultInstallation(_T("maia"), binPath);

      // set the path to openamaia application root
      wxString pathToSet = profile + _T("/maia") ;

      SetPath(pathToSet) ;
    }


  // set all error handlers
  SetErrorHandlers();

  // set parent frame for all windows
  wxFrame *mainFrame = NULL ;

  // under x11 must be a main frame parent of all windows
#ifdef LINUX_SYSTEM
  mainFrame = new wxFrame( NULL, -1, L"Kext - Extended Keyboard", wxPoint(0,0), wxSize(1,1), 
			    wxSIMPLE_BORDER | wxFRAME_FLOAT_ON_PARENT| wxFRAME_NO_TASKBAR) ;
  mainFrame->Show() ; 
#endif

  // declare all application objects

  maiaOptions *appOptions = new maiaOptions() ;  // options object
  appOptions->LoadOptions(_T("maia.conf")) ;   // load default options for application
  omUIActionQueue *appQueue = new omUIActionQueue(50) ;  // event queue checker 
  maiaKeyboard *maia=new maiaKeyboard(mainFrame, appOptions->scanTime);  // Virtual Keyboard 
  omUIGrantFocus2User *checkFocus = new omUIGrantFocus2User(30) ;  // Focus granter
  maiaCommandParser *commandParser = new maiaCommandParser();   // command parser

  appOptions->inDeveloperTree = inDeveloperTree;

  // set options value in function of command line
  if (setDisableSerial)
    appOptions->serialEnabled=false ;
  if (setHostName)
    appOptions->hostName = hostName;
  if (setPort)
    appOptions->port = portnumber;


  omConnector *connector = new omConnector(commandParser, appOptions->hostName, appOptions->port) ;
  if (!setNoConnect)
    connector->Connect();   // try to connect to the server if running


  // ------------ interclass message handling initialization
  objects.focusChecker = checkFocus ;
  objects.options = appOptions ;
  objects.actionsQueue = appQueue ;
  objects.keyboard = maia ;
  objects.commandParser = commandParser ;
  objects.connector = connector ;

  // init all objects
  objects.Init() ;

  // ------------- begin progam operations ---------------
  // load default keyboard
  maia->LoadKeyboard(_T("keyboard.xml")) ;
  // start the keyboard
  maia->Show() ;
  //start all timed objects
  appQueue->Start() ;
  commandParser->Start() ;
  maia->Start() ;
  checkFocus->Start() ;

  

  return TRUE ;

}
