////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#ifndef _maiaButton_
#define _maiaButton_

#include <wx/wx.h>
#include <string>

#include <openmaia/libopenmaia.h>
#include <openmaia/libopenmaiaui.h>


#include "maiaButtonData.h"

/** @ingroup maia
    This class extends omUIButton to add some functionalities and semplification.
    In particualr this button handles tree images used to store button figure in tree states:
    <ul>
      <li>normal: Button displayed in normal status
      <li>scanned: Button slected by automatic scan or cursor scan
      <li>selected: Button pressed
    </ul>

    This button handles two sounds:
    <ul>
      <li>scanSound: sound played when a button is selected by automatic scan
      <li>selSound: Sound played when a button is selected
    </ul>
*/

class maiaButton: public omUIButton {

 private:
  // private variables
  int normalIndex, selIndex, scanIndex ;   ///< Figure index.
  int scanSoundIndex, selSoundIndex ; ///< Sound index.
  bool haveScanSound, haveSelSound ;  ///< Flag.
  maiaButtonData *buttonData ;    ///< Data associated with the button.

  bool isCurrent ;   ///< Flag.

  void RenderPushButton() ;
  void SendSimulation() ;


  DECLARE_EVENT_TABLE()

 public:
  maiaButton( wxWindow *, bool, maiaButtonData *) ; 


  void Show() ;

  void Clean();      // paint the normal button
  void Select();     // make the button selected
  void Scan();       // make the button scanned
  void OnClick();  //implement virtual funtion of the super
  void OnFocus() ; // implement virual on focus function of the super

  void PaintDefaultPicture() ;   // call the Clean() method
  void LightOn() ;  // call the Scan() method, used in the automatic scan mode
  void LightOnSingle() ;  // call the Scan() method, used in the automatic scan mode
  void LightOff() ;  // call the Clean() method, used in the automatic scan mode
  void Execute() ;  // inser command code in the queue, used in automatic scan mode
  void SetCurrent(); // set this node as the current
  void ResetCurrent() ;  // reset this not as the current

} ;


#endif // _maiaButton_

