////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include "maiaButtonData.h"

/** Inizialize all variables */
maiaButtonData::maiaButtonData()
{
  // initialize private variables
  nActions = 0 ;
  currAction = 0 ;

  x = 0;
  y = 0 ;
  w = 0 ;
  h = 0 ;

  name = wxString(_T("")) ;

  normalImage = wxString(_T("")) ;
  scanImage = wxString(_T("")) ;
  selImage = wxString(_T("")) ;

  selSound = wxString(_T("")) ;
  scanSound = wxString(_T(""));
  
}

/** Add a new action to the dinamic vector
    @param a pointer to action to add
*/
void maiaButtonData::AddAction(omCommand *a)
{
  actions.insert(actions.end(), a) ;
  nActions++ ;
}

/** Get the next action in the list 
    @return next action name, if reached end of vector return a NULL Pointer
*/
omCommand *maiaButtonData::GetNextAction()
{
  currAction++ ;
  if ( currAction <= nActions)
    return actions[currAction-1] ;
  else
    return NULL ;     // return a void string if end of vector reached
}

/* Return true if you can fetch more actions */
bool maiaButtonData::MoreActions()
{
  if ( currAction <= (nActions-1))  // at least another action to fetch
    return true ;
  else
    return false ;
}

/** Reset the action fetch counter */
void maiaButtonData::ResetActionFetchCounter()
{
  currAction = 0 ;
}
