////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#ifndef _maiaButtonData_
#define _maiaButtonData_

#include <openmaia/libopenmaia.h>
#include <openmaia/libopenmaiaui.h>

#include <wx/wx.h>
#include <vector>

/** @ingroup maia
    This class is used to store data associated with the MaiaButton class. */

class maiaButtonData: public omObject {
 private:
  int nActions ;  ///< number of stored actions
  int currAction ;  ///< current action in next action fetch mode

 public:
  wxString name ; ///< Button name.
  int x,y,w,h ;  ///< Position and dimension of the button.
  wxString normalImage, scanImage, selImage ; /// Images names of the button.
  wxString selSound, scanSound;  ///< Sounds names of the button.
  //  wxString code;  // simulation code
  std::vector<omCommand *> actions ;   ///< Actions associated with the button.
  

  maiaButtonData() ;
  void AddAction(omCommand *) ;
  omCommand *GetNextAction() ;
  bool MoreActions();
  void ResetActionFetchCounter();

} ;

#endif // _maiaButtonData_
