////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#ifndef _maiaKeyboard_
#define _maiaKeyboard_


#include <openmaia/libopenmaia.h>
#include <openmaia/libopenmaiaui.h>

#include <omcommon.h>

#include "maiaButton.h"
#include "maiaButtonData.h"
#include "maiaOptions.h"
#include "maiaXMLTags.h"
#include "maiaOptions.h"


#include <unistd.h>

#include <wx/wx.h>


/** @ingroup maia

    This class handles a maia keyboard with some MaiaButtons objects stored
    in a tree structure.
    The class is an implementation of MaiaGenericKeyboard class that is gives
    general methods and structures.
    Virtual methods of MaiaGenericKeyboard are implemented here.
*/

class maiaKeyboard: public omUIKeyboard, public wxTimer{
 private:
  bool lastAutoscan, lastJoystick;  ///< Last status of devices actiovation
  bool autoscan, joystick;  ///< Current status of devices actiovation
  int scanTick ;  ///< scan tick in milliseconds

  bool reachKey ;   ///< true if in reaching key data state
  maiaButtonData *currentKeyData ;  ///< used when fill the tree
  wxWindow *parentWindow ;  ///< parent window of all buttons
  

  //Needed in LinkConnectedButtons()
  std::vector<omUIButton *> buttons; ///< Contain list of buttons. Filled in AtEndTag()
  

  omObject *currentButton ;   ///< Pointer to current button selected by the cursor
  maiaOptions *opts;          ///< Application options


  // ------------------------ private functions
  void Notify() ;
  void LinkConnectedButtons();

 public:
  maiaKeyboard(wxWindow *, int) ;
  void Init();
  void Reload();
  // methods used by the parser
  void AtStartDocument() ;  
  void AtEndDocument() ;
  void AtStartTag( wxString, omXMLParameters ) ;
  void AtEndTag( wxString ) ;
  // method used by the automatic scan system
  void Start(); // start automatic scan
  void Stop() ;  // stop automatic scan
  //  void Action() ; // pass the action signal to the tree
  //  void Tick() ;   // force the tick action from the extern
  void SetCurrentButton( omObject *) ;   // set the current selected node
  omObject *GetCurrentButton() ;
  void Action() ;
  void GoDown() ;
  void GoUp() ;
  void GoLeft() ;
  void GoRight() ;
  void LoadKeyboard(wxString) ;  // load the keyboard
} ;



#endif // _maiaKeyboard_
