////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#include <iostream>
#include <unistd.h>

#include "maiaOptions.h"


/** The constructor */
maiaOptions::maiaOptions()
  : omUIOptions()
{
}



// implement parser functions

/** See MaiaGenericXMLParser documentation */
void maiaOptions::AtStartDocument()
{

}

/** See MaiaGenericXMLParser documentation */
void maiaOptions::AtEndDocument()
{
}

/** See MaiaGenericXMLParser documentation */
void maiaOptions::AtStartTag( wxString tagname, omXMLParameters p)
{
  omXMLParameters *pars = &p ;
  wxString n = pars->GetValue(wxString(_T("name"))) ;
  wxString v =  pars->GetValue(wxString(_T("value"))) ;

  wxString tname = omFunctions::ConvertToLower(tagname) ;

  if ( tname == wxString(MAIAXMLOPTIONTAG)) {
    //std::cout << "opzione " << pars->GetLowerValue("name") << " - valore " << 
    //  pars->GetLowerValue("value") << std::endl ;

    // ---- HARDWARE OPTIONS ---
    if ( n.CmpNoCase( MAIAXMLSERIALENABLEDTAG) == 0 ) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE) == 0 )
	serialEnabled = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	serialEnabled = false ;
    }

    if ( n.CmpNoCase(MAIAXMLSERIALPORTTAG) == 0 )
      serialPort = v ;

    if ( n.CmpNoCase(MAIAXMLKEYBOARDENABLEDTAG) == 0 ) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE) == 0 )
	keyboardEnabled = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	keyboardEnabled  = false ;
    }

    // ---- GENERAL OPTIONS ----
    if ( n.CmpNoCase(MAIAXMLAUTOSCANTAG) == 0) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE ) == 0)
	autoscan = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	autoscan = false ;
    }

    if ( n.CmpNoCase(MAIAXMLENABLESOUNDTAG) == 0) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE ) == 0)
	soundEnabled = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	soundEnabled = false ;
    }


    if ( n.CmpNoCase(MAIAXMLJOYSTICKTAG) == 0 ) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE) == 0 )
	joystick = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	joystick = false ;
    }

    if ( n.CmpNoCase(MAIAXMLDIRECTTAG) == 0) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE) == 0 )
	direct = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	direct = false ; 
    }

    if ( n.CmpNoCase(MAIAXMLACTIONKEYTAG) == 0) {
      long value = omFunctions::GetMaiaKeycodeFromXML( v ) ;
      if ( value != -1)
	actionKey = value ;
    }

    if ( n.CmpNoCase(MAIAXMLUPKEYTAG) == 0) {
      long value = omFunctions::GetMaiaKeycodeFromXML( v ) ;
      if ( value != -1)
	upKey = value ;
    }

    if ( n.CmpNoCase(MAIAXMLDOWNKEYTAG) == 0) {
      long value = omFunctions::GetMaiaKeycodeFromXML( v ) ;
      if ( value != -1)
	downKey = value ;
    }

    if ( n.CmpNoCase(MAIAXMLLEFTKEYTAG) == 0) {
      long value = omFunctions::GetMaiaKeycodeFromXML( v) ;
      if ( value != -1)
	leftKey = value ;
    }

    if ( n.CmpNoCase(MAIAXMLRIGHTKEYTAG) == 0) {
      long value = omFunctions::GetMaiaKeycodeFromXML( v) ;
      if ( value != -1)
	rightKey = value ;
    }

    if ( n.CmpNoCase(MAIAXMLGRABKEYSTAG) == 0 ) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE) == 0 )
	grabKeys = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	grabKeys = false ;
    }

    if (n.CmpNoCase(MAIAXMLONPRESSTAG) == 0 ) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE) == 0 )
	actionOnPress = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	actionOnPress = false ;
    }


    //  ------ SCAN OPTIONS ----
    if (n.CmpNoCase(MAIAXMLSCANTIME) == 0 ) 
      scanTime = pars->GetIntValue(wxString(_T("value"))) ;

    if (n.CmpNoCase(MAIAXMLSCANLOOPSTAG) == 0 ) 
      scanLoopsNumber = pars->GetIntValue(wxString(_T("value"))) ;

    if (n.CmpNoCase(MAIAXMLDELAYAFTERCLICKTAG) == 0 ) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE) == 0 )
	delayAfterClick = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	delayAfterClick = false ;
    }

    if (n.CmpNoCase(MAIAXMLDELAYAFTERACTION) == 0 ) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE) == 0 )
	delayAfterAction = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	delayAfterAction = false ;
    }

    if (n.CmpNoCase(MAIAXMLDELAYAFTERENDBLOCK) == 0 ) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE) == 0 )
	delayAfterEndBlock = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	delayAfterEndBlock = false ;
    }

    if (n.CmpNoCase(MAIAXMLSCANRESUMEFROMTAG) == 0 ) {
      if ( v.CmpNoCase(MAIAXMLTHISVALUE) == 0 )
	scanResumeFrom = RESUMEFROMTHIS ;
      if ( v.CmpNoCase(MAIAXMLFIRSTVALUE) == 0 )
	scanResumeFrom = RESUMEFROMFIRST ;
      if ( v.CmpNoCase(MAIAXMLNEXTVALUE) == 0 )
	scanResumeFrom = RESUMEFROMNEXT ;
    }

    if (n.CmpNoCase(MAIAXMLEXITBLOCKAFTERCLICK) == 0 ) {
      if ( v.CmpNoCase(OMXMLTRUEVALUE) == 0 )
	exitBlockAfterClick = true ;
      if ( v.CmpNoCase(OMXMLFALSEVALUE) == 0 )
	exitBlockAfterClick = false ;
    }

  }
}

/** See MaiaGenericXMLParser documentation */
void maiaOptions::AtEndTag( wxString tname)
{

}


/** Load specific option file
    @param fname option file name
*/
void maiaOptions::LoadOptions( wxString fname)
{
  // change to options dir
  chdir("etc") ;
  // start the parser
  this->Parse(fname) ;
  // change to base dir
  chdir("..") ;
}

/** Save options to a file.
    @param fileName Name of the file to save
*/
void maiaOptions::SaveOptions( wxString fileName)
{
  wxString xmlString = this->GenerateOptionsXMLString();

  // if in dev tree don't save options
  if (inDeveloperTree)
    return;

  // save the file
  // change to options dir
  chdir("etc") ;
  std::ofstream optionsFile(fileName.c_str());
  optionsFile << xmlString ;

  chdir("..");
}

/** Generate Options XML string.
    Documentation taken from parent. */
wxString maiaOptions::GenerateOptionsXMLString()
{
  // get parent options
  wxString xmlString = omUIOptions::GenerateOptionsXMLString();

  // maiaOptions has no more options than omUIOptions
  return xmlString;
}
