////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#include "maiaconfig.h"

IMPLEMENT_APP(maiaconfigMain)


/** Main method of the application */
bool maiaconfigMain::OnInit()
{
  
  // initialize variables;
  host = _T("localhost");
  port = 60201;
  profile = _T("default");


  // call parent OnInit to initialize command parser
  if (!wxApp::OnInit())
    return false;


   binPath = wxString(argv[0]);
   inDeveloperTree = false;

  // add handler to use images.
  wxImage::AddHandler(new wxXPMHandler);
  wxImage::AddHandler( new wxPNGHandler);
    
  // initialize xrc system
  wxXmlResource::Get()->InitAllHandlers();    
      

  // wxCmdLineParser parser(argc,argv);
//   OnInitCmdLine(parser);
  
  //parser.Parse();

  // find resource file
  
  wxString resourceFileName = _T("maiaconfig") ;

#ifdef WINDOWS_SYSTEM
  resourceFileName << _T("_win") ;
#endif

#ifdef LINUX_SYSTEM
  resourceFileName << _T("_gtk") ;
#endif

  resourceFileName << _T("_it.xrc") ;

  wxString resourceFile = _T("../../data/resource/maiaconfig/") + resourceFileName;

  if ( omFunctions::TestFile(resourceFile)) 
    {
      inDeveloperTree = true;
    }
  else
    {
      resourceFile = omFunctions::GetSharedPath(binPath) + _T("/resource/maiaconfig/") + resourceFileName ;
    }

  // load resource file
  wxXmlResource::Get()->Load(resourceFile) ;

  // instantiate main frame.
  maiaconfigFrame *frame = new maiaconfigFrame(NULL);
    
  // Show the frame.
  frame->Show(TRUE);

  std::cout << "maiaconfig: frame mostrato" << std::endl;

//   // iconize by default
//   frame->Iconize();
    
  // Return TRUE to tell program to continue (FALSE would terminate).
  return TRUE;
}


/** Reached exit of the application */
int maiaconfigMain::OnExit()
{
  std::cout << "uscita raggiunta" << std::endl;
}


void maiaconfigMain::OnInitCmdLine(wxCmdLineParser &parser)
{
  //wxApp::OnInitCmdLine(parser);

  wxCmdLineEntryDesc cmdLineDesc[] =
    {
      { wxCMD_LINE_SWITCH, _T("h"), _T("help"), _T("show this help message"),
	       wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP },

      { wxCMD_LINE_OPTION, NULL, _T("host"), _T("Messages server name") },

      { wxCMD_LINE_OPTION, NULL, _T("port"), _T("Messages server port "), wxCMD_LINE_VAL_NUMBER },

      { wxCMD_LINE_OPTION, NULL, _T("profile"), _T("Profile to load") },

      { wxCMD_LINE_NONE }
    };

  parser.SetDesc(cmdLineDesc);
  

//       { wxCMD_LINE_OPTION, _T("o"), _T("output"),  _T("output file") },
//       { wxCMD_LINE_OPTION, _T("i"), _T("input"),   _T("input dir") },
//       { wxCMD_LINE_OPTION, _T("s"), _T("size"),    _T("output block size"),
// 	wxCMD_LINE_VAL_NUMBER },
//       { wxCMD_LINE_OPTION, _T("d"), _T("date"),    _T("output file date"),
// 	wxCMD_LINE_VAL_DATE },

//       { wxCMD_LINE_PARAM,  NULL, NULL, _T("input file"),
// 	wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_MULTIPLE },
}



bool maiaconfigMain::OnCmdLineError(wxCmdLineParser& parser)
{
  return true;
}

bool maiaconfigMain::OnCmdLineHelp(wxCmdLineParser& parser)
{
  wxApp::OnCmdLineHelp(parser);
  return false;
}

bool maiaconfigMain::OnCmdLineParsed(wxCmdLineParser& parser)
{
  //  wxApp::OnCmdLineParsed(parser);
  
  wxString stringBuffer;
  long longBuffer;

  std::cout << "finito di parserizzare" << std::endl ;

  if (parser.Found(_T("host"), &stringBuffer))
    {
      host = stringBuffer;
      std::cout << "parametro host" << std::endl;
    }

  if (parser.Found(_T("port"), &longBuffer))
    port = (int) longBuffer;

  if (parser.Found(_T("profile"), &stringBuffer))
    profile = stringBuffer;
}
