////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or (at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of                              
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                                
//  GNU General Public License for more details.                                                
//  You should have received a copy of the GNU General Public License                           
//  along with this program; if not, write to the Free Software                                 
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                                   
////////////////////////////////////////////////////////////////////////////////


#include "maiaconfigFrame.h"



 
/** Build maiaconfigFrame class.
    The constructor builds the class and load maiaconfigFrame frame resource.
    The resource must be initialized by the caller of the class.

    @param parent parent window of the frame (may be null)
*/
maiaconfigFrame::maiaconfigFrame(wxWindow* parent)
  //    : wxFrame(parent, -1,_T("") ,
  //  	    wxPoint(100,100), wxSize(355, 250), wxDEFAULT_FRAME_STYLE & !wxRESIZE_BORDER)
{
   wxXmlResource::Get()->LoadFrame(this, parent, wxT("maiaconfigFrame"));

   

  this->SetTitle(wxString("maia control panel - ver. " + wxString(_T(VERSION))));

  // workaround windows problem
#ifdef WINDOWS_SYSTEM
  this->SetSize(355,325);
#endif

  // find position of resource file
  wxString resourceFile = _T("keyboard");

#ifdef LINUX_SYSTEM
  resourceFile << _T("_gtk");
#endif

#ifdef WINDOWS_SYSTEM
  resourceFile << _T("_win");
#endif

  /// @todo fix language selection
  resourceFile << _T("_it.xrc");

  wxString resourceFilePath;

  if ( wxGetApp().inDeveloperTree)
    resourceFilePath = _T("../../data/resource/common/") + resourceFile;
else
  resourceFilePath = omFunctions::GetSharedPath(wxGetApp().binPath) + _T("/resource/common/") + resourceFile;


#ifdef MAIA_DEBUG
  std::cout << "maiaconfigFrame: resourceFile= " << resourceFilePath << std::endl;
#endif

  // initially not to save
  toSave = false;

  //getKeyDialog = new omGetKeyDialog("../../data/resource/common/keyboard_gtk_it.xrc");
  getKeyDialog = new omGetKeyDialog(resourceFilePath);

  //Open connection with OpenMaia server
  host=wxGetApp().host;
  port = wxGetApp().port;

  connector=new omConnector(this, host, port);
  connector->Connect();

  // initialize pointers to frame objects
  // checkboxes
  autoscan = XRCCTRL(*this, "autoscan", wxCheckBox);
  joystick = XRCCTRL(*this, "joystick", wxCheckBox);
  direct = XRCCTRL(*this, "direct", wxCheckBox);
  actionOnPress = XRCCTRL(*this, "actionOnPress", wxCheckBox); 
  enableKeyboard = XRCCTRL(*this, "enableKeyboard", wxCheckBox);
  grabKeys = XRCCTRL(*this, "grabKeys", wxCheckBox);
  enableJoystick = XRCCTRL(*this, "enableJoystick", wxCheckBox);
  enableSerial = XRCCTRL(*this, "enableSerial", wxCheckBox);

  // text controls
  actionKeyDisplay = XRCCTRL(*this, "actionKeyDisplay", wxTextCtrl);
  upKeyDisplay = XRCCTRL(*this, "upKeyDisplay", wxTextCtrl);
  downKeyDisplay = XRCCTRL(*this, "downKeyDisplay", wxTextCtrl);
  leftKeyDisplay = XRCCTRL(*this, "leftKeyDisplay", wxTextCtrl);
  rightKeyDisplay = XRCCTRL(*this, "rightKeyDisplay", wxTextCtrl);
  scanTimeDisplay = XRCCTRL(*this, "scanTimeDisplay", wxTextCtrl);
  
  // sliders
  scanTime = XRCCTRL(*this, "scanTime", wxSlider);

  // set default value for scanTimeDisplay
  int value = scanTime->GetValue();
  wxString sValue = _T("");
  sValue << value ;
  scanTimeDisplay->SetValue(sValue);

  this->AskAllParameters();

}



/** Receive message from Open Maia Server and parses it.
    This metohod is used to sincronize options from maia to maiaconfig.

    @param mes message sended.
*/
void maiaconfigFrame::NotifyTCPMessage(omTCPMessage mes)
{
  wxCheckBox* check;
  wxSlider* slider;
  long num;

  if (mes.toAgent.CmpNoCase("MAIACONFIG")==0)//If message is sent to 'maiaconfig'
    {

      if (mes.command.CmpNoCase("PARVALUE")==0)//If message contains a parameter value
	{

	  if (mes.parameters[0].CmpNoCase("autoscan")==0)
	    {
	      mes.parameters[1].CmpNoCase("TRUE")==0 ? autoscan->SetValue(TRUE) : autoscan->SetValue(FALSE);
	    }

	  if (mes.parameters[0].CmpNoCase("joystick")==0)
	    {
	      mes.parameters[1].CmpNoCase("TRUE")==0 ? joystick->SetValue(TRUE) : joystick->SetValue(FALSE);
	    }

	  if (mes.parameters[0].CmpNoCase("direct")==0)
	    {
	      mes.parameters[1].CmpNoCase("TRUE")==0 ? direct->SetValue(TRUE) : direct->SetValue(FALSE);
	    }

	  if (mes.parameters[0].CmpNoCase("grabKeys")==0)
	    {
	      mes.parameters[1].CmpNoCase("TRUE")==0 ? grabKeys->SetValue(TRUE) : grabKeys->SetValue(FALSE);
	    }

	  if (mes.parameters[0].CmpNoCase("actionOnPress")==0)
	    {
	      mes.parameters[1].CmpNoCase("TRUE")==0 ? actionOnPress->SetValue(TRUE) : actionOnPress->SetValue(FALSE);
	    }

	  if (mes.parameters[0].CmpNoCase("scanTime")==0)
	    {
	      mes.parameters[1].ToLong(&num);
	      scanTime->SetValue( (int)num );
	      scanTimeDisplay->SetValue(mes.parameters[1]);
	    }

	}

    }
}


/** Ask all parameters values to Maia application. */
void maiaconfigFrame::AskAllParameters()
{
  omTCPMessage mes; //Message to fill
  wxCheckBox* check;
  wxSlider* slider;

  mes.fromAgent="maiaconf";
  mes.toAgent="maia";
  mes.command="GetParValue";

  mes.parameters.Clear();
  mes.AddParameter("autoscan");
  connector->SendMessage(mes);

  mes.parameters.Clear();
  mes.AddParameter("joystick");
  connector->SendMessage(mes);

  mes.parameters.Clear();
  mes.AddParameter("direct");
  connector->SendMessage(mes);

  mes.parameters.Clear();
  mes.AddParameter("grabKeys");
  connector->SendMessage(mes);

  mes.parameters.Clear();
  mes.AddParameter("actionOnPress");
  connector->SendMessage(mes);

  mes.parameters.Clear();
  mes.AddParameter("scanTime");
  connector->SendMessage(mes);

}


/** Send all parameters values to Maia application. */
void maiaconfigFrame::SendAllParameters()
{
  //omTCPMessage mes; //Message to fill
  wxCheckBox* check;
  wxSlider* slider;
  omTCPMessage mes;

  mes.fromAgent="maiaconf";
  mes.toAgent="maia";
  mes.command="SetParValue";

  check = XRCCTRL(*this, "autoscan", wxCheckBox);
  mes.parameters.Clear();
  mes.AddParameter("autoscan");
  mes.AddParameter(check->GetValue() ? "True" : "False");
  connector->SendMessage(mes);

  check = XRCCTRL(*this, "joystick", wxCheckBox);
  mes.parameters.Clear();
  mes.AddParameter("joystick");
  mes.AddParameter(check->GetValue() ? "True" : "False");
  connector->SendMessage(mes);

  check = XRCCTRL(*this, "direct", wxCheckBox);
  mes.parameters.Clear();
  mes.AddParameter("direct");
  mes.AddParameter(check->GetValue() ? "True" : "False");
  connector->SendMessage(mes);

  check = XRCCTRL(*this, "grabKeys", wxCheckBox);
  mes.parameters.Clear();
  mes.AddParameter("grabKeys");
  mes.AddParameter(check->GetValue() ? "True" : "False");
  connector->SendMessage(mes);

  check = XRCCTRL(*this, "actionOnPress", wxCheckBox);
  mes.parameters.Clear();
  mes.AddParameter("actionOnPress");
  mes.AddParameter(check->GetValue() ? "True" : "False");
  connector->SendMessage(mes);

  slider = XRCCTRL(*this, "scanTime", wxSlider);
  mes.parameters.Clear();
  mes.AddParameter("scanTime");
  wxString tmp="";
  tmp<<slider->GetValue();
  mes.AddParameter(tmp.c_str());
  connector->SendMessage(mes);

}


// /// Quit the application
// void maiaconfigFrame::OnQuit(wxCommandEvent& event)
// {
//   Destroy();
// }


/** Save button pressed.
    Send parameters and Save message to maia agent.
*/
void maiaconfigFrame::SaveButtonPressed(wxCommandEvent& WXUNUSED(event))
{
  this->SendAllParameters();

  // send save parameters message
  omTCPMessage mes;

  mes.fromAgent="maiaconf";
  mes.toAgent="maia";
  mes.command="SaveParameters";
  connector->SendMessage(mes);
}


/** Apply button pressed.
    Send all prarameters and Apply message to maia agent.
*/
void maiaconfigFrame::ApplyChangesPressed(wxCommandEvent& WXUNUSED(event))
{
  this->SendAllParameters();
}



/** GetActionKey button pressed.
    Load getKeyDialog dialog.
*/
void maiaconfigFrame::GetActionKey(wxCommandEvent& WXUNUSED(event))
{
  getKeyDialog->ShowModal();
  if (getKeyDialog->IsKeySelected())
    {
      wxString value = getKeyDialog->GetSelectedKey();
      value = omFunctions::ConvertToUpper(value);
      actionKeyDisplay->SetValue(value);
    }
}

/** GetUpKey button pressed.
    Load getKeyDialog dialog.
*/
void maiaconfigFrame::GetUpKey(wxCommandEvent& WXUNUSED(event))
{
  getKeyDialog->ShowModal();
  if (getKeyDialog->IsKeySelected())
    {
      wxString value = getKeyDialog->GetSelectedKey();
      value = omFunctions::ConvertToUpper(value);
      upKeyDisplay->SetValue(value);
    }
}

/** GetDownKey button pressed.
    Load getKeyDialog dialog.
*/
void maiaconfigFrame::GetDownKey(wxCommandEvent& WXUNUSED(event))
{
  getKeyDialog->ShowModal();
  if (getKeyDialog->IsKeySelected())
    {
      wxString value = getKeyDialog->GetSelectedKey();
      value = omFunctions::ConvertToUpper(value);
      downKeyDisplay->SetValue(value);
    }
}

/** GetLeftKey button pressed.
    Load getKeyDialog dialog.
*/
void maiaconfigFrame::GetLeftKey(wxCommandEvent& WXUNUSED(event))
{
  getKeyDialog->ShowModal();
  if (getKeyDialog->IsKeySelected())
    {
      wxString value = getKeyDialog->GetSelectedKey();
      value = omFunctions::ConvertToUpper(value);
      leftKeyDisplay->SetValue(value);
    }
}

/** GetRightKey button pressed.
    Load getKeyDialog dialog.
*/
void maiaconfigFrame::GetRightKey(wxCommandEvent& WXUNUSED(event))
{
  getKeyDialog->ShowModal();
  if (getKeyDialog->IsKeySelected())
    {
      wxString value = getKeyDialog->GetSelectedKey();
      value = omFunctions::ConvertToUpper(value);
      rightKeyDisplay->SetValue(value);
    }
}


/** Process scanTime scroll event
 */
void maiaconfigFrame::OnChangeScanTime(wxScrollEvent& WXUNUSED(event))
{
  int value = scanTime->GetValue();
  wxString stringValue = _T("") ;
  stringValue << value;
  //int intValue = omFunctions::String2Int(value);
  scanTimeDisplay->SetValue(stringValue);
}

/** Destroy the class */
bool maiaconfigFrame::Destroy()
{
  std::cout << "distruttore del frame" << std::endl;
  getKeyDialog->Destroy();
  wxFrame::Destroy();
}


// --------------------------------------------------- event handlers

/// Process resize event (don't work)
void maiaconfigFrame::OnSize( wxSizeEvent event)
{
  
//   this->SetSize(355,455);
//   wxGetApp().Yield();
//   this->Refresh();
//   wxGetApp().Yield();
}

/// Process close event
void maiaconfigFrame::OnClose( wxCloseEvent &event)
{
  this->CloseFrame();
}

/// Process cancel event
void maiaconfigFrame::OnCancel(wxCommandEvent& WXUNUSED(event))
{
  this->CloseFrame();
}


/// Close the frame on close event or cancel event
/// @todo implement controls during closing of the application.
void maiaconfigFrame::CloseFrame()
{
  if (toSave)
    std::cout << "da salvare" << std::endl;

  std::cout << "distruggo frame" << std::endl;
  this->Destroy();
}





// event handlers

BEGIN_EVENT_TABLE(maiaconfigFrame, wxFrame)
  EVT_BUTTON(XRCID("saveButton"), maiaconfigFrame::SaveButtonPressed)
  EVT_BUTTON(XRCID("applyButton"), maiaconfigFrame::ApplyChangesPressed)
  EVT_BUTTON(XRCID("cancel"), maiaconfigFrame::OnCancel)
  EVT_BUTTON(XRCID("getActionKey"), maiaconfigFrame::GetActionKey)
  EVT_BUTTON(XRCID("getUpKey"), maiaconfigFrame::GetUpKey)
  EVT_BUTTON(XRCID("getDownKey"), maiaconfigFrame::GetDownKey)
  EVT_BUTTON(XRCID("getLeftKey"), maiaconfigFrame::GetLeftKey)
  EVT_BUTTON(XRCID("getRightKey"), maiaconfigFrame::GetRightKey)
  EVT_COMMAND_SCROLL(XRCID("scanTime"), maiaconfigFrame::OnChangeScanTime)
  EVT_CLOSE( maiaconfigFrame::OnClose)
END_EVENT_TABLE()


  // soppressed events
  // EVT_SIZE( maiaconfigFrame::OnSize)
  // EVT_CLOSE( maiaconfigFrame::OnQuit)
