
////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////
#ifndef _maiaconfigFrame_
#define _maiaconfigFrame_

#include <wx/wx.h>
#include <wx/xrc/xmlres.h>  

#include <iostream>

#include <openmaia/libopenmaia.h>


class maiaconfigMain;

#include "maiaconfig.h"


/** @ingroup maiaconfig
    
This class is the frame that the user uses to handle maia application.
The value of controls handled by this frame are syncronized using GetParValue message when the frame is build.
The frame handles events form user and send values using SetParValue message when apply or save button is pressed.
Apply button also sand ApplyOptions message and save button send ApplyOptions and SaveOptions messages to maia application.

*/

// Define a new frame type: this is going to be our main frame
class maiaconfigFrame : public wxFrame, public omObject
{

public:
    maiaconfigFrame( wxWindow* parent=(wxWindow *)NULL);    ///<Constructor.

private:
    omConnector *connector; ///<Connector for messages exchange (over tcp/ip)
    wxString host; ///<Host to connect with
    int port;
    omGetKeyDialog *getKeyDialog;

    bool toSave;   ///< Flag indicating if some option changed.

    // ------ pointers to frame objects
    
    // check box
    wxCheckBox *autoscan,
      *joystick,
      *direct,
      *actionOnPress,
      *enableKeyboard,
      *grabKeys,
      *enableJoystick,
      *enableSerial;

    // text controls
    wxTextCtrl *actionKeyDisplay, 
      *upKeyDisplay, 
      *downKeyDisplay,
      *leftKeyDisplay, 
      *rightKeyDisplay,
      *scanTimeDisplay;
    // slider controls
    wxSlider *scanTime;

    void NotifyTCPMessage(omTCPMessage); 

    // generic event handlers
    //void OnQuit(wxCommandEvent&);
    bool Destroy();
    void CloseFrame();

    // sliders events
    void OnChangeScanTime(wxScrollEvent& WXUNUSED(event)); 

    // buttons event handlers
    void SaveButtonPressed(wxCommandEvent& WXUNUSED(event)); 
    void ApplyChangesPressed(wxCommandEvent& WXUNUSED(event)); 
    void GetActionKey(wxCommandEvent& WXUNUSED(event));
    void GetUpKey(wxCommandEvent& WXUNUSED(event));
    void GetDownKey(wxCommandEvent& WXUNUSED(event));
    void GetLeftKey(wxCommandEvent& WXUNUSED(event));
    void GetRightKey(wxCommandEvent& WXUNUSED(event));
    void OnCancel(wxCommandEvent& WXUNUSED(event));

    void AskAllParameters();	
    void SendAllParameters();	
    

    // event handler
    void OnSize( wxSizeEvent);
    void OnClose( wxCloseEvent &);
    
    DECLARE_EVENT_TABLE()

};


#endif  // _maiaconfigFrame_
