////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#include <iostream>

#include "maiasmsPop.h"



enum
{
SERVER_ID=3000,
SOCKET_ID
} ;


BEGIN_EVENT_TABLE(maiasmsPop, wxEvtHandler)
  EVT_SOCKET(SERVER_ID, maiasmsPop::OnServerEvent)
  EVT_SOCKET(SOCKET_ID, maiasmsPop::OnSocketEvent)
END_EVENT_TABLE()

// main function of the wx application
maiasmsPop::maiasmsPop(omSmsManager *smsmanagerConstructor, wxString deviceConstructor) 
{

  status=0;	//Reset status;
  text="";

  // start the server socket
  wxIPV4address addr ;
  addr.Service(POP_PORT) ;
  server = new wxSocketServer(addr) ;
  bool connectionOk = server->Ok() ;


  while (!connectionOk)
    {
      std::cout << "." ;
      delete server ;
      server = new wxSocketServer(addr) ;
      connectionOk = server->Ok() ;
    }
  std::cout << "POP Server is up!" << std::endl;


  server->SetEventHandler(*this, SERVER_ID);
  server->SetNotify(wxSOCKET_CONNECTION_FLAG);
  server->Notify(TRUE);

  // Initialize device
  device=deviceConstructor;
  // Initialize smsManager
  smsmanager=smsmanagerConstructor;
  // Initialize smsArray
  smsarray=new omSmsArray();
  // Initialize MailArray
  mailarray=new wxArrayString();
  mailID=1;

}









void maiasmsPop::CreateMail()
{
  int x,num;
  wxString mail;
  omSms *sms;

  mailarray->Clear();

  num=smsarray->GetCount();
  for(x=0;x<num;x++){
    //Get and remove first message
    sms=smsarray->Item(0);

    //Filling mail field
    mail=_T("FROM: ");
    mail<<sms->sender<<_T("@maiasms.maiaproject.org");
    mail<<_T("\n");

    mail<<_T("DATE: ")<<sms->timestamp<<_T("\n");

    mail<<_T("SUBJECT: ")<<sms->body.Mid(0,sms->body.Find(_T("\n"))).Truncate(80)<<_T("\n");

    mail<<_T("Content-type: text/plain;\n");
    mail<<_T("User-Agent: OpenMaia-MaiaSms\n");

    mail<<_T("\n");//Marker for end of header

    mail<<sms->body;

    mailarray->Add(mail);

    delete sms;
    smsarray->Remove(sms);
  }

}












void maiasmsPop::OnServerEvent(wxSocketEvent &event)
{
  // retrive client informations
  wxSocketBase *sock ;
  wxIPV4address *ad = new wxIPV4address() ;

  // accept the connection
  sock = server->Accept(TRUE) ;

  // get host informations
  sock->GetPeer(*ad) ;

  // add client information to agents hash map
  // the key of the map is 'hostnameport'
  wxString hostname= ad->Hostname() ;
  int port=ad->Service();

  // build the key of the hash map
  wxString key ;
  key << hostname << port ;

#ifdef MAIA_DEBUG
  std::cout << "openmaia main: Accepted connection from " << hostname << ":" << port <<std::endl;
#endif


  // assign socket handler
  sock->SetEventHandler(*this, SOCKET_ID);
  sock->SetNotify(wxSOCKET_INPUT_FLAG);
  sock->Notify(TRUE);



  //Initializing phone device
  int error=0;
  if(smsmanager->InitDevice(device,115200)==0){
    std::cout<<"*****ERROR to initialize the device! NO message will be retrived!*****"<<std::endl;
    //    (void)wxMessageBox(_T("Error to initialize the device!"), _T("Maia SMS Manager"),wxICON_ERROR);
    error=1;
  }


  wxString str="";
  //Respond to client accepting connection
  str=_T("+OK OpenMaia - OpenMaia POP emulator for SMS running on ")+hostname+_T(" at ")+ad->IPAddress()+_T("\n");
  std::cout<<str<<std::endl;
  sock->Write(str, str.Len()) ;


}








void maiasmsPop::OnSocketEvent(wxSocketEvent &event)
{

  // retrive connection informations
  wxIPV4address *ad = new wxIPV4address() ;
  
  wxSocketBase *sock = event.GetSocket();
  sock->GetPeer(*ad) ;

  wxString hostname = ad->Hostname();
  unsigned int port = ad->Service() ;

  // build the map key
  wxString key ;
  key << hostname << port ;


  // retrive communicated char
  char c ;
  sock->Read(&c,1) ;

  text<<c; //adding the new char to text message
//    std::cout<<c<<std::endl;


  int err=1;


  if(c=='\n')	//text message end
    {
      std::cout<<text; //print message

      if(text.Mid(0,4).CmpNoCase(_T("AUTH"))==0)	//If AUTH detected
	{
	  wxString response=_T("-ERR AUTH? I'm not quite sure what you mean, Master.\n");
	  sock->Write(response.c_str(),response.Len());
	  std::cout<<response;

	  status=0;
	  err=0;
	}


      if(text.Mid(0,4).CmpNoCase(_T("CAPA"))==0)	//CAPA: detected
	{
	  wxString response=_T("+OK These are my limits, Sir\n");
	  response+=_T("USER\nUIDL\nEXPIRE NEVER\nIMPLEMENTATION OpenMaia - MaiaSMS POP\n.\n");
	  sock->Write(response.c_str(),response.Len());
	  std::cout<<response;

	  status=0;
	  err=0;
	}


      switch(status)
	{
	case 0:

	  if(text.Mid(0,4).CmpNoCase(_T("USER"))==0)	//USER: detected
	    {
	      username=text.Mid(text.Find(_T("USER"))+4);

	      wxString response=_T("+OK Welcome, do you have any type of ID?\n");
	      sock->Write(response.c_str(),response.Len());
	      std::cout<<response;
	      status=1;	//access to next step

	      err=0;
	    }
	  break;

	case 1:
	  if(text.Mid(0,4).CmpNoCase(_T("PASS"))==0)	//PASS: detected
	    {
	      password=text.Mid(text.Find(_T("PASS"))+4);	//Save password
	      wxString response;

	      //Retrive message
	      if( (smsmanager->ReceiveMessages(smsarray, _T("UNREAD") ))==-1){
		response=_T("-ERR Error retriving sms from phone.\n");
		status=0;
	      }
	      else{
		response=_T("+OK ");
		response<<smsarray->GetCount()<<_T(" messages\n");
		status=2;	//access to next step
	      }

	      sock->Write(response.c_str(),response.Len());
	      std::cout<<response;

	      err=0;
	    }
	  break;

	case 2:
	  if(text.Mid(0,4).CmpNoCase(_T("STAT"))==0)
	    {
	      int fullsize=0;
	      int x,num;

	      CreateMail();//Convert sms to e-mail
	      num=mailarray->GetCount();
	      for(x=0;x<num;x++){
		fullsize+=mailarray->Item(x).Len();
	      }

	      wxString response=_T("+OK ");
	      response<<mailarray->GetCount()<<_T(" ")<<fullsize<<_T(" \n");

	      sock->Write(response.c_str(),response.Len());
	      std::cout<<response;

	      err=0;
	      break;
	    }

	  if(text.Mid(0,4).CmpNoCase(_T("LIST"))==0)
	    {
	      wxString response=_T("+OK\n");

	      wxString mail;
	      int x,num;
	      num=mailarray->GetCount();
	      for(x=0;x<num;x++){
		mail=mailarray->Item(x);
		response<<(x+1)<<_T(" ")<<mail.Len()<<_T("\n");
	      }
	      response<<_T(".\n");

	      sock->Write(response.c_str(),response.Len());
	      std::cout<<response;
	      status=3;

	      err=0;
	      break;
	    }


	case 3:
	  if(text.Mid(0,4).CmpNoCase(_T("UIDL"))==0)
	    {
	      wxString response=_T("+OK\n");

	      wxString mail;
	      int x,num;
	      num=mailarray->GetCount();
	      for(x=0;x<num;x++){
		mail=mailarray->Item(x);
		response<<(x+1)<<_T(" ")<<mailID++<<_T("\n");
//std::cout<<mail<<std::endl;
	      }
	      response<<_T(".\n");

	      sock->Write(response.c_str(),response.Len());
	      std::cout<<response;

	      err=0;
	      break;
	    }

	  if(text.Mid(0,4).CmpNoCase(_T("RETR"))==0)
	    {
	      wxString response=_T("+OK");


	      wxString mail; 
	      mail=mailarray->Item(0);

	      response<<_T(" ")<<mail.Len()<<_T(" bytes\n");
	      response<<mail;

//std::cout<<mail<<std::endl;

	      response<<_T("\r\n.\r\n");

	      sock->Write(response.c_str(),response.Len());
	      std::cout<<response.c_str();

	      mailarray->Remove(0,1);

	      err=0;
	      break;
	    }


	}//end Switch

      if(text.Mid(0,4).CmpNoCase(_T("DELE"))==0)	//QUIT: detected
	{
	  long num;
	  text.Mid(5).Trim().ToLong(&num);
	  smsmanager->DeleteMessage(num);
	  wxString response=_T("+OK message marked for deletion\n");

	  sock->Write(response.c_str(),response.Len());
	  std::cout<<response;

	  err=0;
	}


      if(text.Mid(0,4).CmpNoCase(_T("QUIT"))==0)	//QUIT: detected
	{
	  wxString response=_T("+OK MaiaSMS POP server closing connection\n");
	  sock->Write(response.c_str(),response.Len());
	  std::cout<<response;

	  smsmanager->CloseDevice();
	  sock->Destroy();
	  status=0;

	  err=0;
	}




      if(err)
	{
	  std::cout<<"-ERR MESSAGE UNKNOWN. Server'll be reset."<<std::endl;
	  smsmanager->CloseDevice();
	  sock->Destroy();
	  status=0;
	}



      text="";
    }



}

