////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#include <iostream>

#include "maiasmsSmtp.h"



enum
{
SERVER_ID=2000,
SOCKET_ID
} ;


BEGIN_EVENT_TABLE(maiasmsSmtp, wxEvtHandler)
  EVT_SOCKET(SERVER_ID, maiasmsSmtp::OnServerEvent)
  EVT_SOCKET(SOCKET_ID, maiasmsSmtp::OnSocketEvent)
END_EVENT_TABLE()

// main function of the wx application
maiasmsSmtp::maiasmsSmtp(omSmsManager *smsmanagerConstructor, wxString deviceConstructor) 
{

  status=0;	//Reset status;
  text="";

  // start the server socket
  wxIPV4address addr ;
  addr.Service(SMTP_PORT) ;
  server = new wxSocketServer(addr) ;
  bool connectionOk = server->Ok() ;


  while (!connectionOk)
    {
      std::cout << "." ;
      delete server ;
      server = new wxSocketServer(addr) ;
      connectionOk = server->Ok() ;
    }
  std::cout << "SMTP Server is up!" << std::endl;


  server->SetEventHandler(*this, SERVER_ID);
  server->SetNotify(wxSOCKET_CONNECTION_FLAG);
  server->Notify(TRUE);

  // Initialize smsManager
  smsmanager=smsmanagerConstructor;
  device=deviceConstructor;


}




void maiasmsSmtp::OnServerEvent(wxSocketEvent &event)
{
  // retrive client informations
  wxSocketBase *sock ;
  wxIPV4address *ad = new wxIPV4address() ;

  // accept the connection
  sock = server->Accept(TRUE) ;

  // get host informations
  sock->GetPeer(*ad) ;

  // add client information to agents hash map
  // the key of the map is 'hostnameport'
  wxString hostname= ad->Hostname() ;
  int port=ad->Service();

  // build the key of the hash map
  wxString key ;
  key << hostname << port ;

#ifdef MAIA_DEBUG
  std::cout << "MaiaSmsSMTP: Accepted connection from " << hostname << ":" << port <<std::endl;
#endif


  // assign socket handler
  sock->SetEventHandler(*this, SOCKET_ID);
  sock->SetNotify(wxSOCKET_INPUT_FLAG);
  sock->Notify(TRUE);


//Respond to client accepting connection
  wxString str;
  str="220 OpenMaia SMTP emulator for SMS running on "+hostname+" at "+ad->IPAddress()+"\n";
  sock->Write(str, str.Len()) ;


  //Reinitializing message variables
  text="";
  sender="";
  destination="";
  message="";
}







void maiasmsSmtp::OnSocketEvent(wxSocketEvent &event)
{

  // retrive connection infromations
  wxIPV4address *ad = new wxIPV4address() ;
  
  wxSocketBase *sock = event.GetSocket();
  sock->GetPeer(*ad) ;

  wxString hostname = ad->Hostname();
  unsigned int port = ad->Service() ;

  // build the map key
  wxString key ;
  key << hostname << port ;

  
  // retrive communicated char
  char c ;
  sock->Read(&c,1) ;

  text<<c; //adding the new char to text message
//  std::cout<<c<<std::endl;



  if(c=='\n')	//text message end
    {
      std::cout<<text; //print message

      if(text.Mid(0,4).CmpNoCase("RSET")==0)	//If RSET detected
	{
	  sock->Write("250 OK\n",10);
	  std::cout<<"250 OK\n";
	}

      switch(status)
	{
	case 0:
	  if(text.Mid(0,4).CmpNoCase("HELO")==0)	//If HELO detected
	    {
	      sock->Write("250 Hello\n",10);
	      std::cout<<"250 Hello\n";
	      status=1;	//access to next step
	    }
	  if(text.Mid(0,4).CmpNoCase("EHLO")==0)	//If EHLO detected
	    {
	      sock->Write("500 \n",5);
	      std::cout<<"500 \n";
	    }
	  break;


	case 1:
	  if(text.Mid(0,10).CmpNoCase("MAIL FROM:")==0)	//MAIL FROM: detected
	    {
	      sender=text.Mid(text.Find("MAIL FROM:")+10,text.Len()-12);

	      wxString response="250 "+sender+" is syntactically correct\n";
	      sock->Write(response.c_str(),response.Len());
	      std::cout<<response;
	      status=2;	//access to next step
	    }
	  break;

	case 2:
	  if(text.Mid(0,8).CmpNoCase("RCPT TO:")==0)	//RCPT TO: detected
	    {
	      destination=text.Mid(text.Find("RCPT TO:")+8,text.Len()-10);
	      destination=destination.Trim(FALSE);//trim left space

	      wxString response="250 "+destination+" verified\n";
	      sock->Write(response.c_str(),response.Len());
	      std::cout<<response;
	      status=3;	//access to next step
	    }
	  break;

	case 3:
	  if(text.Mid(0,4).CmpNoCase("DATA")==0)	//RCPT TO: detected
	    {
	      wxString response="354 Enter message, ending with \".\" on a line by itself\n";
	      sock->Write(response.c_str(),response.Len());
	      std::cout<<response;
	      status=4;	//access to next step
	    }
	  break;

	case 4:
	  if(text.Cmp(".\r\n")==0)
	    {
	      std::cout<<"Sending SMS. Please wait..."<<std::endl;
	      wxString response;
	      if(this->SendSMS())
		{
		  response="250 OK\n";
		}
	      else
		{
		  response="500 Error\n";
		  smsmanager->CloseDevice();
		}
	      sock->Write(response.c_str(),response.Len());
	      std::cout<<response;
	      status=5;	//access to next step
	    }
	  else
	    {
	      //I need to remove starting '.' from message because protocol duplicate it.
	      if((text.Left(1)).Cmp(".")==0)
		{
		  text=text.Mid(1);
		}
		  message<<text;
	    }
	  break;

	case 5:
	  if(text.Mid(0,4).CmpNoCase("QUIT")==0)	//QUIT: detected
	    {
	      wxString response="221 "+hostname+" closing connection\n";
	      std::cout<<response;

	      sock->Write(response.c_str(),response.Len());
	      smsmanager->CloseDevice();
	      sock->Destroy();
	      status=0;	//FINISH
	      break;
	    }


	default:
	  text="";
	  status=0;
	  std::cout<<"MESSAGE UNKNOWN. Server'll be reset."<<std::endl;
	}


      text="";
    }


}






int maiasmsSmtp::SendSMS()
{
  int error=0;
    if(smsmanager->InitDevice(device)==0){
      std::cout<<"Error to initialize the device!"<<std::endl;
      (void)wxMessageBox(_T("Error to initialize the device!"), _T("Maia SMS Manager"),wxICON_ERROR);
      error=1;
    }


    if (!error){
      std::cout<<"TO: "<<destination.Mid(1,destination.Find("@")-1)<<std::endl;
      std::cout<<"MESSAGE: \""<<message.Mid(message.First("\n\r\n")+3)<<"\""<<std::endl;
      if(smsmanager->SendSMS(destination.Mid(1,destination.Find("@")-1), message.Mid(message.First("\n\r\n")+3)) )
	{
	  return(true);
	}
      else
	{
	  return(false);
	}
    }

    return(false);
}
