////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#ifdef __GNUG__
    #pragma implementation "maiasmseditFrame.cpp"
#endif


// For compilers that support precompilation
#include "wx/wxprec.h"

#include "wx/filename.h"
#include "wx/config.h"

// Include private headers
#include "maiasmseditFrame.h"

// Notice that wxID_HELP will be processed for the 'About' menu and the toolbar
// help button.

BEGIN_EVENT_TABLE(maiasmseditFrame, wxFrame)
    EVT_SIZE(maiasmseditFrame::OnSize)

    EVT_MENU(wxID_EXIT, maiasmseditFrame::OnQuit)
    EVT_MENU(wxID_HELP, maiasmseditFrame::OnAbout)

    EVT_MENU(ID_SEND, maiasmseditFrame::OnSend)

    EVT_MENU(wxID_PRINT, maiasmseditFrame::OnAbout)

    EVT_MENU(IDM_TOOLBAR_TOGGLETOOLBARSIZE, maiasmseditFrame::OnToggleToolbarSize)
    EVT_MENU(IDM_TOOLBAR_TOGGLETOOLBARORIENT, maiasmseditFrame::OnToggleToolbarOrient)

    EVT_MENU_RANGE(IDM_TOOLBAR_SHOW_TEXT, IDM_TOOLBAR_SHOW_BOTH,
                   maiasmseditFrame::OnToolbarStyle)

    EVT_COMBOBOX(ID_COMBO_PORT, maiasmseditFrame::OnComboPort)
    EVT_COMBOBOX(ID_COMBO_FONT, maiasmseditFrame::OnComboFont)

    EVT_TOOL_ENTER(ID_TOOLBAR, maiasmseditFrame::OnToolEnter)

    EVT_UPDATE_UI(wxID_COPY, maiasmseditFrame::OnUpdateCopyAndCut)
    EVT_UPDATE_UI(wxID_CUT, maiasmseditFrame::OnUpdateCopyAndCut)
    EVT_MENU(wxID_COPY, maiasmseditFrame::OnCopy)
    EVT_MENU(wxID_CUT, maiasmseditFrame::OnCut)
    EVT_MENU(wxID_PASTE, maiasmseditFrame::OnPaste)

    EVT_TEXT(-1, maiasmseditFrame::OnTextUpdate)

END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// maiasmseditFrame
// ----------------------------------------------------------------------------

/// SMS Editor constructor
maiasmseditFrame::maiasmseditFrame(wxFrame* parent,
                 wxWindowID id,
                 const wxString& title,
                 const wxPoint& pos,
                 const wxSize& size,
                 long style)
       : wxFrame(parent, id, title, pos, size, style)
{

  //Initializing path prefix
  //  If exist a developer path, I use it.
  wxString sharedPathPrefix=_T("");
    if ( omFunctions::TestFolder(_T("../../data")) )
      sharedPathPrefix=_T("../../data");
    else
      sharedPathPrefix=wxGetApp().sharedPath;


    //Create TEXTWINDOW
    m_textWindow = new wxTextCtrl(this, -1, _T(""), wxPoint(0, 0), wxSize(-1, -1), wxTE_MULTILINE);
    //Font setting
    m_textWindow->SetDefaultStyle(wxTextAttr(*wxBLUE,wxNullColour,wxFont(40,wxMODERN,wxNORMAL,wxNORMAL)));
    m_textWindow->WriteText("SMS:"); //Needed to SetDefaultStyle() for take effect
    m_textWindow->Clear();
    //    m_textWindow->SetFocusFromKbd(); //Doesn't work?!


    m_smallToolbar = TRUE;
    m_horzToolbar = TRUE;

    // Give it a status line
    CreateStatusBar();
    SetStatusText(_T("Hello, welcome MAIA Sms Manager!"));

    // Give it an icon
    SetIcon(wxIcon(sharedPathPrefix+_T("/icons/maiasms.xpm"),wxBITMAP_TYPE_XPM));

    // Make a menubar
    wxMenu *tbarMenu = new wxMenu;
    tbarMenu->AppendCheckItem(IDM_TOOLBAR_TOGGLETOOLBARSIZE,
                              _T("&Toggle toolbar size\tAlt-Z"),
                              _T("Toggle between big/small toolbar"));

    tbarMenu->AppendCheckItem(IDM_TOOLBAR_TOGGLETOOLBARORIENT,
                              _T("Toggle toolbar &orientation\tAlt-O"),
                              _T("Toggle toolbar orientation"));


    tbarMenu->AppendSeparator();


    tbarMenu->AppendRadioItem(IDM_TOOLBAR_SHOW_TEXT, _T("Show &text\tAlt-T"));
    tbarMenu->AppendRadioItem(IDM_TOOLBAR_SHOW_ICONS, _T("Show &icons\tAlt-I"));
    tbarMenu->AppendRadioItem(IDM_TOOLBAR_SHOW_BOTH, _T("Show &both\tAlt-B"));

    wxMenu *messageMenu = new wxMenu;
    messageMenu->Append(ID_SEND, _T("&Send\tAlt-S"), _T("Send message to phone") );

    wxMenu *fileMenu = new wxMenu;
    fileMenu->Append(wxID_EXIT, _T("E&xit\tAlt-X"), _T("Quit toolbar sample") );

    wxMenu *helpMenu = new wxMenu;
    helpMenu->Append(wxID_HELP, _T("&About"), _T("About toolbar sample"));

    wxMenuBar* menuBar = new wxMenuBar( wxMB_DOCKABLE );

    menuBar->Append(fileMenu, _T("&File"));
    menuBar->Append(tbarMenu, _T("&Toolbar"));
    menuBar->Append(messageMenu, _T("&Message"));
    menuBar->Append(helpMenu, _T("&Help"));

    // Associate the menu bar with the frame
    SetMenuBar(menuBar);

    menuBar->Check(IDM_TOOLBAR_SHOW_BOTH, TRUE);

    // Create the toolbar
    RecreateToolbar();


    // Initialize smsManager
    smsmanager=new omSmsManager();


    // Set the focus to the text Window
    m_textWindow->SetFocusFromKbd();

}




///Recreate toolbar
void maiasmseditFrame::RecreateToolbar()
{
  //Initializing path prefix
  wxString sharedPathPrefix=_T("");
    if ( omFunctions::TestFolder(_T("../../data")) )
      sharedPathPrefix=_T("../../data");
    else
      sharedPathPrefix=wxGetApp().sharedPath;

    // delete and recreate the toolbar
    wxToolBarBase *toolBar = GetToolBar();
    long style = toolBar ? toolBar->GetWindowStyle() : TOOLBAR_STYLE;

    delete toolBar;

    SetToolBar(NULL);

    style &= ~(wxTB_HORIZONTAL | wxTB_VERTICAL);
    style |= m_horzToolbar ? wxTB_HORIZONTAL : wxTB_VERTICAL;

    toolBar = CreateToolBar(style, ID_TOOLBAR);

    // Set up toolbar
    wxBitmap toolBarBitmaps[9];

    toolBarBitmaps[0] = wxBitmap(sharedPathPrefix+_T("/icons/new.xpm"),wxBITMAP_TYPE_XPM);
    toolBarBitmaps[1] = wxBitmap(sharedPathPrefix+_T("/icons/open.xpm"),wxBITMAP_TYPE_XPM);
    toolBarBitmaps[2] = wxBitmap(sharedPathPrefix+_T("/icons/save.xpm"),wxBITMAP_TYPE_XPM);
    toolBarBitmaps[3] = wxBitmap(sharedPathPrefix+_T("/icons/copy.xpm"),wxBITMAP_TYPE_XPM);
    toolBarBitmaps[4] = wxBitmap(sharedPathPrefix+_T("/icons/cut.xpm"),wxBITMAP_TYPE_XPM);
    toolBarBitmaps[5] = wxBitmap(sharedPathPrefix+_T("/icons/paste.xpm"),wxBITMAP_TYPE_XPM);
    toolBarBitmaps[6] = wxBitmap(sharedPathPrefix+_T("/icons/print.xpm"),wxBITMAP_TYPE_XPM);
    toolBarBitmaps[7] = wxBitmap(sharedPathPrefix+_T("/icons/phone.xpm"),wxBITMAP_TYPE_XPM);
    toolBarBitmaps[8] = wxBitmap(sharedPathPrefix+_T("/icons/help.xpm"),wxBITMAP_TYPE_XPM);

    if ( !m_smallToolbar )
    {
        int w = 2*toolBarBitmaps[0].GetWidth(),
            h = 2*toolBarBitmaps[0].GetHeight();
        for ( size_t n = 0; n < WXSIZEOF(toolBarBitmaps); n++ )
        {
            toolBarBitmaps[n] =
                wxBitmap(toolBarBitmaps[n].ConvertToImage().Scale(w, h));
        }

        toolBar->SetToolBitmapSize(wxSize(w, h));
    }

    toolBar->AddTool(wxID_NEW, _T("New"), toolBarBitmaps[0], _T("New message"));
    toolBar->AddTool(wxID_OPEN, _T("Open"), toolBarBitmaps[1], _T("Open message"));
    toolBar->AddTool(wxID_SAVE, _T("Save"), toolBarBitmaps[2], _T("Save message to file"));
    toolBar->AddTool(wxID_COPY, _T("Copy"), toolBarBitmaps[3], _T("Copy"));
    toolBar->AddTool(wxID_CUT, _T("Cut"), toolBarBitmaps[4], _T("Cut"));
    toolBar->AddTool(wxID_PASTE, _T("Paste"), toolBarBitmaps[5], _T("Paste"));
    toolBar->AddTool(wxID_PRINT, _T("Print"), toolBarBitmaps[6], _T("Print message"));

    if ( m_horzToolbar )
    {
        wxComboBox *combo = new wxComboBox(toolBar, ID_COMBO_FONT, _T("Font Size"), wxDefaultPosition, wxSize(50,-1), 0, NULL, wxCB_READONLY);
        combo->Append(_T("20"));
        combo->Append(_T("30"));
        combo->Append(_T("40"));
        combo->Append(_T("50"));
        combo->Append(_T("60"));
        combo->Append(_T("80"));
        combo->Append(_T("90"));
        combo->Append(_T("100"));

	combo->SetSelection(2);
        toolBar->AddControl(combo);

        combo = new wxComboBox(toolBar, ID_COMBO_PORT, _T("Output Device"), wxDefaultPosition, wxSize(130,-1), 0, NULL, wxCB_READONLY );
        combo->Append(_T("Serial Port"));
        combo->Append(_T("IrDA Port (Infrared)"));
        toolBar->AddControl(combo);

    }

    toolBar->AddTool(ID_SEND, _T("Send"), toolBarBitmaps[7], _T("Send Message"));

    toolBar->AddSeparator();
    toolBar->AddTool(wxID_HELP, _T("Help"), toolBarBitmaps[8], _T("Help button"));

    // after adding the buttons to the toolbar, must call Realize() to reflect
    // the changes
    toolBar->Realize();

    toolBar->SetRows(1);
}




///Adapt text window to frame size
void maiasmseditFrame::LayoutChildren()
{
    wxSize size = GetClientSize();

    int offset=0;

    m_textWindow->SetSize(offset, 0, size.x - offset, size.y);
}

void maiasmseditFrame::OnSize(wxSizeEvent& event)
{
  event.Skip();

}


///Toolbar resize
void maiasmseditFrame::OnToggleToolbarSize(wxCommandEvent& WXUNUSED(event))
{
    m_smallToolbar = !m_smallToolbar;

    RecreateToolbar();
}


///Toolabr orientation change
void maiasmseditFrame::OnToggleToolbarOrient(wxCommandEvent& WXUNUSED(event))
{
    m_horzToolbar = !m_horzToolbar;

    RecreateToolbar();
}

void maiasmseditFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    Close(TRUE);
}

void maiasmseditFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
    (void)wxMessageBox(_T("Maia SMS is part of OpenMAIA suite."), _T("Maia SMS Manager"));
}





///Change font size of whole text written
void maiasmseditFrame::OnComboFont(wxCommandEvent& event)
{
  wxString value;

    m_textWindow->SetDefaultStyle(wxTextAttr(*wxBLUE,wxNullColour,wxFont(omFunctions::String2Int(event.GetString()),wxMODERN,wxNORMAL,wxNORMAL)));

    value=m_textWindow->GetValue();
    m_textWindow->Clear();
    m_textWindow->WriteText(value);
    m_textWindow->SetFocusFromKbd();

    wxLogStatus(_T("Setting '%s' font size"), event.GetString().c_str());

}



///Set phone device type.
void maiasmseditFrame::OnComboPort(wxCommandEvent& event)
{
  wxLogStatus(_T("'%s' device selected"), event.GetString().c_str());
}




void maiasmseditFrame::OnUpdateCopyAndCut(wxUpdateUIEvent& event)
{
    event.Enable( m_textWindow->CanCopy() );
}


void maiasmseditFrame::OnCopy(wxCommandEvent& event)
{
  m_textWindow->Copy();
  m_textWindow->SetSelection(0,0);
}

void maiasmseditFrame::OnCut(wxCommandEvent& event)
{
  m_textWindow->Cut();
}

void maiasmseditFrame::OnPaste(wxCommandEvent& event)
{
  m_textWindow->Paste();
}


void maiasmseditFrame::OnToolbarStyle(wxCommandEvent& event)
{
    long style = GetToolBar()->GetWindowStyle();
    style &= ~(wxTB_NOICONS | wxTB_TEXT);

    switch ( event.GetId() )
    {
        case IDM_TOOLBAR_SHOW_TEXT:
            style |= wxTB_NOICONS | wxTB_TEXT;
            break;

        case IDM_TOOLBAR_SHOW_ICONS:
            // nothing to do
            break;

        case IDM_TOOLBAR_SHOW_BOTH:
            style |= wxTB_TEXT;
    }

    GetToolBar()->SetWindowStyle(style);
}



///Show toolbar tips
void maiasmseditFrame::OnToolEnter(wxCommandEvent& event)
{
  wxString str;

    if (event.GetSelection() > -1)
    {
      switch(event.GetSelection()){
      case 5002:
        str=_T("Create new message");
        SetStatusText(str);
	break;

      case 5000:
        str=_T("Open message");
        SetStatusText(str);
	break;

      case 5003:
        str=_T("Save message");
        SetStatusText(str);
	break;

      case 5031:
        str=_T("Copy selected text");
        SetStatusText(str);
	break;

      case 5030:
        str=_T("Cut selected text");
        SetStatusText(str);
	break;

      case 5032:
        str=_T("Paste selected text");
        SetStatusText(str);
	break;

      case 5010:
        str=_T("Print message");
        SetStatusText(str);
	break;

      case 1002:
        str=_T("Send message to phone");
        SetStatusText(str);
	break;

      case 5009:
        str=_T("Open 'about' box");
        SetStatusText(str);
	break;

      default:
	str<<event.GetSelection();
        SetStatusText(str);

      }
    }
    else
      SetStatusText(_T(""));
}




///Update text statistics on info bar.
void maiasmseditFrame::OnTextUpdate(wxCommandEvent& event)
{
  wxString str;

  str=_T("Character writed: ");
  str<<m_textWindow->GetLastPosition();
  str<<_T("   Character remaining: ");
  str<<(int)(160-(m_textWindow->GetLastPosition()%160))<<_T("/160");
  str<<_T("   SMS created: ");
  str<<(int)((m_textWindow->GetLastPosition()/160)+1);

//  std::cout<<str<<std::endl;
  SetStatusText(str);
}



void maiasmseditFrame::OnSend(wxCommandEvent& event)
{
  wxToolBarBase *toolBar = GetToolBar();
  int error=0;

//SERIAL
  if ( ((wxComboBox*)toolBar->FindControl(ID_COMBO_PORT))->GetSelection()==0 ){
    if(smsmanager->InitDevice(_T("/dev/ttyS0"))==0){
      (void)wxMessageBox(_T("Error to initialize the device!"), _T("Maia SMS Manager"),wxICON_ERROR);
      error=1;
    }
  }
//IrDA
  if ( ((wxComboBox*)toolBar->FindControl(ID_COMBO_PORT))->GetSelection()==1){
    if(smsmanager->InitDevice(_T("/dev/ircomm0"))==0){
      (void)wxMessageBox(_T("Error to initialize the device!"), _T("Maia SMS Manager"),wxICON_ERROR);
      error=1;
    }
  }

  if (!error){
  wxTextEntryDialog dialog(this,
                           _T("Insert destination phone number:"),
                           _T("Destination number"),
                           _T(""),
                           wxOK | wxCANCEL);

  if (dialog.ShowModal() == wxID_OK)
  {
    smsmanager->SendSMS(dialog.GetValue(), m_textWindow->GetValue());
  }



  }
}
