////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

/** @defgroup smsedit MAIA SMS EDITOR*/

class maiasmsEditMain;

#ifndef __maiasmseditFrame_h__
#define __maiasmseditFrame_h__

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma interface "maiasmseditFrame.cpp"
#endif

// Include wxWindows' headers

#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include <wx/toolbar.h>
#include <wx/log.h>
#include <wx/image.h>
#include <openmaia/libopenmaia.h>


//#include <wx/tbarsmpl.h>

#include "maiasmsedit.h"


// ----------------------------------------------------------------------------
// constants
// ----------------------------------------------------------------------------

const int ID_TOOLBAR = 500;

static const long TOOLBAR_STYLE = wxTB_FLAT | wxTB_DOCKABLE | wxTB_TEXT ;

enum
{
    IDM_TOOLBAR_TOGGLETOOLBARSIZE = 200,
    IDM_TOOLBAR_TOGGLETOOLBARORIENT,
    IDM_TOOLBAR_TOGGLE_TOOLBAR,
    IDM_TOOLBAR_SHOW_TEXT,
    IDM_TOOLBAR_SHOW_ICONS,
    IDM_TOOLBAR_SHOW_BOTH,

    ID_COMBO_FONT = 1000,
    ID_COMBO_PORT,
    ID_SEND

};




//----------------------------------------------------------------------------
// SMSEdit
//----------------------------------------------------------------------------

class maiasmseditFrame: public wxFrame
{
public:
    maiasmseditFrame(wxFrame *parent,
            wxWindowID id = -1,
            const wxString& title = _T("OpenMAIA Sms Manager"),
            const wxPoint& pos = wxDefaultPosition,
            const wxSize& size = wxDefaultSize,
            long style = wxDEFAULT_FRAME_STYLE);	


 private:
    void RecreateToolbar();			///<Recreate toolbar

    void OnQuit(wxCommandEvent& event);		///<On Quit event
    void OnAbout(wxCommandEvent& event);	///<On about event

    void OnSize(wxSizeEvent& event);		///<On size event

    void OnToggleToolbarSize(wxCommandEvent& event);	///<Toolbar resize
    void OnToggleToolbarOrient(wxCommandEvent& event);	///<Change toolbar orientation

    void OnToolbarStyle(wxCommandEvent& event);

    void OnToolEnter(wxCommandEvent& event);	///<Toolbar enter pressed

    void OnComboPort(wxCommandEvent& event);	///<Select device type
    void OnComboFont(wxCommandEvent& event);	///<Change font size

    void OnUpdateCopyAndCut(wxUpdateUIEvent& event);	///<Enable copy and cut functions
    void OnCopy(wxCommandEvent& event);		///<On Copy event
    void OnCut(wxCommandEvent& event);		///<On Cut event
    void OnPaste(wxCommandEvent& event);	///<On Paste event

    void OnTextUpdate(wxCommandEvent& event);	///<Update text statistics on info bar.

    void OnSend(wxCommandEvent& event);		///<Send message to phone



    void LayoutChildren();			///<Adapt text window to frame size


    bool                m_smallToolbar,
                        m_horzToolbar;

    wxTextCtrl         *m_textWindow;

    omSmsManager       *smsmanager;
    

    DECLARE_EVENT_TABLE()
};


#endif
