////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////




#include "openmaia.h"



IMPLEMENT_APP(openmaiaMain)

BEGIN_EVENT_TABLE(openmaiaMain, wxApp)
  EVT_TIMER(MOUSE_TIMER_ID, openmaiaMain::OnTimer)  
END_EVENT_TABLE()

/// Main function of the wx application
bool openmaiaMain::OnInit() 
{
  currentProfileName = _T("default");

  inDeveloperTree = false;   // by default
  timerIdleWakeUp = new wxTimer(this,MOUSE_TIMER_ID);   

  // local variables
  wxString resourceFile = _T("");   // resource to load
  wxString globalOptionsFile = _T("");  // options file to load
  binPath = wxString(argv[0]);  // path of this binary file

  processOptions = _T("");

  bool localUse = false ;
  
  // find global options file
  if ( omFunctions::TestFile(_T("../../data/openmaia.conf"))) // in developer tree
    {
      inDeveloperTree = true;
      globalOptionsFile = _T("../../data/openmaia.conf") ;
    }
  else
    {
      //omFunctions::SetBasePath(binPath);
      globalOptionsFile = omFunctions::GetUserDataPath(binPath) + _T("/openmaia.conf") ;
      //globalOptionsFile = _T("openmaia.conf") ;
      // to do (set correct profile)
    }

  // load global options
  options = new openmaiaOptions();
#ifdef MAIA_DEBUG
  std::cout << "openmaia: Global Options File " << globalOptionsFile << std::endl;
#endif

  // check for existence of global configuration file
  wxString globalConfFile = omFunctions::GetUserDataPath(binPath) + _T("/openmaia.conf") ;
  if ( !omFunctions::TestFile(globalConfFile) && !inDeveloperTree)
    {
      // check if openmaia data folder exists
      if (!omFunctions::TestFolder( omFunctions::GetUserDataPath(binPath)))
	omFunctions::CreateFolder(omFunctions::GetUserDataPath(binPath), binPath) ;

      wxString originalFile = omFunctions::GetSharedPath(binPath) + _T("/openmaia.conf") ;
      omFunctions::CopyFile(originalFile, globalConfFile, binPath);
#ifdef MAIA_DEBUG
      std::cout << "openmaia: Copying Global options file;\n o: " << originalFile << " d:" << globalConfFile << std::endl;
#endif
    }

  options->LoadOptions(globalOptionsFile);


  if (options->autostartProfileFlag)
    currentProfileName = options->profileToAutostart;
  else
    currentProfileName = options->lastSelectedProfile;

  wxString profileOptionsFile = _T("");
  // load profile options
  if (inDeveloperTree) 
    {
      // in local use only default profile using relative path
      profileOptionsFile = _T("../../data/default/openmaia/openmaia.conf");
    }
  else
    {
      // not local use, move to top user folder
      //omFunctions::SetBasePath(binPath);

      
				

      if ( currentProfileName.CmpNoCase("default") == 0)
	{
	  // in this case we are using default profile so copy from template if 
	  // is the first time the application is lounched 
	  if ( !omFunctions::TestPersonalInstallation(_T("default") , _T("openmaia"), binPath))
	    omFunctions::CreateDefaultInstallation(_T("openmaia"), binPath);
	}
      else
	{
	  // profile is not default, we have no template
	  if (!omFunctions::TestPersonalInstallation(currentProfileName , _T("openmaia"), binPath))
	    {
	      std::cerr << "OpenMAIA: Error when opening " << currentProfileName << 
		" data folder" << std::endl;
	      wxExit();
	    }
	  
	  
	}

      

      // change to profile directory
//       chdir("profiles");
//       chdir( (char *) (options->lastSelectedProfile).c_str()) ;
//       chdir("openmaia");
      currentProfileName = options->lastSelectedProfile;
      profileOptionsFile = omFunctions::GetUserDataPath(binPath) + 
	_T("/profiles/") + currentProfileName + _T("/openmaia/openmaia.conf");
    }

  std::cout << "file opzioni personali " << profileOptionsFile << std::endl;
  options->LoadOptions(profileOptionsFile);

  wxString resourceFileName = _T("openmaia") ;


#ifdef LINUX_SYSTEM
  resourceFileName << _T("_gtk");
#endif
#ifdef WINDOWS_SYSTEM
  resourceFileName << _T("_win");
#endif

  resourceFileName << _T("_") << options->language << _T(".xrc");

  wxString fileToTest = _T("../../data/resource/openmaia/") ;
  fileToTest << resourceFileName ;

  if ( omFunctions::TestFile(fileToTest))
    {
      resourceFile = _T("../../data/resource/openmaia/");
      resourceFile << resourceFileName ;
    }
  else
    {
      resourceFile = omFunctions::GetSharedPath(binPath) + _T("/resource/openmaia/") + resourceFileName ;
    }

#ifdef WINDOWS_SYSTEM
  resourceFile = omFunctions::GetWindowsString(resourceFile);
#endif

  std::cout << "load resource file: " << resourceFile << std::endl;

  // load resource file
  wxXmlResource::Get()->InitAllHandlers();  
  
  wxXmlResource::Get()->Load(resourceFile) ;
  

  // initialize application objecs
  processesHandler = new openmaiaProcessesHandler();  
  server = new openmaiaServer() ;
  mainFrame = new openmaiaFrame() ;

  mainFrame->SetEvtHandlerEnabled(false);

  // initialize objects
  mainFrame->Init();

  this->SetSubprocessParameters();

  
  // show the frame
  mainFrame->Show() ;


  mainFrame->SetEvtHandlerEnabled(true);

  if (options->autostartProfileFlag) 
    {
      mainFrame->ProfileStarted();
      this->StartProfile();
    }
      

//   // start child processes
//   keyboardProcess = new OpenMaiaProcess(this, OPENMAIAKEYBOARD) ;
//   //keyboardPID = ::wxExecute("nice -n 10 ./kext", wxEXEC_ASYNC, keyboardProcess);
//   //keyboardPID = ::wxExecute("./kext", wxEXEC_ASYNC, keyboardProcess);

  timerIdleWakeUp->Start(100);

  return TRUE ;

}


// called at the end of the application
int openmaiaMain::OnExit()
{
//   // call parent method
//   std::cout << "ginopino" << std::endl;
//   wxApp::OnExit() ;

  // stop all processes
  processesHandler->StopAllProcesses();

  std::cout << "uscita raggiunta" << std::endl;
}



/// Load profile data
void openmaiaMain::LoadProfile(wxString name)
{
//   processesHandler->StartProcess("maia");
//   processesHandler->StartProcess("maiaconfig");

//   mainFrame->Iconize();

}

/// Exit from openmaia application
void openmaiaMain::Exit()
{
  ::wxExit();
}

/// Save options
void openmaiaMain::SaveOptions()
{
  wxString generalOptions = options->GenerateOptionsXMLString();
  wxString profileOptions = options->GenerateProfileOptionsXMLString();

  wxString dataHome = omFunctions::GetUserDataPath(binPath);

  if (inDeveloperTree)    // options not saved
    return;

  // find files
  wxString generalFileName = dataHome + _T("/openmaia.conf");
  wxString profileFileName = dataHome + _T("/profiles/") + currentProfileName + _T("/openmaia/openmaia.conf");

  // open files
  std::ofstream generalFile(generalFileName.c_str());
  std::ofstream profileFile(profileFileName.c_str());
//   std::assure(generalFile, generalFileName.c_str());
//   std::assure(profileFile, profileFileName.c_str());

  generalFile << generalOptions;
  profileFile << profileOptions;
  
//   std::cout << options->GenerateOptionsXMLString() << std::endl << std::endl;
//   std::cout << options->GenerateProfileOptionsXMLString() << std::endl << std::endl;

  
}

/// Start child processes indicated by profile options
void openmaiaMain::StartProfile()
{
  if ( options->agentsToStart.GetCount() == 0) // no agents to load
    return;

  // load agents
  int i;
  
  // if we are in the developer tree agents are local and they are located
  // in folder ../<agent name>.
  // if we are not in developer tree agents are stored in <installation path>/bin
  wxString agentPath = _T("");

  // temporary fix to let maiaconfigurator to start only when maia is started
  bool maiaStarted = false;

  for ( i=0; i<options->agentsToStart.GetCount(); i++)
    {
      wxString agentName = options->agentsToStart[i];

      // if we are in the developer tree agents are local and they are located
      // in folder ../<agent name>.
      // if we are not in developer tree agents are stored in <installation path>/bin
      if (inDeveloperTree)
	agentPath = _T("../") + agentName;
      else
	agentPath = omFunctions::GetInstallationPath(binPath) + _T("/bin");

      if (agentName.CompareTo(_T("maia")) == 0)
	maiaStarted = true;

      processesHandler->StartProcess(agentName, processOptions, agentPath);

//       // start configurators
//       if ( agentName.CompareTo(_T("maia")) == 0)
// 	processesHandler->StartProcess(_T("maiaconfig"));
    }

  if (maiaStarted)
    mainFrame->EnableMaiaconfig();

  // iconize main frame (not in linux)
#ifndef LINUX_SYSTEM
  if (options->autoiconize)
    mainFrame->Iconize(TRUE);
#endif

}

void openmaiaMain::StopProfile()
{
  if ( options->agentsToStart.GetCount() == 0) // no agents loaded
    return;

  // kill processes
  processesHandler->StopAllProcesses();

}

/// Start a specific configurator.
/// A configurator is started as a subprocess but is not inserted in the list of 
/// the processes of the profile.
/// @param name name of the configurator to start.
void openmaiaMain::StartConfigurator(wxString name)
{
  wxString agentPath;
  // if we are in the developer tree agents are local and they are located
  // in folder ../<agent name>.
  // if we are not in developer tree agents are stored in <installation path>/bin
  if (inDeveloperTree)
    agentPath = _T("../") + name;
  else
    agentPath = omFunctions::GetInstallationPath(binPath) + _T("/bin");


  processesHandler->StartProcess(name, processOptions,agentPath);
}

/// Notify that a child process has stopped
void openmaiaMain::NotifyProcessStop(wxString name)
{
  // check if process is an extra subprocess of the openmaiaFrame
  int extraPosition = omFunctions::ArrayFind( &(mainFrame->extraProcessesStarted), name);
  if ( extraPosition != -1)
    {
      // remove from the array of extra processes
      mainFrame->extraProcessesStarted.Remove(extraPosition);
      // return to execution
      return;
    }

  if ( processesHandler->GetCount() ==0)
    mainFrame->ProfileStopped();

#ifndef LINUX_SYSTEM
  mainFrame->Iconize(FALSE);
#endif


  // if all processes of the profile are stopped notify a ProcessStopped event
  // to the frame


//   bool keyboardStopped = false;

//   std::cout << "openmaiaMain::NotifyProcessStop- stopped process: " << name << std::endl;

//   if ( name.CmpNoCase(_T("maia")) == 0 ) 
//     {
//       std::cout << "openmaiaMain::NotifyProcessStop- stopping maiaconfig" << std::endl;
//       processesHandler->StopProcess(_T("maiaconfig"));
//       keyboardStopped = true;
//     }
    
//   if ( name.CmpNoCase(_T("maiaconfig")) == 0 )
//     {
//       std::cout << "openmaiaMain::NotifyProcessStop- stopping maia" << std::endl;
//       processesHandler->StopProcess(_T("maia"));
//       keyboardStopped = true;
//     }

//   // doesn't work with gtk 1.2 :(
//   if (keyboardStopped)
//     mainFrame->Iconize(FALSE); 
  
}

void openmaiaMain::SetSubprocessParameters()
{
  // find processes common options
  if (options->connectToServerFlag)
    processOptions << _T(" --host=") << options->serverToConnect <<
      _T(" --port=") << options->portToConnect;

}


// ---------------------------------------------------------------------- PRIVATE METHODS ---

/// Timer handler.
/// Flush child streams.
void openmaiaMain::OnTimer(wxTimerEvent& WXUNUSED(event))
{
  //  wxWakeUpIdle();
  processesHandler->FlushStreams();

}

