////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#ifndef _openmaiaMain_
#define _openmaiaMain_

#include "openmaiaConstants.h"


#include <wx/wx.h>
#include <wx/socket.h>
#include <wx/process.h>
#include <wx/hashmap.h>
#include <wx/xrc/xmlres.h>

#include <openmaia/libopenmaia.h>

class openmaiaFrame;
class openmaiaServer;
class openmaiaOptions;
class openmaiaProcessesHandler;

#include "openmaiaFrame.h"
#include "openmaiaServer.h"
#include "openmaiaOptions.h"
#include "openmaiaProcessesHandler.h"


#include <vector>
#include <iostream>
#include <fstream>

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <getopt.h>


#define MOUSE_TIMER_ID 1

/** @ingroup openmaia.
    
    This class is the openmaia program application class 
*/

class openmaiaMain: public wxApp
{
 public:
  wxString binPath;
  wxString currentProfileName;

  virtual bool OnInit();
  virtual int OnExit() ;

  // frame interface
  void LoadProfile(wxString);
  void StartProfile();
  void StopProfile();
  void StartConfigurator(wxString);
  void SetParameter(wxString, wxString);
  void Exit();
  void SaveOptions();
  void SetSubprocessParameters();

  // process handler interface
  void NotifyProcessStop(wxString);
  

  openmaiaFrame *mainFrame ;
  openmaiaServer *server;
  openmaiaOptions *options;
  openmaiaProcessesHandler *processesHandler;
  
 private:
  bool inDeveloperTree;   ///< Flag 

  wxTimer *timerIdleWakeUp;  ///< This timer is used to flush stdout events from child processes
                             ///< to parent process.
                             ///< Without this child processes will block parent process.
  
  wxString processOptions;   ///< Store child processes common options in base of 
                             ///< openmaia options

  // ---------------------------------------------------------------- private methods ---

  void OnTimer(wxTimerEvent& WXUNUSED(event));  

  
  

  DECLARE_EVENT_TABLE()
  
} ;

DECLARE_APP(openmaiaMain) 


#endif  // _openmaiaMain_
