////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include "openmaiaFrame.h"


/// Build openmaiaFrame object by loading resource file and initializing help dialogs
/// that will be shown at the right moment
openmaiaFrame::openmaiaFrame()
  //  : wxFrame((wxFrame *)NULL, -1, _(""),
  //	    wxDefaultPosition, wxSize(300, 200))
{
  saved = true;

  bool gino = wxXmlResource::Get()->LoadFrame(this, NULL, wxT("openmaiaFrame"));
  this->SetTitle(wxString("OpenMAIA - ver. " + wxString(_T(VERSION))));

  // initialize dialog pointers 
  startServerHelpDialog = new omDialog(this, _T("Help"), _T("startServerHelpDialog"));
  connectToServerHelpDialog= new omDialog(this,_T("Help"),  _T("connectToServerHelpDialog"));
  autostartProfileHelpDialog = new omDialog(this,_T("Help"), _T("autostartProfileHelpDialog"));
  openmaiaHelpDialog = new omDialog(this,_T("Help"), _T("openmaiaHelpDialog"));
  maiaHelpDialog = new omDialog(this,_T("Help"), _T("maiaHelpDialog"));
  maiasmsHelpDialog = new omDialog(this,_T("Help"), _T("maiasmsHelpDialog"));
  autoiconizeHelpDialog = new omDialog(this,_T("Help"), _T("autoiconizeHelpDialog"));
  confirmExitDialog = new omDialog(this,_T("Exit"), _T("confirmExitDialog"));


  // initialize pointers to frame data from XRC
  selectedProfile = XRCCTRL(*this, "selectedProfile", wxComboBox);
  startServerFlag = XRCCTRL(*this, "startServerFlag", wxCheckBox);
  //autostartProfileFlag = XRCCTRL(*this, "autostartProfileFlag", wxCheckBox);
  autostartProfileFlag = XRCCTRL(*this, _T("autostartProfileFlag"),wxCheckBox);
  profileToAutostart = XRCCTRL(*this, "profileToAutostart", wxComboBox);
  connectToServerFlag = XRCCTRL(*this, "connectToServerFlag", wxCheckBox);
  serverToConnect = XRCCTRL(*this, "serverToConnect", wxTextCtrl);
  portToConnect = XRCCTRL(*this, "portToConnect", wxTextCtrl);
  startMaia = XRCCTRL(*this, "startMaia", wxCheckBox);
  startMaiasms = XRCCTRL(*this, "startMaiasms", wxCheckBox);
  autoiconize = XRCCTRL(*this, "autoiconize", wxCheckBox);
  saveButton = XRCCTRL(*this, "saveButton", wxButton);
  startProfile = XRCCTRL(*this, "startProfile", wxButton);
  stopProfile = XRCCTRL(*this, "stopProfile", wxButton);
  startMaiaconf = XRCCTRL(*this, "startMaiaconf", wxButton);

  // set application label
  wxStaticText *titleLabel = XRCCTRL(*this, "titleLabel", wxStaticText);
  wxString title = _T("OpenMAIA ver. ");
  title << _T(VERSION);
  titleLabel->SetLabel(title);

#ifdef WINDOWS_SYSTEM
  this->SetSize(372,300);
  ::wxYield();

#endif


}


/// Do post constructor initialization.
void openmaiaFrame::Init()
{
  // fetch options pointer
  options = wxGetApp().options;
  
  // initialize profiles
  selectedProfile->Append(_T("default"));
  profileToAutostart->Append(_T("default"));

  // initialize current profile
  selectedProfile->SetSelection( selectedProfile->FindString( options->lastSelectedProfile));

  // set title bar
  savedTitle = _T("OpenMAIA ver. ") ;
  savedTitle << _T(VERSION) << _T(" - ") << options->lastSelectedProfile ;
  nosavedTitle = savedTitle;
  nosavedTitle << _T("*");
  this->SetTitle(savedTitle);

  // initialize controls value
  startServerFlag->SetValue(options->startServerFlag);
  autostartProfileFlag->SetValue(options->autostartProfileFlag);
  profileToAutostart->SetSelection( profileToAutostart->FindString( options->profileToAutostart));
  connectToServerFlag->SetValue(options->connectToServerFlag);
  serverToConnect->SetValue(options->serverToConnect);
  wxString port = _T("");
  port << options->portToConnect;
  portToConnect->SetValue(port);

  // check abilitations
  if (options->autostartProfileFlag)
    profileToAutostart->Enable();

  if (options->connectToServerFlag)
    {
      serverToConnect->Enable();
      portToConnect->Enable();
    }
  
  // agents to start
  int i=0;
  for (i=0; i<options->agentsToStart.GetCount(); i++)
    {
      if ( options->agentsToStart[i].CompareTo(_T("maia")) == 0)
	startMaia->SetValue(true);

      if ( options->agentsToStart[i].CompareTo(_T("maiasms")) == 0)
	startMaiasms->SetValue(true);
    }

  if (options->agentsToStart.GetCount() == 0)
    startProfile->Disable();

  //  work around a windows problem with XRC


}

/** This event is generated when all sub processes of the profile are stopped
    individually.
    This method disable stop profile button and enable start profile button
*/
void openmaiaFrame::ProfileStopped()
{
  stopProfile->Hide();
  stopProfile->Disable();
  startProfile->Enable();
  startProfile->Show();
  this->DisableMaiaconfig();
}

/** This event is generated when the profile is started from the main application.
    This notify causes the disabilitation of the start process button and the
    abilitation of stop process button
*/
void openmaiaFrame::ProfileStarted()
{
  startProfile->Hide();
  startProfile->Disable();
  stopProfile->Enable();
  stopProfile->Show();

}


/// Show help
void openmaiaFrame::OnStartServerHelp(wxCommandEvent& WXUNUSED(event))
{
  startServerHelpDialog->ShowModal();
}

/// Show help
void openmaiaFrame::OnConnectToServerHelp(wxCommandEvent& WXUNUSED(event))
{
  connectToServerHelpDialog->ShowModal();
}

/// Show help
void openmaiaFrame::OnAutostartProfileHelp(wxCommandEvent& WXUNUSED(event))
{
  autostartProfileHelpDialog->ShowModal();
}

/// Show help
void openmaiaFrame::OnOpenmaiaHelp(wxCommandEvent& WXUNUSED(event))
{
  openmaiaHelpDialog->ShowModal();
}

/// Show help
void openmaiaFrame::OnMaiaHelp(wxCommandEvent& WXUNUSED(event))
{
  maiaHelpDialog->ShowModal();
}

/// Show help
void openmaiaFrame::OnMaiasmsHelp(wxCommandEvent& WXUNUSED(event))
{
  maiasmsHelpDialog->ShowModal();
}

/// Show help
void openmaiaFrame::OnAutoiconizeHelp(wxCommandEvent& WXUNUSED(event))
{
  autoiconizeHelpDialog->ShowModal();
}


bool openmaiaFrame::CanClose()
{
  if (!saved)
    {
      // show confirm dialog
      confirmExitDialog->ShowModal();
      if ( confirmExitDialog->IsButtonPressed())
	if (confirmExitDialog->GetButtonPressed() != omOK_BUTTON)
	  return false;
    }

  return true;
 
}

void openmaiaFrame::EnableMaiaconfig()
{
  startMaiaconf->Enable();
}

void openmaiaFrame::DisableMaiaconfig()
{
  startMaiaconf->Disable();
}


/// Destroy dialogs, save modifications and exits application
void openmaiaFrame::OnCloseFrame(wxCloseEvent  &event)
{
  std::cout << "OnCloseFrame" << std::endl;
  if ( event.CanVeto() && !CanClose())
    {
      std::cout << "Veto" << std::endl;
      event.Veto();
    }
  else
    {
      std::cout << "Destroy" << std::endl;
      Destroy();
    }
  

}


bool openmaiaFrame::Destroy()
{
  // destroy dialogs
  startServerHelpDialog->Destroy();
  connectToServerHelpDialog->Destroy();
  autostartProfileHelpDialog->Destroy();
  openmaiaHelpDialog->Destroy();
  maiaHelpDialog->Destroy();
  maiasmsHelpDialog->Destroy();
  confirmExitDialog->Destroy();
  autoiconizeHelpDialog->Destroy();

  wxFrame::Destroy();
}


// void openmaiaFrame::LoadProfile(wxCommandEvent& WXUNUSED(event))
// {
//   int selpos = selectedProfile->GetSelection();
//   if (selpos == -1)
//     return;

//   wxString profile = selectedProfile->GetString(selpos);

//   wxGetApp().LoadProfile(profile);
// }


void openmaiaFrame::OnStartServerFlag(wxCommandEvent& WXUNUSED(event))
{
  saved = false;
  saveButton->Enable();
  this->SetTitle(nosavedTitle);
  options->startServerFlag = startServerFlag->GetValue();

  wxGetApp().SetSubprocessParameters();

  // todo (start and stop the server)
}


void openmaiaFrame::OnAutostartProfileFlag(wxCommandEvent& WXUNUSED(event))
{
  saved = false;
  saveButton->Enable();
  this->SetTitle(nosavedTitle);
  options->autostartProfileFlag = autostartProfileFlag->GetValue();
  options->profileToAutostart = profileToAutostart->GetStringSelection();

  if (autostartProfileFlag->IsChecked())
    profileToAutostart->Enable();
  else
    profileToAutostart->Disable();

  wxGetApp().SetSubprocessParameters();
}


void openmaiaFrame::OnAutoiconize(wxCommandEvent& WXUNUSED(event))
{
  saved = false;
  saveButton->Enable();
  this->SetTitle(nosavedTitle);
  options->autoiconize = autoiconize->GetValue();
}


void openmaiaFrame::OnConnectToServerFlag(wxCommandEvent& WXUNUSED(event))
{
  saved = false;
  saveButton->Enable();
  this->SetTitle(nosavedTitle);
  options->connectToServerFlag = connectToServerFlag->GetValue();
  options->serverToConnect = serverToConnect->GetValue();
  options->portToConnect = omFunctions::String2Int(portToConnect->GetValue());
  
  if (connectToServerFlag->IsChecked())
    {
      serverToConnect->Enable();
      portToConnect->Enable();
    }
  else
    {
      serverToConnect->Disable();
      portToConnect->Disable();
    }

  wxGetApp().SetSubprocessParameters();
}

/** Start the profile, hide this button and show stop profile button */
void openmaiaFrame::OnStartProfile(wxCommandEvent& WXUNUSED(event))
{
  startProfile->Hide();
  startProfile->Disable();
  stopProfile->Enable();
  stopProfile->Show();

  wxGetApp().StartProfile();
}

/** Stop the profile by killing all hits processes, hide this button and show
    start profile button
*/
void openmaiaFrame::OnStopProfile(wxCommandEvent& WXUNUSED(event))
{
  stopProfile->Hide();
  stopProfile->Disable();
  startProfile->Enable();
  startProfile->Show();

  wxGetApp().StopProfile();
}


void openmaiaFrame::OnSaveButton(wxCommandEvent& WXUNUSED(event))
{
  wxGetApp().SaveOptions();
  saveButton->Disable();
  saved = true;
  this->SetTitle(savedTitle);
}


void openmaiaFrame::OnStartMaia(wxCommandEvent& WXUNUSED(event))
{
  saved = false;
  saveButton->Enable();
  this->SetTitle(nosavedTitle);

  bool startMaiaValue = startMaia->GetValue();
  if (startMaiaValue)   // add to list of agents to start
    {
      int i=0;
      bool found = false;
      for (i=0; i<options->agentsToStart.GetCount(); i++)
	{
	  if ( options->agentsToStart[i].CompareTo(_T("maia")) == 0)
	    found = true;
	}
      if (!found)
	options->agentsToStart.Add(_T("maia"));
    }
  else    // remove from list of agents to start
    {
      int i=0;
      int found = -1;
      for (i=0; i<options->agentsToStart.GetCount(); i++)
	{
	  if ( options->agentsToStart[i].CompareTo(_T("maia")) == 0)
	    found = i;
	}
      if (found != -1)
	options->agentsToStart.Remove(found);
    }

  if (options->agentsToStart.GetCount() == 0)
    startProfile->Disable();
  else
    startProfile->Enable();

  wxGetApp().SetSubprocessParameters();
}
 
void openmaiaFrame::OnStartMaiasms(wxCommandEvent& WXUNUSED(event))
{
  saved = false;
  saveButton->Enable();
  this->SetTitle(nosavedTitle);

  bool startMaiasmsValue = startMaiasms->GetValue();
  if (startMaiasmsValue)   // add to list of agents to start
    {
      int i=0;
      bool found = false;
      for (i=0; i<options->agentsToStart.GetCount(); i++)
	{
	  if ( options->agentsToStart[i].CompareTo(_T("maiasms")) == 0)
	    found = true;
	}
      if (!found)
	options->agentsToStart.Add(_T("maiasms"));
    }
  else    // remove from list of agents to start
    {
      int i=0;
      int found = -1;
      for (i=0; i<options->agentsToStart.GetCount(); i++)
	{
	  if ( options->agentsToStart[i].CompareTo(_T("maiasms")) == 0)
	    found = i;
	}
      if (found != -1)
	options->agentsToStart.Remove(found);
    }

  if (options->agentsToStart.GetCount() == 0)
    startProfile->Disable();
  else
    startProfile->Enable();

  wxGetApp().SetSubprocessParameters();
}


void openmaiaFrame::OnStartMaiaconf(wxCommandEvent& WXUNUSED(event))
{
  // check if the process has been started yet
  if ( omFunctions::ArrayFind( &extraProcessesStarted, _T("maiaconfig")) != -1)
    return;

  // add extraprocess to array
  extraProcessesStarted.Add(_T("maiaconfig"));

  wxGetApp().StartConfigurator(_T("maiaconfig"));
}

// ------------------------------------------------------------- Event Table -----------------

BEGIN_EVENT_TABLE(openmaiaFrame, wxFrame)
  EVT_BUTTON(XRCID(_T("startServerHelp")), openmaiaFrame::OnStartServerHelp)
  EVT_BUTTON(XRCID(_T("startProfile")), openmaiaFrame::OnStartProfile)
  EVT_BUTTON(XRCID(_T("stopProfile")), openmaiaFrame::OnStopProfile)
  EVT_BUTTON(XRCID(_T("connectToServerHelp")), openmaiaFrame::OnConnectToServerHelp)
  EVT_BUTTON(XRCID(_T("autostartProfileHelp")), openmaiaFrame::OnAutostartProfileHelp)
  EVT_BUTTON(XRCID(_T("openmaiaHelp")), openmaiaFrame::OnOpenmaiaHelp)
  EVT_BUTTON(XRCID(_T("maiaHelp")), openmaiaFrame::OnMaiaHelp)
  EVT_BUTTON(XRCID(_T("maiasmsHelp")), openmaiaFrame::OnMaiasmsHelp)
  EVT_BUTTON(XRCID(_T("saveButton")), openmaiaFrame::OnSaveButton)
  EVT_BUTTON(XRCID(_T("startMaiaconf")), openmaiaFrame::OnStartMaiaconf)
  EVT_BUTTON(XRCID(_T("autoiconizeHelp")), openmaiaFrame::OnAutoiconizeHelp)
  EVT_CHECKBOX(XRCID(_T("startServerFlag")), openmaiaFrame::OnStartServerFlag)
  EVT_CHECKBOX(XRCID(_T("autoiconize")), openmaiaFrame::OnAutoiconize)
  EVT_CHECKBOX(XRCID(_T("autostartProfileFlag")), openmaiaFrame::OnAutostartProfileFlag)
  EVT_CHECKBOX(XRCID(_T("connectToServerFlag")), openmaiaFrame::OnConnectToServerFlag)
  EVT_TEXT(XRCID(_T("serverToConnect")),openmaiaFrame::OnConnectToServerFlag)
  EVT_TEXT(XRCID(_T("portToConnect")),openmaiaFrame::OnConnectToServerFlag)
  EVT_CHECKBOX(XRCID(_T("startMaia")), openmaiaFrame::OnStartMaia)
  EVT_CHECKBOX(XRCID(_T("startMaiasms")), openmaiaFrame::OnStartMaiasms)
  EVT_CLOSE(openmaiaFrame::OnCloseFrame)
END_EVENT_TABLE()


  //EVT_CLOSE(openmaiaFrame::OnCloseFrame)
