////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _openmaiaFrame_
#define _openmaiaFrame_

#include <wx/wx.h>
#include <wx/xrc/xmlres.h>

class openmaiaMain;
class openmaiaOptions;

#include "openmaiaOptions.h"
#include "openmaia.h"

/** @ingroup openmaia
    openmaia main frame.

    This class is the main frame of the openmaia application.
*/

class openmaiaFrame: public wxFrame
{
 public:
  // ------------------------------------------------ public attributes ----
  wxArrayString extraProcessesStarted;    ///< Store the name of processes extra that
                                          ///< were started outside the profile process.

  // ------------------------------------------------ public methods ----
  openmaiaFrame() ;
  void Init();
  void ProfileStopped();
  void ProfileStarted();
  bool Destroy();
  bool CanClose();
  void EnableMaiaconfig();
  void DisableMaiaconfig();

  DECLARE_EVENT_TABLE()


 private:
  // ------------------------------------------------ private attributes ----
  bool saved;
  openmaiaOptions *options;

  wxString savedTitle, nosavedTitle;

  
  // dialogs
  omDialog *startServerHelpDialog,  ///< Pointer to dialog object in the frame.
    *connectToServerHelpDialog,
    *autostartProfileHelpDialog,
    *openmaiaHelpDialog,
    *maiaHelpDialog,
    *maiasmsHelpDialog,
    *confirmExitDialog,
    *autoiconizeHelpDialog;

  // combo boxes
  wxComboBox *selectedProfile,      ///< Pointer to combo box object in the frame.
    *profileToAutostart;
    
  // check boxes
  wxCheckBox *startServerFlag,      ///< Pointer to check box object in the frame.
    *autostartProfileFlag,
    *connectToServerFlag,
    *startMaia,
    *startMaiasms,
    *autoiconize;

  // text controls
  wxTextCtrl *serverToConnect,      /// Pointer to text control object in the frame.
    *portToConnect;

  // buttons
  wxButton *saveButton, 
    *startProfile,
    *stopProfile,
    *startMaiaconf;

  
  // ------------------------------------------------ private methods ----
  // buttons handlers
  void OnStartServerHelp(wxCommandEvent& WXUNUSED(event));
  void OnConnectToServerHelp(wxCommandEvent& WXUNUSED(event));
  void OnAutostartProfileHelp(wxCommandEvent& WXUNUSED(event));
  void OnAutoiconizeHelp(wxCommandEvent& WXUNUSED(event));


  // check boxes handers
  void OnStartServerFlag(wxCommandEvent& WXUNUSED(event));
  void OnAutostartProfileFlag(wxCommandEvent& WXUNUSED(event));
  void OnConnectToServerFlag(wxCommandEvent& WXUNUSED(event));
  void OnAutoiconize(wxCommandEvent& WXUNUSED(event));
  void OnOpenmaiaHelp(wxCommandEvent& WXUNUSED(event));
  void OnMaiaHelp(wxCommandEvent& WXUNUSED(event));
  void OnMaiasmsHelp(wxCommandEvent& WXUNUSED(event));


  void OnStartProfile(wxCommandEvent& WXUNUSED(event));
  void OnStopProfile(wxCommandEvent& WXUNUSED(event));
  void OnSaveButton(wxCommandEvent& WXUNUSED(event));
  void OnStartMaia(wxCommandEvent& WXUNUSED(event));
  void OnStartMaiasms(wxCommandEvent& WXUNUSED(event));
  void OnStartMaiaconf(wxCommandEvent& WXUNUSED(event));

  void OnCloseFrame(wxCloseEvent &);
  
  

  
} ;




#endif  // _openmaiaFrame_
