////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include "openmaiaOptions.h"

/** The constructor initializes all the variables */
openmaiaOptions::openmaiaOptions()
{  
  // options - default page -- loading defaults
  selectedProfile = _T("default");
  lastSelectedProfile = _T("default");
  language = _T("it");
  
  // options - prefs page -- loading defaults
  startServerFlag = true;
  connectToServerFlag = false;
  autostartProfileFlag = false;
  serverToConnect = _T("") ;
  profileToAutostart = _T("default");
  autoiconize = true;


}

/** Generate a string containg a dump of the new XML configure file based
    on current value of options.
    This file store global options to be saved in 
    DATA_PATH/openmaia/openmaia.conf
*/
wxString openmaiaOptions::GenerateOptionsXMLString()
{
  // open xml file
  wxString xmlString = omXMLParser::GetXMLOptionsHeader();


  xmlString << _T("<openmaiaOptions>\n") ;

  // lastSelectedProfile
  xmlString << _T("\t<option name=\"lastSelectedProfile\" value=\"") << 
    lastSelectedProfile << _T("\"/>\n") ;

  // startServerFlag
  xmlString << _T("\t<option name=\"startServerFlag\" value=\"") << 
    omFunctions::Boolean2String(startServerFlag) << _T("\"/>\n") ;

  // autostartProfileFlag
  xmlString << _T("\t<option name=\"autostartProfileFlag\" value=\"") << 
    omFunctions::Boolean2String(autostartProfileFlag) << _T("\"/>\n") ;

  // profileToAutostart
  xmlString << _T("\t<option name=\"profileToAutostart\" value=\"") << 
    profileToAutostart << _T("\"/>\n") ;

  // lastSelectedProfile
  xmlString << _T("\t<option name=\"language\" value=\"") << 
    language << _T("\"/>\n") ;

  // autoiconize
  xmlString << _T("\t<option name=\"autoiconize\" value=\"") << 
    omFunctions::Boolean2String(autoiconize)  << _T("\"/>\n") ;



  // close xml file
  xmlString << _T("</openmaiaOptions>\n") ;


  return xmlString;
}


wxString openmaiaOptions::GenerateProfileOptionsXMLString()
{
  wxString xmlString = omXMLParser::GetXMLOptionsHeader();

  xmlString << _T("<openmaiaProfileOptions>\n") ;
  
  

  // connectToServerFlag
  xmlString << _T("\t<option name=\"connectToServerFlag\" value=\"") << 
    omFunctions::Boolean2String(connectToServerFlag) << _T("\"/>\n") ;

  // serverToConnect
  xmlString << _T("\t<option name=\"serverToConnect\" value=\"") << 
    serverToConnect << _T("\"/>\n") ;

  // portToConnect
  xmlString << _T("\t<option name=\"portToConnect\" value=\"") << 
    portToConnect << _T("\"/>\n") ;

  xmlString << _T("\n\n");

  int i;
  for (i=0; i<agentsToStart.GetCount(); i++)
    xmlString << _T("\t<option name=\"startAgent\" value=\"") << 
      agentsToStart[i] << _T("\"/>\n") ;

  xmlString << _T("</openmaiaProfileOptions>\n") ;

  return xmlString;
}



// parser methods
void openmaiaOptions::AtStartDocument()
{
  agentsToStart.Empty();
}

void openmaiaOptions::AtEndDocument()
{

}


void openmaiaOptions::AtStartTag( wxString n, omXMLParameters p)
{
  omXMLParameters *pars = &p ;
  wxString name = pars->GetValue(wxString(_T("name"))) ;
  wxString value =  pars->GetValue(wxString(_T("value"))) ;

  wxString tname = omFunctions::ConvertToLower(n) ;

  if ( tname == wxString(_T("option"))) 
    {

      // -------------------------------- global options ---------------------

      // lastSelectedProfile
      if ( name.CmpNoCase( _T("lastSelectedProfile")) == 0 ) 
	lastSelectedProfile = value;

      // autostartProfileFlag
      if ( name.CmpNoCase( _T("autostartProfileFlag")) == 0 ) 
	{
	  if (omFunctions::IsBoolean(value)) 
	      autostartProfileFlag = omFunctions::String2Boolean(value);
	  else
	    {
#ifdef MAIA_DEBUG 
	      std::cout << "openmaiaOptions::AtStartTag; autostartProfileFlag, value " << value << "not correct" << std::endl;
#endif
	    }

	}
      
      // profileToAutostart
      if ( name.CmpNoCase( _T("profileToAutostart")) == 0 ) 
	profileToAutostart = value;

      // startServerFlag 
      if ( name.CmpNoCase( _T("startServerFlag")) == 0 ) 
	{
	  if (omFunctions::IsBoolean(value)) 
	      startServerFlag = omFunctions::String2Boolean(value);
	  else
	    {
#ifdef MAIA_DEBUG 
	      std::cout << "openmaiaOptions::AtStartTag; startServerFlag, value " << value << "not correct" << std::endl;
#endif
	    }

	}

      // language
      if ( name.CmpNoCase( _T("language")) == 0 ) 
	language = value;

      

      // autoiconize 
      if ( name.CmpNoCase( _T("autoiconize")) == 0 ) 
	{
	  if (omFunctions::IsBoolean(value)) 
	      autoiconize = omFunctions::String2Boolean(value);
	  else
	    {
#ifdef MAIA_DEBUG 
	      std::cout << "openmaiaOptions::AtStartTag; autoiconize, value " << value << "not correct" << std::endl;
#endif
	    }

	}

      // --------------------------------- profile options --------------------------

      // connectToServerFlag
      if ( name.CmpNoCase( _T("connectToServerFlag")) == 0 ) 
	{
	  if (omFunctions::IsBoolean(value)) 
	      connectToServerFlag = omFunctions::String2Boolean(value);
	  else
	    {
#ifdef MAIA_DEBUG 
	      std::cout << "openmaiaOptions::AtStartTag; connectToServerFlag, value " << value << "not correct" << std::endl;
#endif
	    }

	}

      // serverToConnect
      if ( name.CmpNoCase( _T("serverToConnect")) == 0 ) 
	serverToConnect = value;


      // portToConnect 
      if ( name.CmpNoCase( _T("portToConnect")) == 0 ) 
	{
	  if (omFunctions::IsInt(value)) 
	      portToConnect = omFunctions::String2Int(value);
	  else
	    {
#ifdef MAIA_DEBUG 
	      std::cout << "openmaiaOptions::AtStartTag; portToConnect, value " << value << "not correct" << std::endl;
#endif
	    }

	}

      // start agent
      if ( name.CmpNoCase( _T("startAgent")) == 0 ) 
	agentsToStart.Add(value);

    }   // end option tag

}
 

void openmaiaOptions::AtEndTag( wxString n)
{

}


void openmaiaOptions::LoadOptions( wxString fname)
{
  this->Parse(fname) ;
}
