////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
///////////////////////////////////////////////////////////////////////////////

#include "openmaiaProcess.h"

/** Create and openmaia process associated with an handler.
    This class listen to process events and call handler NotifyProcessStop method
    when the process stops.

    @param proc_handler Pointer to class processes handler
*/
openmaiaProcess::openmaiaProcess(openmaiaProcessesHandler *proc_handler)
  : wxProcess() 
{
  toNotify = proc_handler;
  //  Redirect();

  // work around a windows strange behaviour
#ifndef WINDOWS_SYSTEM
  Redirect();
#endif

}

/** Flush the stream of this process to standard out or standard err 
*/
void openmaiaProcess::FlushStream()
{
  while ( IsInputAvailable() )
    {
      wxTextInputStream tis(*GetInputStream());

      // this assumes that the output is always line buffered
        
      std::cout << tis.ReadLine() << std::endl;
    }

  while( IsErrorAvailable() )
    {
      wxTextInputStream tis(*GetErrorStream());

      // this assumes that the output is always line buffered
      std::cerr << tis.ReadLine() << std::endl;
    }
}

/** Handle terminate process event.
    Lounch the appropriate signat to toNotify object.
*/
void openmaiaProcess::OnTerminate(int pid, int status) 
{
  toNotify->NotifyProcessStop(pid);
}

