////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
///////////////////////////////////////////////////////////////////////////////

#include "openmaiaProcessesHandler.h"

/** Build a void structure to store processes data */
openmaiaProcessesHandler::openmaiaProcessesHandler()
{
}

/** Start a specific subprocess.
    @param name name of the process to start.
    @param options options of the process.
    @param path path of the binary file (without final slash)
*/
void openmaiaProcessesHandler::StartProcess(wxString name, wxString options, wxString path)
{
  wxString programToStart;

  // create openmaiaProcess object
  openmaiaProcess *newProcess = new openmaiaProcess(this);

  // ----- find program name to start ----
#ifdef LINUX_SYSTEM
  programToStart = path + _T("/") + name  ;
  if (!omFunctions::TestFile(programToStart))
    programToStart = name;
#endif

#ifdef WINDOWS_SYSTEM
  wxString binaryPath = omFunctions::GetInstallationPath(wxGetApp().binPath) ;
  // check if in dev tree
  //programToStart = binaryPath + _T("/../") + name + _T("/") + name + _T(".exe") ;
  programToStart = path + _T("/") + name + _T(".exe") ;
  programToStart = omFunctions::GetWindowsString(programToStart);
#endif

  if (!options.IsEmpty())
    programToStart = programToStart + _T(" ") + options;

#ifdef MAIA_DEBUG
  std::cout << "openmaiaProcessesHandler::StartProcess- Starting process " << programToStart << std::endl;
#endif
  
  int pid = ::wxExecute(programToStart, wxEXEC_ASYNC, newProcess);
  //int pid = ::wxExecute(programToStart, wxEXEC_ASYNC, new wxProcess());

  // update hashmaps
  process[name] = newProcess;
  processName[pid] = name ;
  processPID[name] = pid;
}

/** Stop a specific subprocess.
    @param name name of the process to stop.
*/
void openmaiaProcessesHandler::StopProcess(wxString name)
{
  int pid = processPID[name];

//   // remove process information from hash tables
//   processPID.erase(name);
//   processName.erase(pid);
//   process.erase(name);
  
  // kill the process
  ::wxKill(pid);
}

/** Stop all subprocesses */
void openmaiaProcessesHandler::StopAllProcesses()
{
  // copy processPID hash table bacause when a process is stopped this hash table
  // will be modified.

  Name2PIDHashmap processPID2;
  processPID2 = processPID;

  Name2PIDHashmap::iterator i;
  int pid;

  for (i = processPID2.begin(); i != processPID2.end(); i++)
    {
      pid = i->second;    // get pid from iterator
      ::wxKill(pid);
    }
}

/** This method is used by openmaiaProcess class to notify that its process
    has stopped.
*/
void openmaiaProcessesHandler::NotifyProcessStop(int id)
{
  // get process name from pid
  wxString name = processName[id];

  // remove process information from hash tables
  processPID.erase(name);
  processName.erase(id);
  process.erase(name);

  // notify event to application
  wxGetApp().NotifyProcessStop(name);
}


/** Flush all streams of subprocesses to start out and standard err 

    @return true is almost a process has more input
*/
void openmaiaProcessesHandler::FlushStreams()
{
  openmaiaProcess *curProcess;
  ProcessHash::iterator it;
  for (it=process.begin(); it!=process.end(); it++)
    {
      curProcess = it->second;
      curProcess->FlushStream();
    }

}

/** Return the number of the processes running */
int openmaiaProcessesHandler::GetCount()
{
  return processName.size();
}
