////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
///////////////////////////////////////////////////////////////////////////////


#ifndef _openmaiaProcessesHandler_
#define _openmaiaProcessesHandler_

#include <wx/wx.h>
#include <wx/process.h>

// class prototipes
class openmaiaProcess;

#include "openmaia.h"
#include "openmaiaProcess.h"

/** @ingroup openmaia

    This class is used to handle openmaia agents processes.
    The class starts processes using wxExecute function and store its informations and process data.
    When a process exits the class notify its exit to application controller class.
*/


class openmaiaProcessesHandler
{
 private:

  // declare hashmap to associate process name with process class
  WX_DECLARE_STRING_HASH_MAP( openmaiaProcess *, ProcessHash);

  // declare hashmap to associate pid with process name
  WX_DECLARE_HASH_MAP( int,
                         wxString,
                         wxIntegerHash,
                         wxIntegerEqual,
                         PID2NameHashMap );

  // declare hashmap to associate process name with its pid
  WX_DECLARE_STRING_HASH_MAP( int, Name2PIDHashmap);

  

  ProcessHash process;
  PID2NameHashMap processName;
  Name2PIDHashmap processPID;

 public:
  openmaiaProcessesHandler() ;

  void StartProcess(wxString, wxString, wxString);
  void StopProcess(wxString);
  void StopAllProcesses();
  void NotifyProcessStop(int) ;
  int GetCount();

  void FlushStreams();

} ;



#endif  // _openmaiaProcessesHandler_
