////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#include <iostream>

#include "openmaiaServer.h"


enum
{
SERVER_ID=1000,
SOCKET_ID
} ;


BEGIN_EVENT_TABLE(openmaiaServer, wxEvtHandler)
  EVT_SOCKET(SERVER_ID, openmaiaServer::OnServerEvent)
  EVT_SOCKET(SOCKET_ID, openmaiaServer::OnSocketEvent)
END_EVENT_TABLE()

// main function of the wx application
openmaiaServer::openmaiaServer() 
{
  // initialize variables
  nagents = 0 ;

  // start the server socket
  wxIPV4address addr ;
  addr.Service(MAIALISTENPORT) ;
  server = new wxSocketServer(addr) ;
  bool connectionOk = server->Ok() ;

  while (!connectionOk)
    {
      std::cout << "." ;
      delete server ;
      server = new wxSocketServer(addr) ;
      connectionOk = server->Ok() ;
    }

  server->SetEventHandler(*this, SERVER_ID);
  server->SetNotify(wxSOCKET_CONNECTION_FLAG);
  server->Notify(TRUE);
}


/** This method is called when a client attempt to connect to the server.
    this method stores client information and activate event handlig on socket associated
    with this connection
*/
void openmaiaServer::OnServerEvent(wxSocketEvent &event)
{
  
  // retrive client informations
  wxSocketBase *sock ;
  wxIPV4address *ad = new wxIPV4address() ;

  // accept the connection
  sock = server->Accept(TRUE) ;

  // get host informations
  sock->GetPeer(*ad) ;

  // add client information to agents hash map
  // the key of the map is 'hostnameport'
  wxString hostname= ad->Hostname() ;
  int port=ad->Service();

  // build the key of the hash map
  wxString key ;
  key << hostname << port ;

#ifdef MAIA_DEBUG
  std::cout << "openmaia main: Accepted connection from " << hostname << ":" << port <<std::endl;
#endif

  // store socket information
  agentsMap[key] = new openmaiaConnectedAgent(this, sock) ;
  agentKeys.insert(agentKeys.end(), key) ;
  nagents++ ;

  // assign socket handler
  sock->SetEventHandler(*this, SOCKET_ID);
  sock->SetNotify(wxSOCKET_INPUT_FLAG);
  sock->Notify(TRUE);

}

// this method is called when an established connection
// communicates with the server.
// Every message is sent char by char
void openmaiaServer::OnSocketEvent(wxSocketEvent &event)
{
  // retrive connection infromations
  wxIPV4address *ad = new wxIPV4address() ;
  
  wxSocketBase *sock = event.GetSocket();
  sock->GetPeer(*ad) ;

  wxString hostname = ad->Hostname();
  unsigned int port = ad->Service() ;

  // build the map key
  wxString key ;
  key << hostname << port ;

  // retrvie agent 
  openmaiaConnectedAgent *curAgent = agentsMap[key];
  
  // retrive communicated char
  char c ;
  sock->Read(&c,1) ;

  curAgent->AnalizeChar(c);
}


void openmaiaServer::AgentMessage(wxString host, int port, omTCPMessage mess)
{
//   std::cout << "Message from " << host << ":" << port << std::endl <<
//     mess.GetCommand() << std::endl <<
//     mess.GetParameters() << std::endl << std::endl ;


// modificato da qui sotto -------
//   wxString curKey = host;
//   curKey << port ;

//   int i;
//   for(i=0; i<nagents;i++)
//     {
//       wxString key = agentKeys[i];
//       if ( key != curKey)
// 	agentsMap[key]->SendMessage(mess);
//     }

  wxString curKey = host;
  curKey << port ;

  int i;
  for(i=0; i<nagents;i++)
    {
      wxString key = agentKeys[i];
      if ( key.CmpNoCase(curKey) != 0)
	agentsMap[key]->SendMessage(mess);
    }


}




